shell.clear()

### --------------------------------------------------
### Welcome. This is the start of the Python tutorial.
### --------------------------------------------------

the_world_is_flat = 1
if the_world_is_flat:
    print "Be careful not to fall off!"


###
### 3. An Informal Introduction to Python 
###

# Many of the examples in this manual, even those entered at the interactive 
# prompt, include comments. Some examples: 

# this is the first comment
SPAM = 1                 # and this is the second comment
                         # ... and now a third!
STRING = "# This is not a comment."

shell.pause()

###
### 3.1.1 Numbers
###

# The interpreter acts as a simple calculator
2+2
# This is a comment
2+2
2+2  # and a comment on the same line as code
(50-5*6)/4
# Integer division returns the floor:
7/3
7/-3
# Like in C, the equal sign ("=") is used to assign a value to a variable.
# The value of an assignment is not written:
width = 20
height = 5*9
width * height
# A value can be assigned to several variables simultaneously:
x = y = z = 0  # Zero x, y and z
x
y
z

shell.pause()

# There is full support for floating point; operators with mixed type operands 
# convert the integer operand to floating point: 
4 * 2.5 / 3.3
7.0 / 2
# Complex numbers are also supported; imaginary numbers are written with a suffix
# of "j" or "J". Complex numbers with a nonzero real component are written as 
# "(real+imagj)", or can be created with the "complex(real, imag)" function.
1j * 1J
1j * complex(0,1)
3+1j*3
(3+1j)*3
(1+2j)/(1+1j)
# Complex numbers are always represented as two floating point numbers, 
# the real and imaginary part. To extract these parts from a complex number z, 
# use z.real and z.imag.
a=1.5+0.5j
a.real
a.imag
# The conversion functions to floating point and integer 
# (float(), int() and long()) don't work for complex numbers.
a=1.5+0.5j
float(a)
a.real
abs(a)
# In interactive mode, the last printed expression is assigned to the variable _.
tax = 17.5 / 100
price = 3.50
price * tax
price + _
round(_, 2)

shell.pause()

###
### 3.1.2 Strings
###

# Besides numbers, Python can also manipulate strings.
'spam eggs'
'doesn\'t'
"doesn't"
'"Yes," he said.'
"\"Yes,\" he said."
'"Isn\'t," she said.'
# String literals can span multiple lines in several ways. 
# Newlines can be escaped with backslashes, e.g.: 
hello = "This is a rather long string containing\n\
several lines of text just as you would do in C.\n\
    Note that whitespace at the beginning of the line is\
 significant.\n"
print hello
# Or, strings can be surrounded in a pair of matching triple-quotes: """ or '''.
# End of lines do not need to be escaped when using triple-quotes, but they will
# be included in the string.
print """
Usage: thingy [OPTIONS]
     -h                        Display this usage message
     -H hostname               Hostname to connect to
"""
# Strings can be concatenated (glued together) with the + operator, 
# and repeated with *:
word = 'Help' + 'A'
word
'<' + word*5 + '>'
# Two string literals next to each other are automatically concatenated.
import string
'str' 'ing'                   #  <-  This is ok
string.strip('str') + 'ing'   #  <-  This is ok
string.strip('str') 'ing'     #  <-  This is invalid
# Strings can be subscripted (indexed); like in C, the first character of a 
# string has subscript (index) 0. There is no separate character type; 
# a character is simply a string of size one. Like in Icon, substrings can be 
# specified with the slice notation: two indices separated by a colon.
word[4]
word[0:2]
word[2:4]
# Unlike a C string, Python strings cannot be changed.
word[0] = 'x'
word[:1] = 'Splat'
# However, creating a new string with the combined content is easy and efficient:
'x' + word[1:]
'Splat' + word[4]

shell.pause()

# Slice indices have useful defaults; an omitted first index defaults to zero, 
# an omitted second index defaults to the size of the string being sliced.
word[:2]    # The first two characters
word[2:]    # All but the first two characters
# Here's a useful invariant of slice operations: s[:i] + s[i:] equals s.
word[:2] + word[2:]
word[:3] + word[3:]
# Degenerate slice indices are handled gracefully: 
#    an index that is too large is replaced by the string size,
#    an upper bound smaller than the lower bound returns an empty string.
word[1:100]
word[10:]
word[2:1]
# Indices may be negative numbers, to start counting from the right. For example:
word[-1]     # The last character
word[-2]     # The last-but-one character
word[-2:]    # The last two characters
word[:-2]    # All but the last two characters
# But note that -0 is really the same as 0, so it does not count from the right!
word[-0]     # (since -0 equals 0)
# Out-of-range negative slice indices are truncated, but don't try this for 
# single-element (non-slice) indices:
word[-100:]
word[-10]    # error
# The built-in function len() returns the length of a string:
s = 'supercalifragilisticexpialidocious'
len(s)

### --------------------------------------------------
### This is the end of the Python tutorial. Goodbye.
### --------------------------------------------------
