#!/usr/bin/env python
"""Demonstrates how to set things up to be able to use a different editor
and/or interpreter.
"""

__author__ = "Patrick K. O'Brien <pobrien@orbtech.com>"
__cvsid__ = "$Id: PyCrustAlaCarte.py,v 1.5 2001/08/15 12:51:11 pobrien Exp $"
__date__ = "August 2, 2001"
__version__ = "$Revision: 1.5 $"[11:-2]

from wxPython.wx import *

from PyCrust.version import VERSION
from PyCrust.shell import Shell
from PyCrust.editor import Editor
from PyCrust.interpreter import Interpreter

from PyCrust.pseudo import PseudoFileIn, PseudoFileOut, PseudoFileErr


class Frame(wxFrame):
    def __init__(self, parent, id, title):
        wxFrame.__init__(self, parent, id, title)
        intro = 'Welcome To PyCrust %s - The Flakiest Python Shell\n' % VERSION + \
                '-'*60 + '\n' + \
                'This slice of PyCrust is served "a la carte"\n' + \
                'Editor: Default (based on wxStyledTextCtrl)\n' + \
                'Interpreter: Default (based on code.InteractiveInterpreter)\n' + \
                '-'*60
        # Create an editor object.
        self.editor = Editor(parent=self, id=-1)
        # Create pseudo file objects.
        self.stdin = PseudoFileIn(self.editor.readIn)
        self.stdout = PseudoFileOut(self.editor.writeOut)
        self.stderr = PseudoFileErr(self.editor.writeErr)
        # Override the default locals so we have something interesting.
        locals = {'__name__': 'PyCrust a la Carte', 
                  '__doc__': 'PyCrust, the Python shell, served a la carte.'}
        # Create an interpreter object.
        self.interp = Interpreter(locals=locals, 
                                  rawin=self.editor.readRaw,
                                  stdin=self.stdin, 
                                  stdout=self.stdout, 
                                  stderr=self.stderr)
        # Create the shell object passing it the editor and interpreter.
        self.shell = Shell(introText = intro, editor=self.editor, interp=self.interp)
        # Override so the auto complete list comes from the interpreter.
        # Remove this if your editor doesn't support auto completion.
        self.editor.getAutoCompleteList = self.interp.getAutoCompleteList
        # Override so the call tip comes from the interpreter.
        # Remove this if your editor doesn't support call tips.
        self.editor.getCallTip = self.interp.getCallTip



class App(wxApp):
    def OnInit(self):
        self.frame = Frame(parent=None, id=-1, title='PyCrust a la Carte')
        self.frame.Show(true)
        self.SetTopWindow(self.frame)
        return true


def main():
    application = App(0)
    application.MainLoop()

if __name__ == '__main__':
    main()
