"""
__version__ = "$Revision: 1.2 $"
__date__ = "$Date: 2001/08/06 20:38:30 $"
"""

import imp, sys, string
from types import FunctionType, ClassType
from event import *
from config import *
from log import *
from error import *
from res import *
from pom import * 
import event

class EventBinding :
    """
    Dynamically constructs a GUI toolkit-specific EventBindingAdapter
    based on the type of a specified Component.

    We're using a naming convention to allow us to dynamically
    load the module that contains the GUI-specific EventBindingAdapter
    classes.

    The module's file name is:

        <gui-name>_binding.py

    Each adapter is:

        <gui-name>_<component-class-name>EventBinding

    Currently we're using:

        wxPython_binding.py

    ...which contains:

        wxPython_ButtonEventBinding
        ...

    The adapter is responsible for binding the Component's supported
    Events to the underlying GUI toolkit, and generating PythonCard
    Event objects when the GUI toolkit generates a toolkit-specific
    event.
    """
    def __init__( self, aComponent ) :
        assert issubclass( aComponent.__class__, Component ), 'parameter must extend pom.Component'

        guiToolkit = Configuration().getGuiToolkitName() 
        exec( 'import ' + guiToolkit + '_binding' )
        className = guiToolkit + '_' + aComponent.__class__.__name__
        moduleName = guiToolkit + '_binding'
        adapterName = moduleName + '.' + className + 'EventBinding'
        try :
            self._adapter = eval( adapterName )( aComponent )
            self._adapter.bindEvents()
        except :
            Log.getInstance().warning( 'cannot find adapter: ' + adapterName )

class EventBindingAdapter :    
    """
    Defines an interface that must be implemented by GUI-specific
    classes that bind GUI events to PythonCard Events.
    """
    def __init__( self, aComponent ) :
        assert issubclass( aComponent.__class__, Component ), 'parameter must extend pom.Component'
        self._component = aComponent

    def bindEvents( self ) :
        raise AbstractMethodException




        # Unit Test
        """
        if __name__ == '__main__' :

            class AlarmEvent( Event ) :

                def __init__( self, aSource ) :
                    Event.__init__( self, aSource )

            class Alarm( Component ) :

                _events = [ AlarmEvent ]

                def __init__( self, seconds ) :
                    self._seconds = seconds
                    EventBinding( self )

            class WxAlarmEventBinding( WxEventBinding ) :

                def __init__( self, aComponent ) :
                    WxEventBinding.__init__( self, aComponent )

                def bindEvent( self, aEventClass ) :
                    print 'WxAlarmEventBinding: ', aEventClass

        alarm = Alarm( 3 )
        """



