
"""
__version__ = "$Revision: 1.20 $"
__date__ = "$Date: 2001/08/24 00:29:38 $"
"""

from wxPython.wx import *
from event import *
import dialog
import __version__
import uniqueid
import pprint

# KEA pyCrust support
# KEA 2001-08-05
try:
    import PyCrust
    from PyCrust.shell import Shell
    PYCRUST_FOUND = 1
except ImportError:
    PYCRUST_FOUND = 0

PYTHONCARD_DEBUG_MENU_RESOURCE_FILE = 'debugmenu.rsrc.py'

class MessageWatcher( wxMiniFrame, EventListener) :
    def __init__(self, parent, ID, title, pos, size, parentApp ):
        wxMiniFrame.__init__(self, parent, ID, title, pos, size, wxDEFAULT_FRAME_STYLE | wxTINY_CAPTION_HORIZ)
        panel = wxPanel(self, -1)

        self.parentApp = parentApp
        # KEA 2001-07-27
        # no longer needed
        # self._eventMap = EventMap()

        EVT_SIZE(self, self.onSize)
        EVT_CLOSE(self, self.onCloseMe)

        self.hideIdle = wxCheckBox(panel, -1, 'Hide idle', wxPoint(1, 1), wxDefaultSize, wxNO_BORDER)
        self.hideIdle.SetValue(true)

        # KEA 2001-07-27
        # uncommented
        # we want to be able to hide unused messages now
        """"""
        # unused messages aren't bound right now, so there is nothing to hide
        self.hideUnused = wxCheckBox(panel, -1, 'Hide unused', wxPoint(80, 1), wxDefaultSize, wxNO_BORDER)
        self.hideUnused.SetValue(true)
        """"""

        # this field should use a sizer instead
        # and the window should have a minimum height
        # and a fixed width
        # the size attribute doesn't matter here because an onSize event
        # will be fired after init
        self.msgHistory = wxTextCtrl(panel, -1, '', wxPoint(0, 30), wxDefaultSize, style=wxTE_MULTILINE | wxTE_READONLY)
        
        EventQueue().addListener( self )

    def eventOccurred(self, aDispatchEvent):

        event = aDispatchEvent.getDispatchedEvent()

        # KEA 2001-07-27
        # simplified to getName()
        #eventName = self._eventMap.classToName(event.__class__)
        eventName = event.getName()
        sourceName = event.getSource().name
        
        #eventClass = event.__class__.__name__
        #self.msgHistory.AppendText( eventClass + '( '  + sourceName + ' )\n' )

        """
        # from the old messageWatcher.py
        # in case idle needs special handling
        # delete once idle is implemented if this is pointless
        if id == wxEVT_IDLE:
            if not self.hideIdle.GetValue():
                self.msgHistory.AppendText(eventText)
        else:
            if used or (not self.hideUnused.GetValue()):
                self.msgHistory.AppendText(eventText)
        """

        used = aDispatchEvent.getEventWasUsed()

        if used:
            eventText = eventName + ' : '  + sourceName + '\n'
        else:
            eventText = '(' + eventName + ' : '  + sourceName + ')\n'

        if used or (not self.hideUnused.GetValue()):
            self.msgHistory.AppendText(eventText)

    def onSize(self, event):
        size = self.GetClientSize()
        self.msgHistory.SetSize(wxSize(size.width, size.height - 30))
        # tell the event system to continue looking for an event handler,
        # so the default handler will get called.
        event.Skip()

    def onCloseMe(self, event):
        # KEA if the menu item exists, then uncheck it
        #id = self.parentApp.frame.GetMenuBar().FindMenuItem("Script", "Message Watcher")
        #self.parentApp.frame.GetMenuBar().Check(id, false)
        
        # Really close the window, otherwise the app hangs -rds
        #self.Destroy()
        # KEA fixed showMessageWatcher in model.py
        # however, the fix probably needs more work
        self.Show(false)


# KEA this is a load of dingos' kidneys and needs to be rewritten
class PropertyEditor(wxMiniFrame, ChangeListener) :
    def __init__(self, parent, ID, title, pos, size, parentApp):
        wxMiniFrame.__init__(self, parent, ID, title, pos, size, wxDEFAULT_FRAME_STYLE | wxTINY_CAPTION_HORIZ)
        panel = wxPanel(self, -1)

        self.parentApp = parentApp
        bg = self.parentApp.getCurrentBackground()
        bg.components.addChangeEventListener(self)

        EVT_SIZE(self, self.onSize)
        EVT_CLOSE(self, self.onCloseMe)

        # removed 'helpText'
        self.propList = ['name', 'command', 'backgroundColor', 'foregroundColor', 'enabled',
                         'font', 'position', 'size', 'toolTip', 'visible']

        self.optionalProps = {'StaticLine': ['layout'],
                              'Button': ['label'],
                              'StaticText': ['text', 'alignment'],
                              'TextField': ['editable', 'text'],
                              'PasswordField': ['editable', 'text'],
                              'TextArea': ['editable', 'text'],
                              'Choice': ['items', 'selected'],
                              'List': ['items', 'selected'],
                              'CheckBox': ['label', 'checked'],
                              'Image': ['file'],
                              'ImageButton': ['file', 'border'],
                              'RadioGroup': ['label', 'items', 'selected', 'layout', 'max'],
                              'Slider': ['min', 'max', 'value', 'layout']}
        #self.propList.sort()

        wxStaticText(panel, -1, 'Name  :  Class', wxPoint(3, 3))
        wxStaticText(panel, -1, 'Properties', wxPoint(228, 3))
        self.wComponentList = wxListBox(panel, -1, wxPoint(0, 20), wxSize(200, 100))
        self.wPropertyList = wxListBox(panel, -1, wxPoint(225, 20), wxSize(125, 100))
        self.wPropertyList.InsertItems(self.propList, 0)
        self.wName = wxStaticText(panel, -1, 'name:', wxPoint(3, 135), wxSize(90, -1), style = wxALIGN_RIGHT | wxST_NO_AUTORESIZE)
        self.wField = wxTextCtrl(panel, -1, '', wxPoint(100, 130), wxSize(180, -1))
        self.wColor = wxButton(panel, -1, 'Color', wxPoint(290, 130), wxSize(50, -1))
        self.wColor.Show(0)
        self.wFont = wxButton(panel, -1, 'Font', wxPoint(290, 130), wxSize(50, -1))
        self.wFont.Show(0)
        self.wTextArea = wxTextCtrl(panel, -1, '', wxPoint(100, 130), wxSize(250, 50), style=wxTE_MULTILINE)
        self.wTextArea.Show(0)
        self.wChecked = wxCheckBox(panel, -1, '', wxPoint(100, 132), wxDefaultSize, wxNO_BORDER)
        self.wChecked.Show(0)
        self.wPop = wxChoice(panel, -1, wxPoint(100, 130), wxDefaultSize, [])
        self.wPop.Show(0)

        self.wCopy = wxButton(panel, -1, 'Copy resource to clipboard', wxPoint(5, 185), wxSize(150, -1))
        self.wCopy.Show(0)
        
        self.wUpdate = wxButton(panel, -1, 'Update', wxPoint(265, 185))
        self.wUpdate.Show(0)

        self.editItems = [self.wField, self.wColor, self.wFont, self.wTextArea, self.wChecked, self.wPop]        

        #self.wSep = wxButton(panel, -1, '', wxPoint(5, 5), wxSize(150, 3))
        #self.wSep.Enable(0)

        # KEA 2001-08-14
        # this was causing an assertion error with the hybrid wxPython
        #self.wComponentList.SetSelection(0)
        if self.wComponentList.GetStringSelection() == "":
            wClass = ""
        else:
            wName, wClass = self.wComponentList.GetStringSelection().split("  :  ")
        self.setValidProps(wClass)

        EVT_LISTBOX(self, self.wComponentList.GetId(), self.onSelectComponentListEvent)
        EVT_LISTBOX(self, self.wPropertyList.GetId(), self.onSelectPropertyListEvent)
        EVT_BUTTON(self, self.wColor.GetId(), self.onSelectColor)
        EVT_BUTTON(self, self.wFont.GetId(), self.onSelectFont)
        EVT_BUTTON(self, self.wCopy.GetId(), self.onSelectCopy)
        EVT_BUTTON(self, self.wUpdate.GetId(), self.onSelectUpdate)

        #EventQueue().addListener( self )

    def addWidgetToComponentsList(self, widget):
        wName = widget.name
        wClass = str(widget.__class__).split('.')
        self.wComponentList.Append(wName + "  :  " + wClass[len(wClass) - 1])

    def deleteWidgetFromComponentsList(self, wName, wClass):
        i = self.wComponentList.GetSelection()
        j = self.wComponentList.FindString(wName + "  :  " + wClass)
        if i == -1 or i != j:
            self.wComponentList.Delete(j)
        else:            
            self.wComponentList.SetSelection(j - 1)
            self.wComponentList.Delete(j)
            if self.wComponentList.GetSelection() == -1:
                self.setValidProps("")
                self.hideAllBut(self.wField)
                self.wUpdate.Show(0)
                self.wCopy.Show(0)
            else:
                wName, wClass = self.wComponentList.GetStringSelection().split("  :  ")
                # deselect the name from properties list
                self.setValidProps(wClass)
                propName = self.wPropertyList.GetStringSelection()
                if propName == "":
                    propName = "name"
                    self.wPropertyList.SetStringSelection("name")
                self.displayProperty(wName, wClass, propName)

    def selectComponentsList(self, wName, wClass):
        self.wComponentList.SetStringSelection(wName + "  :  " + wClass)
        self.setValidProps(wClass)
        propName = self.wPropertyList.GetStringSelection()
        #print propName
        if propName == "":
            propName = "name"
            self.wPropertyList.SetStringSelection("name")
        self.displayProperty(wName, wClass, propName)

    def changed(self, event):
        comp = self.parentApp.getCurrentBackground().components
        wName, wClass = event.getOldValue().split(",")
        if comp.has_key(wName):
            # new item added
            self.addWidgetToComponentsList(comp[wName])
        else:
            # item deleted
            self.deleteWidgetFromComponentsList(wName, wClass)

    def onSelectCopy(self, event):
        wName, wClass = self.wComponentList.GetStringSelection().split("  :  ")
        # what needs to happen here is to have a method for the Widget class that
        # will provide a valid resource description, each subclass of widget would
        # override the method to deal with their specific resource attributes
        # the Widget class should provide some ordering so that 'type',
        # 'position', 'size' comes before less commonly used items, the actual
        # ordering could just be defined in a list, so it is easy to change
        # also, if the current values match the defaults for a widget attribute
        # then that attribute should not be provided as part of the output
        print "this is just a placeholder method right now,"
        print "the resource is not actually copied to the clipboard yet"
        pprint.pprint(self.parentApp._iterator.getCurrentBackground().components[wName])

    def onSelectUpdate(self, event):
        # make these attributes of self, since they are duplicated below in displayProperty
        checkItems = ['enabled', 'visible', 'editable', 'checked']
        popItems = ['layout', 'border', 'style', 'alignment']
        cantmodify = ['name', 'alignment', 'layout', 'style', 'border', 'font' ]

        wName, wClass = self.wComponentList.GetStringSelection().split("  :  ")
        propName = self.wPropertyList.GetStringSelection()

        if propName not in cantmodify:
            #print "updating", wName
            if propName in checkItems:
                value = self.wChecked.GetValue()
            elif propName in popItems:
                value = self.wPop.GetStringSelection()
            elif wClass == 'TextArea' and propName == 'text':
                value = self.wTextArea.GetValue()
            else:
                value = self.wField.GetValue()

            try:
                value = eval(value)
            except:
                pass
            widget = self.parentApp._iterator.getCurrentBackground().components[wName]
            setattr(widget, propName, value)

    def onSelectColor(self, event):
        dlg = dialog.ColorDialog(self)
        if dlg.accepted():
            self.wField.SetValue(str(dlg.getColor()))

    def onSelectFont(self, event):
        dlg = dialog.FontDialog(self)
        if dlg.accepted():
            color = dlg.getColor()
            font = dlg.getFont()
            self.wField.SetValue("%s;%s" % (font, color))

    def setValidProps(self, wClass):
        #print "setValidProps", wClass
        oldProp = self.wPropertyList.GetStringSelection()
        if wClass == "":
            self.wPropertyList.Clear()
        else:
            props = self.propList + self.optionalProps[wClass]
            props.sort()
            self.wPropertyList.Clear()
            self.wPropertyList.InsertItems(props, 0)
            self.wPropertyList.SetStringSelection(oldProp)

    def hideAllBut(self, widget):
        for w in self.editItems:
            if widget.GetId() != w.GetId():
                w.Show(0)

    def displayProperty(self, wName, wClass, propName):
        checkItems = ['enabled', 'visible', 'editable', 'checked']
        popItems = ['layout', 'border', 'style', 'alignment']
        cantmodify = ['name', 'alignment', 'layout', 'style', 'border', 'font' ]

        self.wName.SetLabel(propName + ":")
        widget = self.parentApp._iterator.getCurrentBackground().components[wName]
        value = str(getattr(widget, propName))
        
        if propName in checkItems:
            self.hideAllBut(self.wChecked)
            self.wChecked.Show(1)
            self.wChecked.SetValue(int(value))
        elif propName in popItems:
            self.hideAllBut(self.wPop)
            self.wPop.Show(1)
            self.wPop.Clear()
            if wClass == 'RadioGroup' or wClass == 'StaticLine' or wClass == 'StaticLine':
                self.wPop.Append('vertical')
                self.wPop.Append('horizontal')
            else:
                # ImageButton
                self.wPop.Append('transparent')
                self.wPop.Append('3d')
                self.wPop.Append('None')
            self.wPop.SetStringSelection(value)
        elif wClass == 'TextArea' and propName == 'text':
            #print 'displaying TextArea'
            self.hideAllBut(self.wTextArea)
            self.wTextArea.Show(1)
            self.wTextArea.SetValue(value)
        else:
            self.hideAllBut(self.wField)
            self.wField.Show(1)
            if propName == 'foregroundColor' or propName == 'backgroundColor':
                self.wColor.Show(1)
            elif propName == 'font':
                self.wFont.Show(1)
            self.wName.SetLabel(propName + ":")
            self.wField.SetValue(value)
        # this should only display if the attribute is settable
        # so name, alignment, and others are read-only
        self.wUpdate.Show(propName not in cantmodify)

    def onSelectComponentListEvent(self, event):
        #print 'selectComponentListEvent: %s\n' % event.GetString()
        wName, wClass = event.GetString().split("  :  ")
        # change the wPropertiesList to only show relevant properties
        # either display the name by default or try and preserve the
        # wPropertiesList selection and display that item, so for example
        # you could look at size for all widgets, simply by going up and down
        # the components list
        self.setValidProps(wClass)
        propName = self.wPropertyList.GetStringSelection()
        #print propName
        if propName == "":
            propName = "name"
            self.wPropertyList.SetStringSelection("name")
        self.displayProperty(wName, wClass, propName)
        #self.wCopy.Show(1)  # probably only need to do this once, but what the hey

    def onSelectPropertyListEvent(self, event):
        propName = event.GetString()
        wName, wClass = self.wComponentList.GetStringSelection().split("  :  ")
        if wName != "":
            self.displayProperty(wName, wClass, propName)

    def displayComponents(self, componentsList):
        for c in componentsList:
            self.addWidgetToComponentsList(c)
            """
            wName = c.name
            wClass = str(c.__class__).split('.')
            self.wComponentList.Append(wName + "  :  " + wClass[len(wClass) - 1])
            """

    def eventOccurred(self, aDispatchEvent):

        event = aDispatchEvent.getDispatchedEvent()


    def onSize(self, event):
        size = self.GetClientSize()
        #self.msgHistory.SetSize(wxSize(size.width, size.height - 30))
        event.Skip()

    def onCloseMe(self, event):
        self.Show(false)


class PyCrustFrame(wxMiniFrame):
    def __init__(self, parent, ID, title, pos, size, parentApp):
        wxMiniFrame.__init__(self, parent, ID, title, pos, size, style=wxDEFAULT_FRAME_STYLE | wxTINY_CAPTION_HORIZ)
        
        #parentApp.shell = Shell(editorParent=self)
        parentApp.shell = Shell(self, -1)
        # KEA this is temporary until we decide what reference
        # we want here
        parentApp.shell.interp.locals['pcapp'] = parentApp
        self.parentApp = parentApp

        EVT_CLOSE(self, self.onCloseMe)

    def onCloseMe(self, event):
        self.Show(false)


class DebugMenu:

    def __init__(self, parentApp):
        self.parentApp = parentApp
        # decided not to use a resource in order to avoid getting the Debug
        # menu caught up in the PythonCard event model
        #basePath = os.path.dirname(os.path.abspath( __file__))
        #path = os.path.join(basePath, PYTHONCARD_DEBUG_MENU_RESOURCE_FILE)
        #self._resource = ResourceFile(path).getResource()

    def createMenu(self, bg):
        menu = wxMenu()
        id =  uniqueid.UniqueIdFactory().createUniqueId()
        menu.Append(id, 'Message Watcher')
        id =  uniqueid.UniqueIdFactory().createUniqueId()
        menu.Append(id, 'Property Editor')
        id =  uniqueid.UniqueIdFactory().createUniqueId()
        menu.Append(id, 'Shell')
        menu.AppendSeparator()
        id =  uniqueid.UniqueIdFactory().createUniqueId()
        menu.Append(id, 'About PythonCard...')
        
        menubar = bg.GetMenuBar()
        if menubar is None:
            menubar = wxMenuBar()
            bg.SetMenuBar(menubar)
        menubar.Append(menu, 'Debug')

    def bindMenuEvents(self, bg):
        # it appears that at the time bindMenuEvent and createMenu are
        # called from Background, the _iterator doesn't exist yet?!
        # so I had to pass in the current background
        #bg = self.parentApp._iterator.getCurrentBackground()
        menubar = bg.GetMenuBar()
        id = menubar.FindMenuItem('Debug', 'About PythonCard...')
        EVT_MENU(bg, id, self.doAboutPythonCard)
        id = menubar.FindMenuItem('Debug', 'Message Watcher')
        EVT_MENU(bg, id, self.toggleMessageWatcher)
        id = menubar.FindMenuItem('Debug', 'Property Editor')
        EVT_MENU(bg, id, self.togglePropertyEditor)
        id = menubar.FindMenuItem('Debug', 'Shell')
        if PYCRUST_FOUND:
            EVT_MENU(bg, id, self.toggleShell)
        else:
            menubar.Enable(id, 0)

    def toggleMessageWatcher(self, event):
        self.parentApp.mw.Show(not self.parentApp.mw.IsShown())

    def togglePropertyEditor(self, event):
        self.parentApp.pw.Show(not self.parentApp.pw.IsShown())

    def toggleShell(self, event):
        self.parentApp.shellFrame.Show(not self.parentApp.shellFrame.IsShown())
        
    # the references to self will have to be modified depending on where this
    # method ends up going. we do need a parent for the dialog
    def doAboutPythonCard(self, event):
        """Displays a ScrolledMessageDialog containing info about PythonCard and version numbers"""

        aboutTitle = "About PythonCard"
        txt = """PythonCard prototype
        
PythonCard is a software construction kit (in the spirit of Apple's HyperCard) written in Python.

"""
        txt += "PythonCard version: %s\n" % __version__.ver
        txt += "wxPython version: %s\n" % wxc.__version__
        try:
            txt += "PyCrust version: %s\n" % PyCrust.version.VERSION
            #parentApp.shell.version
            #self.stack.app.shell.version
        except:
            pass
        txt += "Python version: %s\n" % sys.version
        txt += "Platform: %s\n" % os.sys.platform
        txt += """
        
For more information see the docs directory included with this distribution.

Latest release files:
http://sourceforge.net/project/showfiles.php?group_id=19015

PythonCard home page
http://pythoncard.sourceforge.net/

SourceForge summary page
http://sourceforge.net/projects/pythoncard/

Mailing list
http://lists.sourceforge.net/lists/listinfo/pythoncard-users

PythonCard requires Python 2.1 or later and wxPython 2.3.x. wxPython is available at http://www.wxpython.org/
PythonCard relies on wxPython, so it will support the Macintosh once wxPython has been ported to the Mac.
"""
        dlg = dialog.ScrolledMessageDialog(self.parentApp._iterator.getCurrentBackground(), txt, aboutTitle)
