import time
from wxPython.wx import *
from wxPython.lib.dialogs import wxScrolledMessageDialog

ICON_EXCLAMATION = wxICON_EXCLAMATION   # Shows an exclamation mark icon.  
ICON_HAND = wxICON_HAND                 # Shows an error icon.  
ICON_ERROR = wxICON_ERROR               # Shows an error icon - the same as wxICON_HAND.  
ICON_QUESTION = wxICON_QUESTION         # Shows a question mark icon.  
ICON_INFORMATION = wxICON_INFORMATION   # Shows an information (i) icon.

BUTTON_OK = wxOK                        # Show an OK button.  
BUTTON_CANCEL = wxCANCEL                # Show a Cancel button.  
BUTTON_YES_NO = wxYES_NO                # Show Yes and No buttons.  
BUTTON_YES_DEFAULT = wxYES_DEFAULT      # Used with wxYES_NO, makes Yes button the default - which is the default behaviour.  
BUTTON_NO_DEFAULT = wxNO_DEFAULT        # Used with wxYES_NO, makes No button the default.  

RETURNED_OK = wxID_OK
RETURNED_CANCEL = wxID_CANCEL
RETURNED_YES = wxID_YES
RETURNED_NO = wxID_NO

#TEXT_MULTILINE = wxTE_MULTILINE
TEXT_PASSWORD = wxTE_PASSWORD

class ModalDialog :
                       
    def __init__( self, aParent ) :
        self._parent = aParent
        self._accepted = 0
        self._returned = 0
        self._dialog = None

    def _getParent( self ) :
        return self._parent

    def _setDialog( self, aWxDialog ) :
        self._dialog = aWxDialog

    def _getDialog( self ) :
        return self._dialog

    # KEA I don't think we need this
    def _setAccepted( self, aBoolean ) :
        self._accepted = aBoolean

    def _showModal( self ) :
        # KEA added wxID_YES to accepted condition
        # and added a returned value
        self._returned = self._dialog.ShowModal()
        if self._returned == wxID_OK or self._returned == wxID_YES :
            self._accepted = 1
        # KEA don't destroy until after we get the results
        # so self._destroy() added to each dialog class below
        #self._destroy()

    # KEA should these be getAccepted, getReturned?
    def accepted( self ) :
        return self._accepted

    # KEA added accessor for self._returned
    # should we just return numeric values instead?
    # should the the strings all be lowercase
    # Ok is actually displayed as OK in dialogs
    def returned( self ) :
        ret = self._returned
        if ret == RETURNED_OK:
            return "Ok"
        elif ret == RETURNED_CANCEL:
            return "Cancel"
        elif ret == RETURNED_YES:
            return "Yes"
        elif ret == RETURNED_NO:
            return "No"

    def _destroy( self ) :
        self._dialog.Destroy()

class ColorDialog( ModalDialog ) :
    
    def __init__( self, aParent ) :
        self._color = None
        ModalDialog.__init__( self, aParent )
        dialog = wxColourDialog( self._getParent() ) # add color data support later
        dialog.GetColourData().SetChooseFull(true)
        self._setDialog( dialog )
        self._showModal()
        # KEA why does this work? the dialog should already be destroyed
        # by _showModal()
        self._colorData = dialog.GetColourData()
        self._color = self._colorData.GetColour().Get()
        self._destroy()
        
    def getColor( self ) :
        return self._color

# do not rely on the return value of getFont
# I just added this today 2001-07-28 and haven't
# done enough with fonts to know what I need to use and ignore
class FontDialog( ModalDialog ) :
    
    def __init__( self, aParent ) :
        self._color = None
        self._font = None
        ModalDialog.__init__( self, aParent )
        # RDS: You have to pass a wxFontData instance below,
        # None causes a seg fault.
        dialog = wxFontDialog( self._getParent(), wxFontData() ) # add font data support later
        self._setDialog( dialog )
        self._showModal()
        # KEA why does this work? the dialog should already be destroyed
        # by _showModal()
        fontData = dialog.GetFontData()
        self._color = fontData.GetColour().Get()
        font = fontData.GetChosenFont()
        #self._font = (font.GetFamily(), font.GetFaceName(), font.GetPointSize(), font.GetStyle())
        self._font = (font.GetFaceName(), font.GetPointSize())
        self._destroy()
        
    def getColor( self ) :
        return self._color

    def getFont( self ) :
        return self._font

class TextEntryDialog( ModalDialog ) :
    
    def __init__( self, aParent, aWindowTitle, aMessage, aDefaultText, aStyle=0 ) :
        self._text = None
        ModalDialog.__init__( self, aParent )
        dialog = wxTextEntryDialog( self._getParent(), 
                                    aMessage,
                                    aWindowTitle,
                                    aDefaultText,
                                    aStyle | wxOK | wxCANCEL )
        self._setDialog( dialog )
        self._showModal()
        # KEA why does this work? the dialog should already be destroyed
        # by _showModal()
        self._text = dialog.GetValue()
        self._destroy()
        
    def getText( self ) :
        return self._text


class MessageDialog( ModalDialog ) :
    
    def __init__( self, aParent, aMessage, aTitle,
                  aIcon = ICON_INFORMATION, aButtons = BUTTON_OK | BUTTON_CANCEL) :
        ModalDialog.__init__( self, aParent )
        dialog = wxMessageDialog(self._getParent(), 
                                  aMessage,
                                  aTitle, 
                                  aIcon | aButtons )
        self._setDialog( dialog )
        self._showModal()
        self._destroy()


# KEA this class should go away
# it does the exact same thing as the default for MessageDialog
"""
class ConfirmationDialog( ModalDialog ) :
    
    def __init__( self, aParent, aMessage, aTitle ) :
        ModalDialog.__init__( self, aParent )
        dialog = wxMessageDialog( self._getParent(), 
                                  aMessage,
                                  aTitle, 
                                  ICON_INFORMATION | BUTTON_OK | BUTTON_CANCEL )
        self._setDialog( dialog )
        self._showModal()
"""

# KEA alerts are common, so providing a class rather than requiring the user code
# to set up the right icons and buttons with MessageDialog
class AlertDialog(MessageDialog) :
    def __init__(self, aParent, aMessage, aTitle) :
        MessageDialog.__init__(self, aParent, aMessage, aTitle, ICON_EXCLAMATION, BUTTON_OK)

class ScrolledMessageDialog( ModalDialog ) :
    
    def __init__( self, aParent, aMessage, aTitle ) :
        ModalDialog.__init__( self, aParent )
        dialog = wxScrolledMessageDialog( self._getParent(), 
                                  aMessage,
                                  aTitle)
        self._setDialog( dialog )
        self._showModal()
        self._destroy()
        

class FileDialog( ModalDialog ) :
 
    def __init__( self, aParent, aMessage, aPath, aFileName, aFilter ) :
        ModalDialog.__init__( self, aParent )
        self._message = aMessage
        self._path = aPath
        self._fileName = aFileName
        self._filter = aFilter
        self._value = None
        self._paths = None

        dialog = wxFileDialog( self._getParent(), 
                               aMessage,
                               aPath, 
                               aFileName, 
                               aFilter, 
                               wxOPEN | wxMULTIPLE )

        self._setDialog( dialog )

        # KEA changed to self, was:
        # ModalDialog._showModal( self )
        self._showModal()

        if self.accepted() :
            self._paths = self._dialog.GetPaths()
        self._destroy()
 
    def getPaths( self ) :
        return self._paths


class DirectoryDialog( ModalDialog ) :
 
    def __init__( self, aParent, aMessage, aPath ) :
        ModalDialog.__init__( self, aParent )
        self._message = aMessage
        self._path = aPath
        self._value = None
        self._setDialog( wxDirDialog( self._getParent() ) )

        if self._message is not None :
            self._dialog.SetMessage( self._message )

        if self._path is not None :
            self._dialog.SetPath( self._path )

        # KEA changed to self, was:
        # ModalDialog._showModal( self )
        self._showModal()

        if self.accepted() :
            self._value = self._dialog.GetPath()
        self._destroy()
 
    def getPath( self ) :
        return self._value
