"""
__version__ = "$Revision: 1.2 $"
__date__ = "$Date: 2001/08/14 00:41:09 $"
"""

import sys
from wxPython.wx import * # rds-fix
from dispatch import *
from res import *
from event import *
from pom import *
from uniqueid import *

class MenuBar :
    """
    Defines the PythonCard Menu object model.

    A MenuBar contains 0..n Menu objects.

    Currently, menus can only  be created from a resource.
    The interfaces for dynamically creating menus and items,
    enabling/disabling items, deleting items, etc., are not
    implemented.

    This is a VERY thin wrapper for wxPython.  In the future
    we should use the Adapter pattern:

        MenuBar <>---- MenuBarAdapter <--- wxPython_MenuBar

        Menu <>---- MenuAdapter <--- wxPython_Menu

        MenuItem <>---- MenuItemAdapter <--- wxPython_MenuItem

        Where MenuBar contains and delegates to a MenuBarAdapter
        and wxPython_MenuBar is a concrete implementation of
        MenuBarAdapter.

        We can then dynamically load/switch concrete 
        implementations by name.
    """

    def __init__( self, aParent, aResource ) :
        self.menus = []
        self.parseMenus( aParent, aResource.stack.menubar.menus )
        self.delegate = WxMenuBar( aParent, self )
       
    def parseMenus( self, aParent, aResource ) :
        for menu in aResource :
            self.menus.append( Menu( aParent, menu ) )

    def __repr__( self ) :
        return 'MenuBar=' + str( self.__dict__ )

    def appendMenu( self, aMenu ) :
        pass

    def insertMenu( self, aMenu, aIndex ) :
        pass

    def deleteMenu( self, aName ) :
        pass

    def getMenu( self, aName ) :
        pass

    def getMenuByIndex( self, aIndex ) :
        pass

    def enable( self ) :
        pass

    def disable( self ) :
        pass

    def getMenus( self ) :
        return self.menus

    def enableCommand( self, aString ) :
        self.delegate.enableCommand( aString )

    def disableCommand( self, aString ) :
        self.delegate.disableCommand( aString )

class Menu :
    """
    A Menu contains 0..n MenuItem objects.
    """

    def __init__( self, aParent, aResource ) :
        self.name = aResource.name
        self.label = aResource.label
        self.items = []
        self.parseMenuItems( aParent, aResource.items )

    def parseMenuItems( self, aParent, aResource ) :
        for item in aResource :
            self.appendMenuItem( MenuItem( aParent, item ) )

    def _getName( self ) :
        return self.name

    def getLabel( self ) :
        return self.label

    def appendMenuItem( self, aMenuItem ) :
        self.items.append( aMenuItem )

    def insertMenuItem( self, aMenuItem, aIndex ) :
        pass

    def deleteMenuItem( self, aName ) :
        pass

    def getMenuItem( self, aName ) :
        pass

    def getMenuItemByIndex( self, aIndex ) :
        pass

    def enable( self ) :
        pass

    def disable( self ) :
        pass

    def getMenuItems( self ) :
        return self.items

    def __repr__( self ) :
        return 'Menu=' + str( self.__dict__ )

class MenuItem( Component ) :
    """
    A MenuItem represents one selectable item in a Menu.

    A MenuItem with the name '-' is interpreted as
    a  separator.
    """

    def __init__( self, aScriptable, aResource ) :
        Component.__init__( self )
        self.name = aResource.name
        self.label = aResource.label
        self.command = aResource.command
        EventDispatch( self, aScriptable )

    def _getName( self ) :
        return self.name

    def getLabel( self ) :
        return self.label

    def getCommand( self ) :
        return self.command

    def setName( self, aString ) :
        self.name = aString
             
    def setLabel( self, aString ) :
        self.label = aString

    def setCommand( self, aString ) :
        self.command = aString

    def __repr__( self ) :
        return 'MenuItem=' + str( self.__dict__ )

class WxMenuBar :

    def __init__( self, aParent, aMenuBar ) :

        self.parent = aParent

        self.id2itemMap = {}

        aWxMenuBar = wxMenuBar()

        menus = aMenuBar.getMenus()

        for menu in menus :

            aWxMenu = wxMenu()

            items = menu.getMenuItems()

            for item in items :

                id =  UniqueIdFactory().createUniqueId()

                self.id2itemMap[ id ] = item

                if item.getLabel() == '-' :
                    aWxMenu.AppendSeparator()
                else :
                    aWxMenu.Append( id, item.getLabel() )
                    #aWxMenuItem = wxMenuItem( None, id, item.getLabel() )
                    #item.delegate = aWxMenuItem
                    #aWxMenu.Append( aWxMenuItem )

                EVT_MENU( aParent, id, self.doMenu )

            aWxMenuBar.Append( aWxMenu, menu.getLabel() )

        aParent.SetMenuBar( aWxMenuBar )

    def doMenu( self, aWxMenuEvent ) :
        id = aWxMenuEvent.GetId()
        #if self.id2cmdMap.has_key( id ) :
        #    command = self.id2cmdMap[ id ]
        #    if command == None :
        #        event = MenuEvent( self.id2nameMap[ id ], self )
        #    else  :

        item = self.id2itemMap[ id ]
        if item.getCommand() is not None :
            event = CommandEvent( item.getCommand(), item )
        else :
            event = SelectEvent( item )
        item.notifyEventListeners( event )
            
    def enableCommand( self, aString ) :
        for item in self.id2itemMap.values() :
            if item.getCommand() is aString :
                #print 'enabling menu item', item.getName()
                pass

    def disableCommand( self, aString ) :
        for item in self.id2itemMap.values() :
            if item.getCommand() is aString :
                #print 'disabling menu item', item.getName()
                pass

# Unit Test

if __name__ == '__main__' :
    resource = ResourceFile( sys.argv[ 1 ] ).getResource()



