
"""
__version__ = "$Revision: 1.27 $"
__date__ = "$Date: 2001/08/21 19:45:50 $"
"""

import imp, string, sys
import wxPython
from event import *
from dispatch import *
from res import *
from monitor import *
from widget import *
from menu import *
from navigation import *
from binding import *
from config import *
import debug
import types
import warnings
import UserDict

# KEA 2001-07-27
from wxPython.lib import colourdb

class WidgetDict(UserDict.UserDict, Changeable):
    def __init__(self, parent, dict=None):
        self.__dict__['parent'] = parent
        self.__dict__['data'] = {}
        if dict is not None: self.update(dict)
        Changeable.__init__(self)

    # __getattr__ and __setattr__ need to be cleaned up
    # to support more attributes
    def __getattr__(self, key):
        if key == '_listeners':
            return self.__dict__[key]
        else:
            return self.data[key]

    def __setattr__(self, key, item):
        if key == '_listeners':
            self.__dict__[key] = item
        else:
            self.__setitem__(key, item)

    """
    def __delattr__(self, key):
        if key == '_listeners':
            pass
        else:
            self.__delitem__(key)
    """

    def __setitem__(self, key, item):
        if type(item) is types.DictionaryType:
            item = Resource(item)
        widget = WidgetFactory().createWidget(self.parent, self.parent.panel, item)
        self.data[key] = widget
        # notify listeners
        self.fireChanged(key + "," + str(widget.__class__.__name__))

    def __delitem__(self, key):
        widget = self.data[key]
        wClass = str(widget.__class__.__name__)
        if widget._delegate.Destroy():
            del self.data[key]
            self.fireChanged(key + "," + wClass)

    def clear(self):
        for key in self.data.keys():
            self.__delitem__(key)

    def _getAttributeNames(self):
        return self.data.keys()


class PythonCardApp( wxApp ) :
    """
    The runtime environment for PythonCard Stacks.
    """

    def __init__( self, aFileName ) :
        showProperties = Configuration().getOption( 'showPropertyEditor' )
        showMessages = Configuration().getOption( 'showMessageWatcher' )
        enableLogging = Configuration().getOption( 'enableLogging' )
        showShell = Configuration().getOption( 'showShell' )
 
        if enableLogging:
            Log.getInstance().enable()

        # Load the user-defined resource file, 
        # validating it against spec.py
        self.resource = ResourceFile(aFileName).getResource()
        oldCwd = os.getcwd()	# KEA save the current directory just in case
        # RDS - Need to get the absolute pathname of aFileName before
        # getting the directory name .
        path = os.path.dirname( os.path.abspath( aFileName ) )   # keep track of the path to prefix with
        os.chdir(path)                      # always run in the app's directory
        bg = self.resource.stack.backgrounds[0]
        moduleName, ext = bg.file.split('.')

        # KEA 2001-08-09 support for loading .pyw or .py module
        # I'm assuming here that the .rsrc.py still specifies .py for the file
        if os.path.exists(bg.file):
            module = imp.load_source( moduleName, bg.file )
        elif os.path.exists(moduleName + ".pyw"):
            module = imp.load_source( moduleName, moduleName + ".pyw")
        else:
            # KEA should probably throw an exception here since we can't
            # find a module file to load with either a .py or .pyw extension
            print "unable to load", bg.file

        clazz = module.__dict__[ bg.classname ]
        self.debug = false
        self.frame = None
        self.frameClass = clazz
        self.pw = None
        self.mw = None
        self.shellFrame = None
        self.shell = None
        self._showProperties = showProperties 
        self._showMessages = showMessages
        self._showShell = showShell
        # KEA 2001-08-11
        # if any of the debug windows are going to be used then
        # we will have a debug menu and all the windows will be available
        # but only the ones requested at startup will be visible initially
        # this seems like a nice compromise, but we can change the behavior
        # if it seems strange
        self._showDebugMenu = showProperties + showMessages + showShell
        self._debugMenu = debug.DebugMenu(self)
        colourdb.updateColourDB()   # KEA 2001-07-27
        wxInitAllImageHandlers()
        wxApp.__init__( self, 0 )

        
    def showPropertyEditor(self):
        if self._showDebugMenu:
            bg = self._iterator.getCurrentBackground()
            position = Configuration().getOption('propertyEditorPosition')
            size = Configuration().getOption('propertyEditorSize')
            self.pw = debug.PropertyEditor(None, -1, "Property Editor",
                              position, size,
                              parentApp=self)
            #self.pw.displayComponents(self._iterator.getCurrentBackground().findAllComponents())
            self.pw.displayComponents(self._iterator.getCurrentBackground().components.values())
            self.pw.Show(self._showProperties)

    def showMessageWatcher(self):

        if self._showDebugMenu:
            bg = self._iterator.getCurrentBackground()
            position = Configuration().getOption('messageWatcherPosition')
            size = Configuration().getOption('messageWatcherSize')
            # KEA frame isn't bound to anything, so I think this is supposed
            # to be debug.MessageWatcher( self._iterator.getCurrentBackground()
            # instead of debug.MessageWatcher( self.frame
            # however I don't know what happens once there are
            # multiple backgrounds ?!
            # perhaps there should always be a hidden parent PythonCard app
            # window that all other windows are children of ?!
            # KEA also changed mw to self.mw
            # KEA 2001-08-06
            # there is a bug in destroying the windows, so mw is still a child
            # of the main window, while shell and pw are not
            self.mw = debug.MessageWatcher(bg, -1, "Message Watcher",
                              position, size,
                              parentApp=self)
            self.mw.Show(self._showMessages)

    def showShell(self):

        if debug.PYCRUST_FOUND and self._showDebugMenu:
            # KEA see comments in showMessageWatcher about frames
            bg = self._iterator.getCurrentBackground()
            position = Configuration().getOption('shellPosition')
            size = Configuration().getOption('shellSize')
            self.shellFrame = debug.PyCrustFrame(None, -1, 'Shell',
                                                 position, size,
                                                 parentApp=self)
            self.shellFrame.Show(self._showShell)
            
            path = os.path.dirname(__file__)
            filename = os.path.join(path, 'pycrustrc.py')
            if os.path.exists(filename):
                try:
                    self.shell.runfile(filename)
                except:
                    pass

            # we will have already changed into the running module
            # directory, so this should be okay
            filename = 'pycrustrc.py'
            if os.path.exists(filename):
                try:
                    self.shell.runfile(filename)
                except:
                    pass




    # wxWindows calls this method to initialize the application
    def OnInit( self ) :
        Log.getInstance().debug( 'Initializing Background...' )
        self._stack = Stack( self, self.resource )
        self._iterator = StackIterator( self._stack )
        bg = self._iterator.getCurrentBackground()
        bg.Show( true )
        self.showPropertyEditor()
        self.showMessageWatcher()
        self.showShell()    # KEA pyCrust support
        return 1

    def getCurrentBackground(self):
        return self._iterator.getCurrentBackground()

class Scriptable :
    """
    All classes that may contain PythonCard Handler definitions
    must implement Scriptable.

    A Scriptable object may be specified as the parent of
    this object.  The parent will be searched for Handlers
    if a Handler can't be found in this object.
    """

    def __init__( self, aScriptableParent ) :
        self._parentScript = aScriptableParent
        self._handlers = {}
        self._parseHandlers()

    def _parseHandlers( self ) :
        """
        Find all of the methods in this object that are
        PythonCard handlers, and register them.
        """
        aClass = self.__class__
        for key in aClass.__dict__.keys() :
            value = aClass.__dict__[ key ]
            if type( value ) is FunctionType :
                if self._isHandler( value ) :
                    handler = Handler( value.__name__, value )
                    self._addHandler( value.__name__, handler )

    def _isHandler( self, aFunction ) :
        """
        Return true if the Function object is a 
        PythonCard handler.
        """
        return string.split( aFunction.__name__, '_' )[ 0 ] == 'on'

    def _addHandler( self, aString, aHandler ) :
        # Add the Handler to our Handler list.
        self._handlers[ aString ] = aHandler

    def findHandler( self, aString ) :
        """
        Look for a Handler that matches aString in our 
        list of Handlers.  If a Handler is not found,
        ask our parent script to look for the Handler, 
        continuing up the Scriptable hierarchy until
        either a Handler is found, or None is returned.
        """
        handler = self.findLocalHandler( aString ) 

        if handler is not None :
            return handler

        # We couldn't find a handler, so look in our parent.

        if self._parentScript is not None :
            handler = self._parentScript.findHandler( aString )
        
        # have we found a Handler yet?

        if handler is not None :
            return handler
        

        # Change the handler name to target this Scriptable object
        # and look in our list of Handlers.

        words = string.split( aString, '_' )
        aString = words[ 0 ] + '_' + self._getName() + '_' + words[ 2 ]
        return self.findLocalHandler( aString )

    def findLocalHandler( self, aString ) :
        """
        Look for a Handler in our list of Handlers only.
        """
        for key in self._handlers.keys() :
            handler = self._handlers[ key ]
            if handler.getName() == aString :
                return handler
        return None


class PythonCardObject( Changeable ) :
    """
    The superclass of Background and Card classes.  The unique id
    property is never repeated and is globally unique within a Stack?

    The name property is optional.

    The number property is dynamic...
    """

    def __init__( self ) :
        """
        Initialize this instance.
        """
        self.id = UniqueIdFactory().createUniqueId()
        self.name = ''
        Changeable.__init__( self )

    def getId( self ) :
        """
        Get the unique id of this object.
        """
        return self.id 

    def setName( self, name ) :
        """
        Set the name of this object.

        Q. Are we going to enforce unique names?
           Should they be unique to a Background
           or the entire stack?
        """
        oldName = self.name
        self.name = name 
        self.fireChanged( oldName )

    def _getName( self ) :
        """
        Get the name of this object.
        """
        return self.name 

class ObjectMap( ChangeListener ) :
    """
    Maintains a mapping of PythonCardObjects by name and
    id.
    """

    def __init__( self ) :
        """
        Initialize this instance.
        """
        self.objects = []
        self.idIndex = {}
        self.nameIndex = {}

    def listenTo( self, aPythonCardObject ) :
        """
        Register this object as a ChangeListener
        with aPythonCardObject.
        """
        aPythonCardObject.addChangeEventListener( self )

    def changed( self, aChangeEvent ) :
        """
        Called by a Changeable object that is referenced
        by this map.  The only current responsibility
        is to update the nameIndex.
        """
        object = aChangeEvent.getChangedObject()
        oldValue = aChangeEvent.getOldValue()
        assert self.nameIndex.has_key( object._getName() ) == 0, 'duplicate name! ' + oldValue
        del self.nameIndex[ oldValue ]
        self.nameIndex[ object._getName() ] = object

    def append( self, aPythonCardObject ) :
        """
        Append the object to our list of objects, and update
        the id and name indexes.
        """
        self.objects.append( aPythonCardObject )
        self._addToIndexes( aPythonCardObject )
        self.listenTo( aPythonCardObject )

    def insert( self, index, aPythonCardObject ) :
        """
        Insert the object to our list of objects at index, and update
        the id and name indexes.
        """
        assert index > -1 and index < ( len( self.objects ) - 1 ), 'index out of range!'
        self.objects.insert( index, aPythonCardObject )
        self._addToIndexes( aPythonCardObject )
        self.listenTo( aPythonCardObject ) 

    def deleteById( self, aUniqueId ) :
        """
        Delete the object identified by aUniqueId from the
        object list and the id and name indexes.
        """
        target = self.getById( aUniqueId )
        self._deleteFromIndexes( target )
        self._deleteFromObjects( target )

    def deleteByName( self, aString ) :
        """
        Delete the object with name==aString from the
        object list and the id and name indexes.
        """
        target = self.getByName( aString )
        self._deleteFromIndexes( target )
        self._deleteFromObjects( target )
    
    def deleteByNumber( self, aNumber ) :
        """
        Delete the object in the object list at index==aNumber
        from the object list and the id and name indexes.
        """
        aNumber = aNumber - 1 
        target = self.objects[ aNumber ]
        self._deleteFromIndexes( target )
        self._deleteFromObjects( target )

    def getFirstObject( self ) :
        """
        Get the first object in the objects list.
        """
        return self.objects[ 0 ]

    def getLastObject( self ) :
        """
        Get the last object in the objects list.
        """
        return self.objects[ -1 ]

    def getById( self, aUniqueId ) : # returns a PythonCardObject
        """
        Get an object by unique id.
        """
        return self.idIndex[ aUniqueId ]
    
    def getByName( self, aString ) :
        """
        Get an object by name.
        """
        assert self.nameIndex.has_key( aString ), aString
        return self.nameIndex[ aString ]

    def getByNumber( self, aNumber ) :
        """
        Get an object by index.
        """
        aNumber = aNumber - 1 
        return self.objects[ aNumber ]

    def getIndexOfObject( self, aObject ) :
        """
        Return the index of aObject.
        """
        i = 1
        for object in self.objects :
            if object == aObject :
                return i
        return -1
    
    def getNumObjects( self ) :
        """
        Return the number of objects in this map.
        """
        return len( self.objects )

    def _addToIndexes( self, aPythonCardObject ) :
        """
        Add an object to the id and name indexes.
        """
        self.idIndex[ aPythonCardObject.getId() ] = aPythonCardObject
        self.nameIndex[ aPythonCardObject._getName() ] = aPythonCardObject

    def _deleteFromIndexes( self, aPythonCardObject ) :
        """
        Delete an object from the id and name indexes.
        """
        del self.idIndex[ aPythonCardObject.getId() ]
        del self.nameIndex[ aPythonCardObject._getName() ]
    
    def _deleteFromObjects( self, aPythonCardObject ) :
        """
        Delete the object from the object list.
        """
        uniqueId = aPythonCardObject.getId()
        for object in self.objects :
            if object.getId() == uniqueId :
                self.objects.remove( object )
                return

# The number is a dynamic property, based on context.
# For example, the number of a Background is it's index
# in the enclosing Stack's list of Backgrounds.  And
# the number of a Card is it's index in the enclosing
# background's list of Cards.

    #def setNumber( self, number ) :
    #    self.number = number 

    #def getNumber( self ) :
    #    return self.number 

class Stack( PythonCardObject, Scriptable ) :
    """
    Models an application written for the PythonCard environment.
    """
   
    def __init__( self, aPythonCardApp, aResource ) :
        """
        Initialize this instance.
        """
        PythonCardObject.__init__( self )
        Scriptable.__init__( self, None )
        self.app = aPythonCardApp
        self.setName( aResource.stack.name )
        # KEA 2001-08-13
        # override for application defined position
        # we'll have to revisit this as we start supporting
        # multiple windows for an application, stack, etc.
        position = Configuration().getOption('defaultStackPosition')
        if position is None:
            self.setPosition(aResource.stack.position)
        else:
            self.setPosition(position)
        self.setSize( aResource.stack.size )
        self.setTitle( aResource.stack.title )
        self.map = ObjectMap()
        self._initBackgrounds( aResource )
        self.curBg = self.map.getFirstObject()

    def setPosition( self, aPosition ) :
        self.position = aPosition

    def setSize( self, aSize ) :
        self.size= aSize


    def getPosition( self ) :
        return self.position

    def getSize( self ) :
        return self.size

    def setTitle( self, aString ) :
        self.title = aString

    def getTitle( self ) :
        return self.title

    def _initBackgrounds( self, aResource ) :
        backgrounds = aResource.stack.backgrounds
        for bgRsrc in backgrounds :            
            bg = self.app.frameClass( self, 1, aResource, bgRsrc )
            self.map.append( bg )

    def newBackground( self, aName, aLayout ) :
        """
        Create a new Background for this Stack.
        """
        b = Background( aName )
        self.map.append( b )

    def deleteBackground( self, aUniqueId ) :
        """
        Delete a Background based on it's unique id.
        """
        self.map.delete( aUniqueId )
    
    def getFirstBackground( self ) :
        """
        Get the first Background in the Stack.
        """
        return self.map.getFirstObject()

    def getLastBackground( self ) :
        """
        Get the last Background in the Stack.
        """
        return self.map.getLastObject()

    def getBackgroundById( self, aUniqueId ) :
        """
        Get a Background that matches aUniqueId.
        """
        return self.map.getById( aUniqueId )
    
    def getBackgroundByName( self, aString ) :
        """
        Get a Background that's name matches aString.
        """
        return self.map.getByName( aString )

    def getBackgroundByNumber( self, aNumber ) :
        """
        Get a Background by it's index in the Stack's
        list of Backgrounds.
        """
        return self.map.getByNumber( aNumber )

    def getIndexOfBackground( self, aBackground ) :
        """
        Return the index of aBackground.
        """
        return self.map.getIndexOfObject( aBackground )

    def getNumBackgrounds( self ) :
        """
        Return the number of Backgrounds in this Stack.
        """
        return self.map.getNumObjects()

class Background( PythonCardObject, Scriptable, wxFrame ) :
    """
    A Background contains the Widgets
    that appear on the Background, and a list of Cards
    that contain data to be displayed in the Background.
    """

    def __init__( self, aStack, aId, aStackRsrc, aBgRsrc ) :
        """
        Initialize this instance.
        """
        PythonCardObject.__init__( self )
        Scriptable.__init__( self, aStack )
        self.stack = aStack
        self.id = UniqueIdFactory().createUniqueId()
        self.setName( aBgRsrc.name )
        #self.setForegroundColor( aBgRsrc.foregroundColor )
        #self.setBackgroundColor( aBgRsrc.backgroundColor )
        self.setImage( aBgRsrc.image )
        self.setTiled( aBgRsrc.tiled )
        self.map = ObjectMap()
        self.newCard( '', None )
        #self.components = {}
        self.components = WidgetDict(self)
        self.menuBar = None

        # First, call the base class' __init__ method to create the frame
        wxFrame.__init__( self, None,   
                          -1, 
                          #self.name,
                          aStack.getTitle(),
                          aStack.getPosition(), 
                          aStack.getSize() )

        self._initLayout( aBgRsrc.components )

        # KEA 2001-08-13
        # can't set the colors until after the panel
        # has been created. this initialization should probably work differently
        self.setForegroundColor( aBgRsrc.foregroundColor )
        self.setBackgroundColor( aBgRsrc.backgroundColor )

        self._createMenus( aStackRsrc )

        self._bindWindowEvents()

        self.doMeFirst()

        self.stack.app.SetTopWindow( self )
 
    def setName( self, aString ) :
        self.name = aString

    def setBackgroundColor(self, aColor):
        self.panel.setBackgroundColor(aColor)

    def setForegroundColor(self, aColor):
        self.panel.setForegroundColor(aColor)

    def setImage( self, aPath ) :
        self.image = aPath
        # Call wxPython
        #raise NotImplementedError

    def setTiled( self, aBoolean ) :
        if type(aBoolean) is types.StringType:
            if aBoolean == 'false':
                aBoolean = 0
            else:
                aBoolean = 1
        self.tiled = aBoolean
        # Call wxPython
        #raise NotImplementedError

    def _getName( self ) :
        return self.name

    def getBackgroundColor(self):
        return self.panel.getBackgroundColor()

    def getForegroundColor(self):
        return self.panel.getForegroundColor()

    def getImage( self ) :
        return self.image

    def getTiled( self ) :
        return self.tiled

    def getPosition( self ) :
        return self.GetPosition()

    def setPosition( self, aPosition ) :
        self.SetPosition(aPosition)

    def getSize( self ) :
        return self.GetSize()

    def setSize( self, aSize ) :
        self.SetSize(aSize)

    def doMeFirst( self ) :
        pass

    def enableCommand( self, aString ) :
        """
        Fined every component with a 'command' attribute
        that matches aString, and enable the component.
        """
        self.menuBar.enableCommand( aString )

        for component in self.components.values() :
            if component._getCommand() is aString :
                component.setEnabled( 1 )

    def disableCommand( self, aString ) :
        """
        Fined every component with a 'command' attribute
        that matches aString, and disable the component.
        """
        self.menuBar.disableCommand( aString )

        for component in self.components.values() :
            if component._getCommand() is aString :
                component.setEnabled( 0 )

    
    def _initLayout( self, aResourceList ) :
        """
        Create the gui Widgets for this Background and lay
        them out in a Frame.
        """
        self.panel = Panel( self, self.image, self.tiled )

        for resource in aResourceList :
            #widget = self.createWidget( resource )
            self.components[resource.name] = resource

    def _createMenus( self, aResource ) :
        # RDS - Only create a menubar if one is defined
        #       in the stack's resource file.
        #       This is a hack, I shouldn't be accessing
        #       the stack resource's __dict)__ directly.
        if aResource.stack.__dict__.has_key( 'menubar' ) :
            self.menuBar = MenuBar( self, aResource )
        # KEA and as a further hack, I now add a Debug menu
        # to the menubar. createMenu will create a menubar
        # if one doesn't already exist
        if self.stack.app._showDebugMenu:
            self.stack.app._debugMenu.createMenu(self)
            self.stack.app._debugMenu.bindMenuEvents(self)
            

    def _bindWindowEvents( self ) :
        # Associate some events with methods of this class
        EVT_SIZE(self, self.OnSize)
        EVT_MOVE(self, self.OnMove)
        EVT_CLOSE(self, self.OnCloseWindow) # KEA 2001-08-06

    def newCard( self, aName, aLayout ) :
        """
        Create a new Card for this Background.
        This should generate some type of event
        that the storage component will be listening
        for...
        """
        b = Card( aName )
        self.map.append( b )

    def deleteCard( self, aUniqueId ) :
        """
        Delete a Card matching aUniqueId from this Background.
        This should generate some type of event
        that the storage component will be listening
        for...
        """
        self.map.delete( aUniqueId )
    
    def getCardById( self, aUniqueId ) :
        """
        Get a Card matching aUniqueId
        """
        return self.map.getById( aUniqueId )
    
    def getCardByName( self, aString ) :
        """
        Get a Card that's name matches aString.
        """
        return self.map.getByName( aString )

    def getCardByNumber( self, aNumber ) :
        """
        Get a Card at the specified index in this
        Backgrounds list of Cards.
        """
        return self.map.getByNumber( aNumber )

    """
    # KEA 2001-08-13
    # not just deprecated, but eradicated
    def find( self, target ) :
        warnings.warn("use components instead", DeprecationWarning)
        return self.components[target]

    def findByName( self, targetPath ) :
        # Find the widget that matches the name
        # identified in targetPath.
        warnings.warn("use components instead", DeprecationWarning)
        return self.components[targetPath]
    """
    
    def findId( self, targetPath ) :
        # Find the widget that matches the name
        # identified in targetPath.
        return self.getIDbyName( targetPath )

    """
    # cheap widget creation from resource list
    def createWidget( self, aResource ) :
        warnings.warn("use components instead", DeprecationWarning)
        if type(aResource) is types.DictionaryType:
            aResource = Resource(aResource)
        self.components[aResource.name] = aResource
        return aResource
    """

    """
    # KEA 2001-08-04
    # cheap widget deletion without much error checking
    # since I'm not sure how this would impact the message handlers
    # or other items such as saving data or state before the widget
    # is destroyed, so this is a "use at your own risk" routine
    def deleteWidget( self, name ) :
        warnings.warn("use components instead", DeprecationWarning)
        if self.components.has_key(name):
            widget = self.components[name] # self.findByName(name)
            if widget._delegate.Destroy():
                del self.components[name]   # remove from the components list
    """

    def getIDbyName( self, widgetName ) :
        try:
            return self.components[ widgetName ].getId()
        except:
            return -1

    def getObjectByName( self, widgetName ) :
        try:
            return self.components[ widgetName ]
        except:
            return -1

#    def postEvent( self, aEvent ) : 
#        self.eventQueue.postEvent( aEvent )

    def OnCloseWindow(self, event):
        """
        This method is called automatically when the CLOSE event is
        sent to this window.
        """
        if self.stack.app.pw != None:
            #print "closing property editor window"
            self.stack.app.pw.Destroy()
        #if self.stack.app.mw != None:
            #print "closing message watcher window"
        #    self.stack.app.mw.Destroy()
        if self.stack.app.shellFrame != None:
            #print "closing shell window"
            self.stack.app.shellFrame.Destroy()
        #print "destroying main window"
        # tell the window to kill itself
        self.Destroy()


    def OnSize(self, event):
        """
        This method is called by the System when the window is resized,
        because of the association above.
        """
        size = event.GetSize()

        # tell the event system to continue looking for an event handler,
        # so the default handler will get called.
        event.Skip()

    def OnMove(self, event):
        """
        This method is called by the System when the window is moved
        because of the association above.
        """
        pos = event.GetPosition()

    def OnExit(self, evt):
        self.Close(true)

    def findAllComponents( self ) :
        """
        Return a copy of the list of Components in this
        Background.  We're not just returning 
        self.components.values because we don't want
        someone to inadvertently whack the internal list
        of Components.
        """
        components = []
        for component in self.components.values() :
            components.append( component )
        return components

    def findComponentsByClass( self, aComponentClass ) :
        """
        Return a list of Component's that are instances
        of the specified Component class.
        """
        components = []
        for component in self.components.values() :
            if isinstance( component, aComponentClass ) :
                components.append( component )
        return components

    # KEA 2001-07-31
    # we may want to put this someplace else, but we do need access
    # to the componenet list
    def findFocus( self ):
        widgetWX = wxWindow_FindFocus() # the wxPython widget that has focus
        id = widgetWX.GetId()
        components = self.findAllComponents()
        for widget in components:
            if id == widget._delegate.GetId():
                return widget
        return None # is this even possible? focus in another window maybe?
        

class Card( PythonCardObject ) :
    """
    A Card is synonymous with the concept of a 'record'.

    A Background contains 1 or more Cards, where the
    Cards are views into the data stored in the
    background.
    """

    def __init__( self, aName='' ) :
        """
        Initialize this instance.
        Card's typicaly don't have names, so default to 
        an empty string for Card.name.
        """
        PythonCardObject.__init__( self )
        self.setName( aName )

    def __repr__( self ) :
        """
        Return a string representation of this object.
        """
        return str( self.__dict__ )


