"""
__version__ = "$Revision: 1.1.1.1 $"
__date__ = "$Date: 2001/08/06 19:53:12 $"
"""

import os, string, sys
from error import *

class ResourceFile :
    """
    Reads a Resource file into a dictionary.
    """

    def __init__( self, rsrcFileName ) :
        self.dictionary = eval( open( rsrcFileName ).read() )

    def getResource( self ) :
        return Resource( self.dictionary )

class Resource :

    spec = None
    """
    A generic tree of dictionary, array and primitive objects
    that is accessible using dot(.) notation.
    """
    def __init__ ( self, aDictionary ) :
           
        if Resource.spec is None :
            thisDir = os.path.dirname( os.path.abspath( __file__ ) )
            specPath = os.path.join( thisDir, 'spec.py' )
            specDict = eval( open( specPath ).read() )
            Resource.spec = Spec( specDict )

        keys = aDictionary.keys()

        for key in keys :

            value = aDictionary[ key ]

            if type( value ) is type( {} ) :

                aDictionary[ key ] = Resource( value )

            if type( value ) is type( [] ) :
                i = 0
                for item in value :
                    if type( item ) is type( {} ) :
                        value[ i ] = Resource( item )
                    i = i + 1
            
        self.__dict__.update( aDictionary )

        self.enforceSpec( aDictionary )

    def enforceSpec( self, aDictionary ) : 
        """
        If Resource.spec is not None, then we look at the Spec definition
        and enforce mandatory/optionl/default constraints on this
        Resource.

        Although I think the Resource  and Spec file formats are
        reasonable, this code is a hack job of the highest degree,
        and I don't mind admitting it - RDS ;)

        Maybe we'll have time to redesign ( or just design ) on the
        next pass ;)
        """            
        attributes = None
        typeStr = None

        if aDictionary.has_key( 'type' ) :
            typeStr = aDictionary[ 'type' ]
            # Get the appropriate spec.
            if Resource.spec is not None :
                spec = Resource.spec.getEntry( typeStr )
                attributes = spec.getAttributes()

        if attributes is not None :
            for key in attributes.keys() :
                attribute = attributes[ key ]
                if attribute.isRequired() :
                    if not self.hasAttribute( key ) :
                        msg =  'error! ' + typeStr + '.' + key + ' is mandatory'
                        raise ResourceException, msg
                elif attribute.isOptional() :
                    if not self.hasAttribute( key ) :
                        self.__dict__[ key ] = attribute.getDefaultValue()


    def hasAttribute( self, aString ) :
        return self.__dict__.has_key( aString )

    def __repr__( self ) :
        return 'Resource: ' + str( self.__dict__ )
    

class AttributeSpec :

    def __init__( self, name, properties ) :
        self.name = name
        self.presence = properties[ 'presence' ]
        self.default = None
        if self.presence is 'optional' :
            self.default = properties[ 'default' ]
        self.values = None
        if properties.has_key( 'values' ) :
            self.values = properties[ 'values' ]

    # PUBLIC METHODS

    def getName( self ) :
        return self.name

    def isRequired( self ) :
        return self.presence is 'mandatory'

    def isOptional( self ) :
        return self.presence is 'optional'

    def getDefaultValue( self ) :
        return self.default      

    def hasDefaultValueList( self ) :
        return self.values is not None

    def getDefaultValueList( self ) :
        return self.values

class ComponentSpec :

    def __init__( self, aDictionary ) :
        # PRIVATE MEMBERS
        self.name = aDictionary[ 'name' ]
        self.parentName = aDictionary[ 'info' ][ 'parent' ]
        self.parent = None
        self.events = aDictionary[ 'info' ][ 'events' ]
        self.attributes = self.parseAttributes( aDictionary[ 'info' ][ 'attributes' ] )
        self.requiredAttributes = self.parseRequiredAttributes()
        self.optionalAttributes = self.parseOptionalAttributes()

    # PUBLIC METHODS

    def getName( self ) :
        return self.name

    def getParentName( self ) :
        return self.parentName

    def getParent( self ) :
        return self.parent

    def getEvents( self ) :
        return self.events

    def getAttributes( self ) :
        return self.attributes

    def getRequiredAttributes( self ) :
        return self.requiredAttributes

    def getOptionalAttributes( self ) :
        return self.optionalAttributes

    # PRIVATE METHODS

    def parseAttributes( self, aDictionary ) :
        attributes = {}
        for key in aDictionary.keys() :
            value = aDictionary[ key ]
            attribute = AttributeSpec( key, value )
            attributes[ attribute.getName() ] = attribute
        return attributes

    def parseRequiredAttributes( self ) :
        required = {}
        keys = self.attributes.keys()
        for key in keys :
            attribute = self.attributes[ key ]
            if attribute.isRequired() :
                required[ key ] = attribute
        return required

    def parseOptionalAttributes( self ) :
        optional = {}
        keys = self.attributes.keys()
        for key in keys :
            attribute = self.attributes[ key ]
            if attribute.isOptional() :
                optional[ key ] = attribute
        return optional


class Spec :
    """
    A Spec is a dictionary that contains meta type information that
    is used to validate a Resource file.

    TODO: Raise a warning when unspecified attributes are found in
          a resource.  It may be interesting to process them and
          set them as member with get/set on instances of the 
          class where they are found? - RDS
    """
    def __init__ ( self, specArray ) :

        self.entries = []

        for item in specArray :
            self.mergeChildren( item, specArray )
            self.entries.append( ComponentSpec( item ) ) 

    # PUBLIC METHODS

    def getEntries( self ) :
        return self.entries

    def getEntry( self, aString ) :
        """
        Find a ComponentSpec by name.
        """
        for entry in self.entries :
            if entry.getName() is aString :
                return entry
        raise ResourceExcetion, 'error! cannot find ', aString

    # PRIVATE METHODS

    def mergeChildren( self, item, specArray ) :
        for child in specArray :
            if child[ 'info' ][ 'parent' ] is item[ 'name' ] :
                self.mergeContents( child, item )

    def mergeContents( self, child, parent ) :
        childEvents = child[ 'info' ][ 'events' ]
        parentEvents = parent[ 'info' ][ 'events' ]
        events = []
        for e in childEvents :
           events.append( e )
        for e in parentEvents :
           events.append( e )
        child[ 'info' ][ 'events' ] = events
        childAttributes = child[ 'info' ][ 'attributes' ]
        parentAttributes = parent[ 'info' ][ 'attributes' ]
        childAttributes.update( parentAttributes )
                

# Unit Test

if __name__ == '__main__' :

    #SpecDoc( 'spec.py', 'ComponentSpecTextFormat' )
    SpecDoc( 'spec.py', 'ComponentSpecHtmlFormat' )

    sys.exit(1)

    if len( sys.argv ) is 2 :
        s = None
        d = eval( open( sys.argv[ 1 ] ).read() )
    else :
        s = eval( open( sys.argv[ 1 ] ).read() )
        d = eval( open( sys.argv[ 2 ] ).read() )

    spec = Spec( s )
    
    print 'PasswordField spec = ', spec.getEntry( 'PasswordField' )[ 'info' ][ 'attributes' ]
    
    #print spec

    resource = Resource( d )

