#!/usr/bin/python

"""
__version__ = "$Revision: 1.3 $"
__date__ = "$Date: 2001/08/23 05:54:43 $"

Currently only displays Open topic items. Providing more options
and displaying the full info set from the tracker database is left
as an exercise for the reader. I did the hard part. ;-)
"""

import PythonCardPrototype
from PythonCardPrototype.loader import configOptions
import os, sys

from xml.parsers.expat import ParserCreate
import string
import urllib

BUGS = 'Bug Reports'
FEATURE_REQUESTS = 'Feature Requests'
TOPIC_SEPARATOR = "  :  "

class MyParser:
    def __init__(self):
        self.p = ParserCreate()
        #self.p.returns_unicode = 0
        self.p.StartElementHandler = self.start_element
        self.p.EndElementHandler = self.end_element
        self.p.CharacterDataHandler = self.char_data

        self.data = []
        self.trackerDict = {}
        self.dict = {}
        self.artifact = None

    def start_element(self, name, attrs):
        try:
            #print 'Start element:', name, attrs
            self.data = []
            if name == 'artifact':
                if self.artifact != None:
                    #print "adding", self.dict
                    self.trackerDict[self.artifact] = self.dict
                    self.dict = {}
                self.artifact = str(attrs['id'])
        except:
            pass
            
    def end_element(self, name):
        try:
            #print 'End element:', name
            if name == 'summary':
                s = string.join(self.data)
                # summary shouldn't have any tabs or newlines
                s.replace('\n', '')
                s.replace('\t', '')
                #self.summary = s
                #print "SUMMARY:", self.data
            elif name == 'detail':
                #print "DETAIL:", self.data
                s = string.join(self.data)
                s.strip()
                """
                # these didn't seem to do anything
                s.replace(u'\n', '\n')
                s.replace('\n ', '\n')
                s.replace(' \n', '\n')
                s.replace('\t', '')
                """
                #self.trackerDict[self.summary] = s
            else:
                s = string.join(self.data)
                s.replace('\n', '')
                s.replace('\t', '')
                #self.trackerDict[self.artifact]
            self.dict[name] = s
        except:
            pass

    def char_data(self, data):
        try:
            #print 'Character data:', repr(data)
            self.data.append(data)
        except:
            pass

    def cleanString(self, xml):
        """
        # none of this worked to replace the bogus Unicode characters
        # I finally just gave up and did a brute force deletion
        #print "len(xml)", len(xml)
        xml.replace(u'u\0013', '')
        xml.replace(u'u\1300', '')
        xml.replace(chr(0), '')
        xml.replace(chr(19), '')
        #print xml.find(hex(13))
        print "len(xml)", len(xml)
        """
        
        iTab = ord('\t')
        iNewline = ord('\n')
        offset = 0
        badChars = []
        for c in xml:
            i = ord(c)
            if (i > 31 and i < 128) or i == iTab or i == iNewline:
                pass
            else:
                badChars.append(offset)
                # ord(c) of the bogus characters returned 19 and 0
                # print i
            offset += 1
            
        #print badChars
        i = len(badChars) - 1
        while i + 1:
            offset = badChars[i]
            start = xml[:offset]
            end = xml[offset + 1:]
            xml = start + end
            #print i, len(xml)
            i = i - 1

        return xml


class Tracker(PythonCardPrototype.model.Background):

    def __init__(self, aParent, aId, aStackRsrc, aBgRsrc):
        PythonCardPrototype.model.Background.__init__(self, aParent, aId, aStackRsrc, aBgRsrc)

        self.baseSFUrl = 'http://sourceforge.net/export/sf_tracker_export.php?'
        self.groupIds = {'Boa':1909,
                         'PyChecker':24686, 'PyCrust':31263,
                         'Python': 5470, 'PythonCard':19015,
                         'Scintilla':2439,
                         'wxPython':10718
                         }
        self.categoryIds = {'Boa': {BUGS:101909, FEATURE_REQUESTS:351909},
                            'PyChecker': {BUGS:382217, FEATURE_REQUESTS:382220},
                            'PyCrust': {BUGS:401706, FEATURE_REQUESTS:401709},
                            'Python': {BUGS:105470, FEATURE_REQUESTS:355470},
                            'PythonCard': {BUGS:119015, FEATURE_REQUESTS:369015},
                            'Scintilla': {BUGS:102439, FEATURE_REQUESTS:352439},
                            'wxPython': {BUGS:310718, FEATURE_REQUESTS:360718}
                            }
        self.defaultGroup = 'PythonCard'
        self.defaultCategory = FEATURE_REQUESTS

        temp = self.groupIds.keys()
        temp.sort()
        self.components.choiceGroups.items = temp
        self.components.choiceGroups.selection = self.defaultGroup
        self.components.choiceCategories.selection = self.defaultCategory

        self.parser = None
        self.displayTopics(self.defaultGroup, self.defaultCategory)

    def status(self, txt):
        self.components.staticStatus.text = txt

    def buildUrl(self, group, category):
        groupId = self.groupIds[group]
        atId = self.categoryIds[group][category]
        return self.baseSFUrl + 'atid=' + str(atId) + '&group_id=' + str(groupId)

    def buildFilename(self, group, category):
        return group + '_' + category.replace(' ', '') + '.xml'

    def loadXML(self, group, category):
        filename = self.buildFilename(group, category)
        try:
            fp = open(filename, 'rb')
            xml = fp.read()
            fp.close()
            return xml
        except:
            return ''

    def doParse(self, xml):
        try:
            self.status('Parsing XML...')
            self.parser = MyParser()
            xml = self.parser.cleanString(xml)
            self.parser.p.Parse(xml)
        except EOFError:
            pass
        self.status('')

    def displayTopics(self, group, category):
        filename = self.buildFilename(group, category)
        #print filename
        try:
            if not os.path.exists(filename):
                #print "downloading", filename, url
                url = self.buildUrl(group, category)
                self.downloadFile(url, filename)
            xml = self.loadXML(group, category)
            #print xml[:60]
            self.doParse(xml)
            #print "done parsing"

            self.status('Display Topics...')
            topics = []
            for artifact in self.parser.trackerDict.keys():
                # handling other variations is left as an exercise for the reader
                #print artifact
                if self.parser.trackerDict[artifact]['status'] == 'Open':
                    topics.append(artifact + TOPIC_SEPARATOR + self.parser.trackerDict[artifact]['summary'])
            topics.sort()
            self.components.topicList.clear()
            for t in topics:
                self.components.topicList.append(t)
        except:
            pass
        self.status('')

    def downloadFile(self, url, filename):
        try:
            #print "url", url
            self.status("Downloading...")
            fp = urllib.urlopen(url)
            xml = fp.read()
            fp.close()
            #print "downloaded", url

            self.status("Writing file...")
            #print filename
            op = open(filename, 'wb')
            op.write(xml)
            op.close()
            #print "wrote", filename
        except:
            pass
            # show a warning dialog one of these days
        self.status('')

    def on_choiceGroups_select(self, target, event):
        group = target.selection
        category = self.components.choiceCategories.selection
        self.displayTopics(group, category)

    def on_choiceCategories_select(self, target, event):
        group = self.components.choiceGroups.selection
        category = target.selection
        self.displayTopics(group, category)

    def on_topicList_select(self, target, event):
        artifact, summary = target.getStringSelection().split(TOPIC_SEPARATOR)
        txt = self.parser.trackerDict[str(artifact)]['detail']
        # need to change the parsing to get all the follow_ups parsed correctly
        if self.parser.trackerDict[str(artifact)].has_key('follow_ups'):
            txt += "\n\n\n\nFollow-ups:\n\n" + self.parser.trackerDict[str(artifact)]['follow_ups']
        self.components.topicDetail.text = txt

    def on_buttonDownload_mouseClick(self, button, event):
        group = self.components.choiceGroups.selection
        category = self.components.choiceCategories.selection
        filename = self.buildFilename(group, category)
        url = self.buildUrl(group, category)
        #print filename
        #print url
        self.downloadFile(url, filename)
        self.displayTopics(group, category)

    def on_menuFileExit_select(self, menu, event):
        self.Close()


if __name__ == '__main__':
    base, ext = os.path.splitext(sys.argv[0])
    filename = base + ".rsrc.py"

    configOptions()

    app = PythonCardPrototype.model.PythonCardApp(filename)
    app.MainLoop()

