#!/usr/bin/python

"""
__version__ = "$Revision: 1.1 $"
__date__ = "$Date: 2001/08/11 06:08:53 $"
"""

import PythonCardPrototype
from PythonCardPrototype.loader import configOptions
import os, sys


class Conversion:
    def __init__(self, components):
        pass

    def toDown(self, value):
        pass

    def toUp(self, value):
        pass
    
class TemperatureConversion(Conversion):
    def __init__(self, components):
        components.labelUp.text = 'Fahrenheit'
        components.btnConvertUp.label = 'Celsius to Fahrenheit'
        components.btnConvertDown.label = 'Fahrenheit to Celsius'
        components.labelDown.text = 'Celsius'

    def toDown(self, degrees):
        return str(self.FahrenheitToCelsius(float(degrees)))

    def toUp(self, degrees):
        return str(self.CelsiusToFahrenheit(float(degrees)))

    def FahrenheitToCelsius(self, degrees):
        return (degrees - 32.0) / 9.0 * 5.0

    def CelsiusToFahrenheit(self, degrees):
        return degrees * 9.0 / 5.0 + 32.0

class MorseCodeConversion(Conversion):
    def __init__(self, components):
        """
        A B C D E F G
        H I J K L M N
        O P Q R S T U
        V W X Y Z
        """
        self.morseAlphabet = ['.-', '-...', '-.-.', '-..', '.', '..-.', '--.',
                              '....', '..', '.---', '-.-', '.-..', '--', '-.',
                              '---', '.--.', '--.-', '.-.', '...', '-', '..-',
                              '...-', '.--', '-..-', '-.--', '--..']
        
        components.labelUp.text = 'English'
        components.btnConvertUp.label = 'Morse code to English'
        components.btnConvertDown.label = 'English to Morse code'
        components.labelDown.text = 'Morse code'

    def toDown(self, txt):
        return self.convertToMorse(txt)

    def toUp(self, txt):
        return self.convertFromMorse(txt)

    def convertToMorse(self, txt):
        a = ord('A')
        z = ord('Z')
        converted = ''
        for c in txt[:]:
            ordC = ord(c.upper())
            if c == ' ':
                # three spaces between words
                # when you include the space after each character
                converted = converted + '  '
            elif ordC < a or ordC > z:
                return converted + "\n\ncharacter out of bounds, unable to complete conversion"
            else:
                converted += self.morseAlphabet[ordC - a] + ' '
        return converted[:-1]

    def convertFromMorse(self, txt):
        a = ord('A')
        converted = ''
        words = txt.split('  ')
        for w in words:
            letters = w.split(' ')
            for c in letters:
                if c == '':
                    continue
                try:
                    ordC = self.morseAlphabet.index(c)
                except:
                    return converted + "\n\nmorse out of bounds error, unable to complete conversion"
                converted += chr(ordC + a)
            converted += ' '
        return converted

class Conversions(PythonCardPrototype.model.Background):
    def __init__(self, aParent, aId, aStackRsrc, aBgRsrc):
        PythonCardPrototype.model.Background.__init__(self, aParent, aId, aStackRsrc, aBgRsrc)
        self.conversion = TemperatureConversion(self.components)

    def on_btnConvertDown_mouseClick(self, button, event):
        self.components.field2.text = self.conversion.toDown(self.components.field1.text)

    def on_btnConvertUp_mouseClick(self, button, event):
        self.components.field1.text = self.conversion.toUp(self.components.field2.text)
        
    def on_menuConvertMorseCode_select(self, menu, event):
        self.conversion = MorseCodeConversion(self.components)

    def on_menuConvertTemperature_select(self, menu, event):
        self.conversion = TemperatureConversion(self.components)

    def on_menuFileExit_select(self, menu, event):
        self.Close()


if __name__ == '__main__':
    base, ext = os.path.splitext(sys.argv[0])
    filename = base + ".rsrc.py"

    configOptions()

    app = PythonCardPrototype.model.PythonCardApp(filename)
    app.MainLoop()
