#!/usr/bin/python

"""
__version__ = "0.1"
__date__ = "20th August 2001"
__author__ = "Andy Todd <andy47@crosswinds.net>"

App Name: dbBrowser
Description: The first useful application what I wrote

This is designed to be a sample application which will allow you to browse 
database tables.
"""
import PythonCardPrototype
from PythonCardPrototype.loader import configOptions
import os, sys
from mySQLBrowse import *
"""
AJT 20.08.2001
The use of mySQLBrowse is just for this alpha sample. In the next version
we should determine the type of database to connect to (from the sign on 
screen) and import the appropriate handler.
"""

browsingButtons=['btnFirstRow', 'btnPreviousRow', 'btnNextRow', 'btnLastRow']

class DbBrowser(PythonCardPrototype.model.Background):
    def __init__(self, aParent, aId, aStackRsrc, aBgRsrc):
        PythonCardPrototype.model.Background.__init__(self, aParent, aId, aStackRsrc, aBgRsrc)
        # Bind the appropriate database class. Quite easy in the first instance
        # as we are only supporting MySQL
        self._dbClass=mySQLBrowse

    def _buttonEnabled(self, enabled):
        "Hide or show the browsing control buttons"
        for button in browsingButtons:
            self.components[button].enabled=enabled

    def disableButtons(self):
        "Hide the browsing control buttons"
        self._buttonEnabled(0)

    def enableButtons(self):
        "Show the browsing control buttons"
        self._buttonEnabled(1)

    def getARow(self, selectedTable):
        "Get a row from the table and put the values in our widgets"
        row=self._database.getRow(selectedTable)
        if row:
            self._row.append(row)
            self.showRow(self._index)
        else:
            self._index-=1

    def showRow(self, index):
        "Show row number <index>"
        widgetCount=0
        for item in self._widgets:
            # All of our widgets are text, watch out for those date conversions!
            self.components[item].text=str(self._row[index][widgetCount])
            widgetCount+=1

    def OnCloseWindow(self, event):
        # Common exit code - should close DB connection here
        # In the meantime, just call the parent method
        PythonCardPrototype.model.Background.OnCloseWindow(self, event)

    def on_btnBrowse_mouseClick(self, button, event):
        selectedTable=self.components["chsTables"].selected
        if selectedTable:
            if hasattr(self, "_widgets"):
                # Remove the currently painted widgets
                # This is a complete bodge and will be replaced in version x
                for i in self._widgets:
                    del self.components[i]
                    del self.components['lbl'+i[3:]]
            self._widgets=[] # This will hold the names of the dynamically created widgets
            rowHeight=25
            xPos, yPos=1, 106
            # Get all of the columns for this table and create a widget for each
            # NB this should really be put into a seperate function
            for columns in self._database.getColumns(selectedTable):
                # Column name are of the form 'word_word_word', we will 
                # slightly change this to 'WordWordWord'
                columnName=columns[0].title().replace("_", "")
                columnTitle=columns[0]
                # Second item is column type and precision, eg 'int(4)' 
                # oh, and ignore anything past the ',' as thats just the number
                # of digits past the decimal point.
                columnType=columns[1].split("(")[0]
                if columnType!="date":
                    columnPrecision=int(columns[1].split("(")[1][:-1].split(",")[0])
                else:
                    columnPrecision=10 # 2001-08-04 is the default format
                # Form a widget per column, as well as some boilerplate
                widgetDefn={'type':'StaticText',
                            'name':'lbl'+columnName,
                            'position':(xPos, yPos),
                            'text':columnTitle}
                self.components[widgetDefn['name']]=widgetDefn
                xPos += len(columnTitle)*6 # replace 6 with a 'constant'
                widgetDefn={'type':'TextField',
                            'name':'txt'+columnName,
                            'position':(xPos, yPos),
                            'text':'',
                            'size':(columnPrecision*8, -1)} # replace 8 with a constant
                self.components[widgetDefn['name']]=widgetDefn
                self._widgets.append('txt'+columnName)
                xPos += columnPrecision*8 # replace 8 with a 'constant'
                xPos += 10 # leave a gap between the end of this field and the next bit of boilerplate
                if xPos > 400:
                    xPos=1
                    yPos+=26
            """ 
            Now that we've painted the screen initialise the cursor and get 
            the first row. Note that this assumes that the columns returned 
            by getRow are in the same order as the call to getColumns above
            """
            self._index=0
            self._row=[]
            self.getARow(selectedTable)
            self.enableButtons()

    def on_btnConnect_mouseClick(self, button, event):
        """
        Database connection is a dictionary so we can adapt the members
        depending on the RDBMS we select (in v1.0, natch)
        """
        connection={'username':self.components["txtUsername"].text,
                    'password':self.components["txtPassword"].text,
                    'database':self.components["txtDatabase"].text}
        # Get the choice widget and add the available database tables to it
        self._database=self._dbClass(connection)
        if self._database:
            self.components['chsTables'].items=self._database.getTables()

    def on_btnNextRow_mouseClick(self, button, event):
        "Get the next row from the database"
        length=len(self._row)-1
        if self._index<length:
            self._index+=1
            self.showRow(self._index)
        elif self._index==length:
            selectedTable=self.components["chsTables"].selected
            self._index+=1
            self.getARow(selectedTable)

    def on_btnPreviousRow_mouseClick(self, button, event):
        "Show the previous row"
        if self._index:
            if self._index>len(self._row):
                self._index-=1
            self._index-=1
            self.showRow(self._index)

    def on_btnFirstRow_mouseClick(self, button, event):
        "Show the first row (if we've already got it)"
        self._index=0
        self.showRow(self._index)

    def on_btnLastRow_mouseClick(self, button, event):
        "Show the last row we have fetched so far"
        if self._index<(len(self._row)-1):
            self._index=(len(self._row)-1)
            self.showRow(self._index)

    def on_mnuExit_select(self, menu, event):
        self.Close()

if __name__=="__main__":
    base, ext=os.path.splitext(sys.argv[0])
    filename=base+".rsrc.py"
    configOptions()
    app=PythonCardPrototype.model.PythonCardApp(filename)
    app.MainLoop()
