#!/usr/bin/python



"""

__version__ = "0.1"

__date__ = "13th August 2001"

__author__ = "Andy Todd <andy47@crosswinds.net>"



App Name: dbBrowse

Description: Plug in for PythonCard application dbBrowse to provide MySQL specific functionality



AJT Notes to myself

I've used the standard convention for function names. I'm not an OO guru so all I've done is encapsulate the functions I have written as a class. This may change as I get to grips with _proper_ object oriented design.



Constant/configuration values are currently maintained in the source code. If we are to optimise this application they should be split into seperate configuration files (as per PythonCard/Webware style guidelines)



The structure of this module should be replicated for different RDBMS so that they can be interchanged by dbBrowse - hopefully.



To Do 14.08.2001;

Date columns - Change getQueryString to format DATE columns to DD-MON-YYYY

  select date_format(<date column>, '%d-%b-%Y')

"""

import MySQLdb



class mySQLBrowse:



    # Connection should be a dictionary with at least three keys, 'username',

    # 'password', 'database' - may need to be normalised for other RDBMS

    def __init__(self, connection):

        "Setup the database connection"

        self._system_tables=['columns_priv', 'db', 'host', 'tables_priv', 'user']

        db = MySQLdb.connect( user=connection['username']

                             ,passwd=connection['password']

                             ,db=connection['database'] )

        self._cursor=db.cursor()

        # This one is used in getRow

        self._tableName=''



    def getTables(self):

        "Return a list of all of the non-system tables in <database>"

        stmt = 'show tables'

        self._cursor.execute(stmt)

        # I'm using a list comprehension here instead of a for loop,

        # either will do but I think this is more concise (unlike this comment)

        return [ x[0] for x in self._cursor.fetchall() if x[0] not in self._system_tables ]



    def getColumns(self, tableName):

        "Get the definition of the columns in tableName"

        stmt = 'describe ' + tableName

        self._cursor.execute(stmt)

        # format of MySQL definitions is;

        #  column name,type (incl length & precision),null,key,default,extra

        return self._cursor.fetchall()



    def getQueryString(self, tableName):

        "Return a SQL statement which queries all of the columns in tableName"

        tableStructure=self.getColumns(tableName)

        # Construct and return the string

        stmt='SELECT '

        for columnList in tableStructure:

            stmt+=columnList[0]+', '

        stmt=stmt[:-2]+' FROM '+tableName

        return stmt



    def getRow(self, tableName):

        "Get a row from tableName"

        # When we upgrade to 2.2 this will be a great candidate for a

        # generator/iterator. In the meantime we use self._tableName to keep

        # track of what we are doing

        if tableName!=self._tableName:

            self._tableName=tableName

            self._cursor.execute(self.getQueryString(tableName))

        return self._cursor.fetchone()



if __name__ == '__main__':

    # We are in an interactive session so run our test routines

    # Connect to the database

    connection={ 'username':'andy'

                ,'password':'andy'

                ,'database':'mysql'}

    dbHolder = mySQLBrowse(connection)

    # Return all of our table names into user_tables

    user_tables = dbHolder.getTables()



    # Print out the structure of each table and its first row

    print "--------------------------------------------------"

    for table in user_tables:

        print dbHolder.getQueryString(table)

        print dbHolder.getRow(table)

        print "--------------------------------------------------"

