#!/usr/bin/python

"""
__version__ = "$Revision: 1.11 $"
__date__ = "$Date: 2001/08/23 23:51:40 $"
"""

import PythonCardPrototype
from PythonCardPrototype.loader import configOptions
from PythonCardPrototype import dialog
import os, sys
from wxPython.wx import *
import types
import pprint

SIZING_HANDLE_SIZE = 7

class ResourceEditor(PythonCardPrototype.model.Background):
    
    def __init__(self, aParent, aId, aStackRsrc, aBgRsrc):
        PythonCardPrototype.model.Background.__init__(self, aParent, aId, aStackRsrc, aBgRsrc)

        self.cursors = {}
        self.cursors['topRight'] = wxStockCursor(wxCURSOR_SIZENESW)
        self.cursors['bottomLeft'] = self.cursors['topRight']
        self.cursors['topMiddle'] = wxStockCursor(wxCURSOR_SIZENS)
        self.cursors['bottomMiddle'] = self.cursors['topMiddle']
        self.cursors['topLeft'] = wxStockCursor(wxCURSOR_SIZENWSE)
        self.cursors['bottomRight'] = self.cursors['topLeft']
        self.cursors['middleLeft'] = wxStockCursor(wxCURSOR_SIZEWE)
        self.cursors['middleRight'] = self.cursors['middleLeft']
        self.cursors['general'] = wxStockCursor(wxCURSOR_SIZING)
        self.cursors['null'] = wxNullCursor
 
        self.sizingHandleNames = ['topLeft', 'topMiddle', 'topRight',
                         'middleLeft', 'middleRight',
                         'bottomLeft', 'bottomMiddle', 'bottomRight']
        
        sizingHandleTemplate = {'type':'ImageButton', 'name':'topLeft', 'position':(3, 3),
                       'size':(SIZING_HANDLE_SIZE, SIZING_HANDLE_SIZE),
                       'file':'sizingHandle.bmp', 'border':'none', 'visible':0}
        
        for sizingHandle in self.sizingHandleNames:
            sizingHandleTemplate['name'] = sizingHandle
            self.components[sizingHandle] = sizingHandleTemplate

        self.startName = ''
        self.startPosition = (0, 0)
        self.startSize = (0, 0)
        self.offset = (0, 0)
        
        try:
            self.readme = open('readme.txt').read()
        except:
            self.readme = ''     

    def clearWidgets(self):
        for w in self.components.keys():
            if w not in self.sizingHandleNames:
                del self.components[w]

    def positionSizingHandles(self, position, size):
        x, y = position
        width, height = size
        halfHandleSize = SIZING_HANDLE_SIZE / 2
        
        self.components['topLeft'].position = (x - SIZING_HANDLE_SIZE, y - SIZING_HANDLE_SIZE)
        self.components['topMiddle'].position = (x + (width / 2) - halfHandleSize, y - SIZING_HANDLE_SIZE)
        self.components['topRight'].position = (x + width, y - SIZING_HANDLE_SIZE)

        self.components['middleLeft'].position = (x - SIZING_HANDLE_SIZE, y + (height / 2) - halfHandleSize)
        self.components['middleRight'].position = (x + width, y + (height / 2) - halfHandleSize)

        self.components['bottomLeft'].position = (x - SIZING_HANDLE_SIZE, y + height)
        self.components['bottomMiddle'].position = (x + (width / 2) - halfHandleSize, y + height)
        self.components['bottomRight'].position = (x + width, y + height)

    def showSizingHandles(self, name):
        self.positionSizingHandles(self.components[name].position, self.components[name].size)
        for sizingHandle in self.sizingHandleNames:
            self.components[sizingHandle].visible = 1

    def hideSizingHandles(self):
        if self.components.topLeft.visible:
            for sizingHandle in self.sizingHandleNames:
                self.components[sizingHandle].visible = 0


    def setToolTip(self, target):
        x, y = target.position
        width, height = target.size
        tip = "name: " + target.name + "\n" + \
              "position: " + str(x) + ", " + str(y) + "\n" + \
              "size: " + str(width) + ", " + str(height)
        if tip != target.toolTip:
            target.toolTip = tip

    def on_bgDrag_mouseEnter(self, target, event):
        name = target.name
        if name in self.sizingHandleNames:
            try:
                self.SetCursor(self.cursors[name])
            except:
                # not all platforms have all cursors, but hopefully
                # the general sizing cursor is always available
                self.SetCursor(self.cursors['general'])
        else:
            self.setToolTip(target)

    def on_bgDrag_mouseLeave(self, target, event):
        if target.name in self.sizingHandleNames:
            self.SetCursor(self.cursors['null'])


    def on_bgDrag_mouseDown(self, target, event):
        if target.name not in self.sizingHandleNames:
            self.startName = target.name
            if self.stack.app.pw != None:
                self.stack.app.pw.selectComponentsList(target.name, str(target.__class__.__name__))
            
        self.hideSizingHandles()
        self.startPosition = self.components[self.startName].position
        self.startSize = self.components[self.startName].size
        self.offset = event.getNativeEvent().GetPosition()
            
    def on_bgDrag_mouseDrag(self, target, event):
        # protect against double-clicks in the open file dialog
        # when switching rsrc.py files
        if target.name not in self.sizingHandleNames and self.components.has_key(self.startName):
            self.hideSizingHandles()
            x, y = target.position
            xOffset = x + event.getNativeEvent().GetX() - self.offset[0]
            yOffset = y + event.getNativeEvent().GetY() - self.offset[1]
            self.components[self.startName].position = (xOffset, yOffset)
            self.setToolTip(self.components[self.startName])

    def on_bgDrag_mouseUp(self, target, event):
        # protect against double-clicks in the open file dialog
        # when switching rsrc.py files
        if self.components.has_key(self.startName):
            self.showSizingHandles(self.startName)
            self.setToolTip(self.components[self.startName])


    def doTopLeftResize(self, target, event):
        xOffset = event.getNativeEvent().GetX() - self.offset[0]
        yOffset = event.getNativeEvent().GetY() - self.offset[1]
        xStartOffset = self.startPosition[0] + event.getNativeEvent().GetX() - self.offset[0]
        yStartOffset = self.startPosition[1] + event.getNativeEvent().GetY() - self.offset[1]
        width = self.startSize[0] - xOffset
        height = self.startSize[1] - yOffset
        self.components[self.startName].position = (xStartOffset, yStartOffset)
        self.components[self.startName].size = (width, height)

    def on_topLeft_mouseDrag(self, target, event):
        self.doTopLeftResize(target, event)
        

    def doTopMiddleResize(self, target, event):
        yOffset = event.getNativeEvent().GetY() - self.offset[1]
        yStartOffset = self.startPosition[1] + event.getNativeEvent().GetY() - self.offset[1]
        width = self.startSize[0]
        height = self.startSize[1] - yOffset
        self.components[self.startName].position = (self.startPosition[0], yStartOffset)
        self.components[self.startName].size = (width, height)
    
    def on_topMiddle_mouseDrag(self, target, event):
        self.doTopMiddleResize(target, event)


    def doTopRightResize(self, target, event):
        xOffset = event.getNativeEvent().GetX() - self.offset[0]
        yOffset = event.getNativeEvent().GetY() - self.offset[1]
        yStartOffset = self.startPosition[1] + event.getNativeEvent().GetY() - self.offset[1]
        width = self.startSize[0] + xOffset
        height = self.startSize[1] - yOffset
        self.components[self.startName].position = (self.startPosition[0], yStartOffset)
        self.components[self.startName].size = (width, height)

    def on_topRight_mouseDrag(self, target, event):
        self.doTopRightResize(target, event)


    def doMiddleLeftResize(self, target, event):
        xOffset = event.getNativeEvent().GetX() - self.offset[0]
        xStartOffset = self.startPosition[0] + event.getNativeEvent().GetX() - self.offset[0]
        width = self.startSize[0] - xOffset
        height = self.startSize[1]
        self.components[self.startName].position = (xStartOffset, self.startPosition[1])
        self.components[self.startName].size = (width, height)

    def on_middleLeft_mouseDrag(self, target, event):
        self.doMiddleLeftResize(target, event)

    def doMiddleRightResize(self, target, event):
        xOffset = event.getNativeEvent().GetX() - self.offset[0]
        width = self.startSize[0] + xOffset
        height = self.startSize[1]
        self.components[self.startName].size = (width, height)

    def on_middleRight_mouseDrag(self, target, event):
        self.doMiddleRightResize(target, event)
        

    def doBottomLeftResize(self, target, event):
        xOffset = event.getNativeEvent().GetX() - self.offset[0]
        yOffset = event.getNativeEvent().GetY() - self.offset[1]
        xStartOffset = self.startPosition[0] + event.getNativeEvent().GetX() - self.offset[0]
        width = self.startSize[0] - xOffset
        height = self.startSize[1] + yOffset
        self.components[self.startName].position = (xStartOffset, self.startPosition[1])
        self.components[self.startName].size = (width, height)

    def on_bottomLeft_mouseDrag(self, target, event):
        self.doBottomLeftResize(target, event)
        

    def doBottomMiddleResize(self, target, event):
        yOffset = event.getNativeEvent().GetY() - self.offset[1]
        width = self.startSize[0]
        height = self.startSize[1] + yOffset
        self.components[self.startName].position = (self.startPosition[0], self.startPosition[1])
        self.components[self.startName].size = (width, height)
    
    def on_bottomMiddle_mouseDrag(self, target, event):
        self.doBottomMiddleResize(target, event)
        

    def doBottomRightResize(self, target, event):
        xOffset = event.getNativeEvent().GetX() - self.offset[0]
        yOffset = event.getNativeEvent().GetY() - self.offset[1]
        width = self.startSize[0] + xOffset
        height = self.startSize[1] + yOffset
        self.components[self.startName].size = (width, height)

    def on_bottomRight_mouseDrag(self, target, event):
        self.doBottomRightResize(target, event)


    def on_menuFileOpen_select(self, menu, event):
        wildcard = "resource files (*.rsrc.py)|*.rsrc.py"
        dlg = dialog.FileDialog(None, "Import which resource file?", '', '', wildcard )
        if dlg.accepted():
            self.startName = ''
            self.startPosition = (0, 0)
            self.startSize = (0, 0)
            self.offset = (0, 0)
            self.stack.app.pw.selectComponentsList('topLeft', 'ImageButton')
            self.hideSizingHandles()
            self.clearWidgets()

            path = dlg.getPaths()[0]
            resource = PythonCardPrototype.res.ResourceFile(path).getResource()
            self.setImage(resource.stack.backgrounds[0].image)
            self.setTiled(resource.stack.backgrounds[0].tiled)
            self.panel._imageFile = self.image
            self.panel._backgroundTiling = self.tiled

            defaultBgColor = wxSystemSettings_GetSystemColour(wxSYS_COLOUR_3DLIGHT)
            self.setBackgroundColor(defaultBgColor)
            self.setForegroundColor(resource.stack.backgrounds[0].foregroundColor)
            self.setBackgroundColor(resource.stack.backgrounds[0].backgroundColor)
            if self.image is not None :
                self.panel._bitmap = PythonCardPrototype.widget.Bitmap(self.image)
                EVT_ERASE_BACKGROUND( self.panel, self.panel.onEraseBackground )
            else:
                self.panel.Disconnect(-1, -1, wxEVT_ERASE_BACKGROUND)
                self.panel._bitmap = None

            """
            try:
                self.SetPosition(resource.stack.position)
            except:
                pass
            """
            try:
                self.SetSize(resource.stack.size)
            except:
                pass
            
            for w in resource.stack.backgrounds[0].components:
                self.components[w.name] = w


    def on_menuFileExit_select(self, menu, event):
        self.Close()

    def on_menuHelpAbout_select(self, menu, event):
        dialog = PythonCardPrototype.dialog.ScrolledMessageDialog(self, self.readme, 'About resourceEditor...')


    def on_editDuplicateWidget_command(self, target, event):
        if self.components.has_key(self.startName):
            widget = self.components[self.startName]
            dlg = dialog.TextEntryDialog(self, 'Duplicate ' + self.startName,
                                         'Name for copy:',
                                         self.startName + 'Copy')
            if dlg.accepted():
                name = dlg.getText()
                if self.components.has_key(name):
                    dlg = dialog.AlertDialog(self, name + " already exists", 'Error: Unable to duplicate widget')
                else:
                    # now loop through the original widget and build a dictionary suitable
                    # for making a copy
                    d = {}
                    d['type'] = widget.__class__.__name__
                    for key in widget._getAttributeNames():
                        # I'm not exactly sure why I have to special-case these tuples
                        if key == 'bitmap':
                            # this should get recreated from the file attribute
                            pass
                        elif key in ['position', 'size']:
                            d[key] = getattr(widget, key)
                        elif getattr(widget, key) != None:
                            d[key] = getattr(widget, key)
                    d['name'] = name
                    d['toolTip'] = ''
                    #print d
                    
                    # a lot of this is common to add widget below
                    # so refactor so the common code is in a method
                    self.components[name] = d
                    if self.stack.app.pw != None:
                        self.stack.app.pw.selectComponentsList(name, str(d['type']))

                    self.startName = name
                    self.startPosition = self.components[self.startName].position
                    self.startSize = self.components[self.startName].size
                    self.offset = (0, 0)
                    self.showSizingHandles(name)


    def on_editDeleteWidget_command(self, target, event):
        if self.components.has_key(self.startName):
            widget = self.components[self.startName]
            msg = "Are you sure you want to delete %s %s?" % (widget.__class__.__name__, widget.name)
            dlg = dialog.MessageDialog(self, msg, 'Delete Widget',
                                       dialog.ICON_INFORMATION)
            if dlg.accepted():
                self.hideSizingHandles()
                del self.components[widget.name]

    def on_editAddWidget_command(self, target, event):
        className = target.name[11:]
        baseDesc = "{'type':'" + className + "', 'name':'genericName', 'position':(10, 10), 'size':(-1, -1), "
        if className == 'Button':
            desc = baseDesc + "'label':'Button n'}"
        elif className == 'CheckBox':
            desc = baseDesc + "'label':'CheckBox n', 'checked':0}"
        elif className == 'Choice':
            desc = baseDesc + "'items':[], 'selected':''}"
        elif className == 'Image':
            desc = baseDesc + "'file':''}"
        elif className == 'ImageButton':
            desc = baseDesc + "'file':''}"
        elif className == 'List':
            desc = baseDesc + "'items':[], 'selected':''}"
        elif className == 'PasswordField':
            desc = baseDesc + "'text':''}"
        elif className == 'RadioGroup':
            desc = baseDesc + "'label':'RadioBox n', 'items':[], 'selected':'', 'layout':'horizontal'}"
        elif className == 'Slider':
            desc = baseDesc + "'min':1, 'max':100, 'value':1, 'layout':'horizontal'}"
        elif className == 'StaticLine':
            desc = baseDesc + "'layout':'horizontal'}"
        elif className == 'StaticText':
            desc = baseDesc + "'text':''}"
        elif className == 'TextArea':
            desc = baseDesc + "'text':''}"
        elif className == 'TextField':
            desc = baseDesc + "'text':''}"
        else:
            return

        dlg = dialog.TextEntryDialog(self, 'Add ' + className,
                                     'Description:',
                                     desc)
        if dlg.accepted():
            #print dlg.getText()
            try:
                d = eval(dlg.getText())
                name = d['name']
                if self.components.has_key(name):
                    dlg = dialog.AlertDialog(self, name + " already exists", 'Error: Unable to add widget')
                else:
                    self.components[name] = d
                    if self.stack.app.pw != None:
                        self.stack.app.pw.selectComponentsList(name, str(d['type']))

                    self.startName = name
                    self.startPosition = self.components[self.startName].position
                    self.startSize = self.components[self.startName].size
                    self.offset = (0, 0)
                    self.showSizingHandles(name)

            except:
                # GIGO, bad monkey, no banana
                pass


    def widgetAttributes(self, widget):
        # make sure these primary attributes show up
        # at the beginning of the listing
        # the remaining ones will be in alphabetical order
        dStr = "{'type':'%s', \n" % widget.__class__.__name__
        dStr += "    'name':'%s', \n" % widget.name
        dStr += "    'position':%s, \n" % getattr(widget, 'position')
        dStr += "    'size':%s, \n" % getattr(widget, 'size')
        for key in widget._getAttributeNames():
            if key in ['bitmap', 'name', 'position', 'selection', 'size', 'toolTip']:
                pass
            elif getattr(widget, key) != None:
                value = getattr(widget, key)
                if type(value) == types.StringType:
                    # need to escape strings
                    #pprint.pprint(value)
                    if value != '':
                        dStr += "    '%s':%s, \n" % (key, repr(value))
                else:
                    if key in ['editable', 'enabled', 'visible'] and value == 1:
                        # don't include default values
                        pass
                    else:
                        dStr += "    '%s':%s, \n" % (key, value)
        dStr += '    },\n'
        return dStr
    
    def on_displayAttributes_command(self, target, event):
        widgetList = ""
        widgetList = "'position':%s,\n'size':%s\n\n" % (str(self.GetPosition()), str(self.GetSize()))
        for w in self.components.keys():
            if w not in self.sizingHandleNames:
                widgetList += self.widgetAttributes(self.components[w]) + "\n"
        #print widgetList
        dialog = PythonCardPrototype.dialog.ScrolledMessageDialog(self, str(widgetList), 'Attributes')

    def buildResourceFileDescription(self):
        pass
    

if __name__ == '__main__':
    base, ext = os.path.splitext(sys.argv[0])
    filename = base + ".rsrc.py"

    configOptions()
    # now force the property editor to be enabled
    PythonCardPrototype.config.Configuration().setOption('showPropertyEditor', 1)

    app = PythonCardPrototype.model.PythonCardApp(filename)
    app.MainLoop()
