from math import * # Also for export
from types import *

class aPen:
    def __init__(self):
        """ this should be called after a subclass has done its init """
        self._tracing = 0
        # self._tracing = 1 to showTurtle, MUCH SLOWER on tk !!!
        self._degrees()

        # this caused some problems when reset() in the subclass tried
        # to call its superclass aPen.reset() so I probably wasn't doing
        # the method calls correctly
        # until I figure it out, I put all the needed code of reset() into
        # the subclass
        self.reset()
       
    def distance(self, t2):
        """returns the distance between two turtles"""
        return sqrt(pow(self._position[0] - t2._position[0], 2) + 
            pow(self._position[1] - t2._position[1], 2))

    def distanceXY(self, x, y):
        """returns the distance between turtle (self) and a point (x, y)"""
        return sqrt(pow(self._position[0] - x, 2) + pow(self._position[1] - y, 2))

    def _degrees(self, fullcircle=360.0):
        self._fullcircle = fullcircle
        self._invradian = pi / (fullcircle * 0.5)

	# illegal syntax, so I used None and the if tests instead
    #def plot(self, x=self._position[0], y=self._position[1]):
    def plot(self, x=None, y=None):
        """ override """
        pass

    # may have to do multiple versions of methods if case-sensitivity
    # is an issue
    def dot(self, x, y):
        """Logo plot, should only be 1 pixel wide though"""
        self.plot(x, y)
    # there are also setX and setY, but I don't remember if they do a plot or a moveTo...

    def getXY(self):
        """returns the turtle position"""
        return self._position

    # this may actually be the Logo plot command rather than
    # the equivelant of a moveTo
    # need to change these methods so they can handle
    # x, y or a tuple (x, y)
    def setXY(self, pos):
        """another name for moveTo
        this may actually be the Logo plot command rather than the
        equivelant of a moveTo"""
        self.moveTo(pos[0], pos[1])

    def getHeading(self):
        """returns the turtle heading in degrees"""
        return self._angle

    def setHeading(self, angle):
        """set the turtle heading in degrees"""
        if self._dirty:
            self._drawTurtle()
        self._angle = angle % self._fullcircle
        if self._visible:
            self._drawTurtle()

    # my math is shaky, there has to be a shorter
    # way to return an angle between 0 - 360
    # remember however that 0,0 is the top-left corner
    # and positive x, positive y is the bottom-right
    # not positive x, negative y as in normal cartesian coordinates
    # THIS WAS HACKED TOGETHER SO IT COULD VERY WELL BE BUGGY
    def towardsXY(self, x, y):
        """returns the angle the turle would need to heading towards
        to face the position x, y"""
        xDir = x - self._position[0]
        yDir = y - self._position[1]
        try:
            angle = atan(yDir / xDir) * 180 / pi
        except:
            # should occur when xDir == 0.0 (divide by zero)
            if yDir == 0.0:
                return 0.0
            elif yDir < 0:
                return 90.0
            else:
                return 270.0
        if xDir < 0:
            angle = 180 - angle
        elif yDir < 0:
            angle = -1.0 * angle
        elif yDir > 0:
            angle = 360.0 - angle
        return angle

    def towards(self, t):
        """equivelant to towardsXY
        returns the angle the turle would need to heading towards
        to face the position x, y"""
        return self.towardsXY(t._position[0], t._position[1])

    def _radians(self):
        self._degrees(2.0*pi)

    def forward(self, distance):
        """moves the turtle forward distance"""
        x0, y0 = start = self._position
        x1 = x0 + distance * cos(self._angle*self._invradian)
        y1 = y0 - distance * sin(self._angle*self._invradian)
        self._goto(x1, y1)
        
    def backward(self, distance):
        """moves the turtle backward distance"""
        self.forward(-distance)

    def left(self, angle):
        """turns the turtle left angle degrees"""
        if not self._drawingTurtle and self._dirty:
            self._drawTurtle()
        self._angle = (self._angle + angle) % self._fullcircle
        #print self._angle
        if not self._drawingTurtle and self._visible:
            self._drawTurtle()

    def right(self, angle):
        """turns the turtle right angle degrees"""
        self.left(-angle)

    # is there an equivelant, but more efficient manner
    # of specifying these abbreviations?
    def fd(self, distance):
        """equivelant to forward
        moves the turtle forward distance"""
        self.forward(distance)      # logo forward
    def back(self, distance):
        """equivelant to backward
        moves the turtle backward distance"""
        self.forward(-distance)   # logo backward
    def bk(self, distance):
        """equivelant to backward
        moves the turtle backward distance"""
        self.forward(-distance)     # logo backward
    def lt(self, angle):
        """equivelant to left
        turns the turtle left angle degrees"""
        self.left(angle)               # logo left
    def rt(self, angle):
        """equivelant to right
        turns the turtle right angle degrees"""
        self.left(-angle)              # logo right
    
    def polygon(self, sides, distance):
        """draws a polygon with sides of length distance
        (e.g. polygon(4, 100) draws a square)
        polygon uses the forward and right methods to move the turtle"""
        angle = 360.0 / sides
        for i in range(sides):
            self.forward(distance)
            self.right(angle)

    def cPolygon(self, sides, radius):
        """draws a polygon centered on the current turtle position
        (e.g. cPolygon(4, 100) draws a square)
        the radius determines the size of the polygon, so that
        a circle with the same radius would intersect the vertices
        cPolygon uses the forward and right methods to move the turtle"""
        self.pu()
        self.fd(radius)
        angle = 180.0 - (90.0 * (sides - 2) / sides)
        self.right(angle)   # must be the same as polygon turn above
        self.pd()
        edge = 2 * radius * sin(pi / sides) # logo uses sin(180 / sides); Python uses radians
        self.polygon(sides, edge)
        self.left(angle)    # turn back to where we started
        self.pu()
        self.bk(radius)
        self.pd()

    def showTurtle(self):
        """makes the turtle visible
        the visibility of the turtle is independent of the turtle pen,
        use penUp() and penDown() to control the pen state"""
        if not self._visible:
            self._drawTurtle()
        self._visible = 1

    def hideTurtle(self):
        """hides the turtle
        the visibility of the turtle is independent of the turtle pen,
        use penUp() and penDown() to control the pen state"""
        if self._dirty:
            self._drawTurtle()
        self._visible = 0

    def _drawTurtle(self):
        """ override """
        pass

    def st(self):
        """equivelant to showTurtle
        makes the turtle visible
        the visibility of the turtle is independent of the turtle pen,
        use penUp() and penDown() to control the pen state"""
        self.showTurtle() # logo showTurtle abbreviation

    def ht(self):
        """equivelant to hideTurtle
        hides the turtle
        the visibility of the turtle is independent of the turtle pen,
        use penUp() and penDown() to control the pen state"""
        self.hideTurtle() # logo hideTurtle abbreviation

    def suspendOdometer(self):
        """suspends the turtle odometer"""
        self._odometerOn = 0

    def resumeOdometer(self):
        """resumes the turtle odometer"""
        self._odometerOn = 1

    def getOdometer(self):
        """returns the turtle odometer"""
        return self._odometer

    def resetOdometer(self):
        """resets the turtle odometer to 0.0"""
        self._odometer = 0.0

    def penUp(self):
        """raises the turtle pen, so no drawing will occur on subsequent
        commands until the pen is lowered with penDown"""
        self._drawing = 0

    def penDown(self):
        """lowers the turtle pen"""
        self._drawing = 1

    # may have to do multiple versions of methods if case-sensitivity
    # is an issue, however this seems a bit much, so I may only
    # support lowercase versions and abbreviations
    #def PenUp(self): self.penUp()      # logo Pen Up
    def pu(self):
        """equivelant to penUp
        raises the turtle pen, so no drawing will occur on subsequent
        commands until the pen is lowered with penDown"""
        self.penUp()         # logo Pen Up
    #def PenDown(self): self.penDown()  # logo Pen Down
    def pd(self):
        """equivelant to penDown
        lowers the turtle pen"""
        self.penDown()       # logo Pen Down
    #def up(self): self.penUp()         # Guido Pen Up
    #def down(self): self.penDown()  # Guido Pen Down

    """
    # Logo functions to implement
    # see http://www.embry.com/rLogo/rLogoReference.html
    # and http://www.embry.com/rLogo/Index.html for a working logo applet
    
    # setpencolor will be handled by color()
    setpencolor and setpc (red) (green) (blue)
    - changes the pen color to the specified color.
    (red), (green), and (blue) must range from 0 to 255.
    
    setbackcolor and setbc (red) (green) (blue)
    - changes the background color to the specified color.
    (red), (green), and (blue) must range from 0 to 255

    write
    """
    def write(self, txt):
        """ override """
        pass
    
    def width(self, width):
        """set the pen width"""
        self._width = float(width)

    def home(self):
        """resets the turtle back to its starting location"""
        x0, y0 = self._origin
        self._goto(x0, y0)
        #self._goto(self._origin)
        self._angle = 0.0

    def _clear(self):
        """ override """
        pass

    def clearScreen(self):
        """clears the screen"""
        self._clear() # logo clear graphics (screen)
    def cg(self):
        """equivelant to clearScreen
        clears the screen"""
        self._clear()          # logo clear graphics
    def cs(self):
        """equivelant to clearScreen
        clears the screen"""
        self._clear()          # logo clear graphics (screen)
    def cls(self):
        """equivelant to clearScreen
        clears the screen"""
        self._clear()         # logo clear graphics (screen)

    # need to enhance this to support the various
    # color settings: colourName, rgb
    """ valid named colors from wxWindows http://www.lpthe.jussieu.fr/~zeitlin/wxWindows/docs/wxwin64.htm#wxcolourdatabase

aquamarine, black, blue, blue violet, brown, cadet blue, coral, cornflower blue, cyan, dark grey, 
dark green, dark olive green, dark orchid, dark slate blue, dark slate grey dark turquoise, dim 
grey, firebrick, forest green, gold, goldenrod, grey, green, green yellow, indian red, khaki, 
light blue, light grey, light steel blue, lime green, magenta, maroon, medium aquamarine, medium 
blue, medium forest green, medium goldenrod, medium orchid, medium sea green, medium slate blue, 
medium spring green, medium turquoise, medium violet red, midnight blue, navy, orange, orange 
red, orchid, pale green, pink, plum, purple, red, salmon, sea green, sienna, sky blue, slate 
blue, spring green, steel blue, tan, thistle, turquoise, violet, violet red, wheat, white, 
yellow, yellow green
    """

    def color(self, *args):
        """ override """
        pass

    def setPenColor(self, *args):
        """set the foreground pen color"""
        self.color(self, *args)
    def setpc(self, *args):
        """equivelant to setPenColor
        set the foreground pen color"""
        self.color(self, *args)
        
    def setBackColor(self, *args):
        """ override """
        pass

    def setbc(self, *args):
        """equivelant to setBackColor
        set the background pen color"""
        self.setbackcolor(self, *args)

    def turtleDelay(self, s):
        """set the delay the turtle waits between drawing commands"""
        self._turtleDelay = s

    def reset(self):
        """ this should be called after a subclass has done its reset """
        """ but currently it needs to be overridden"""
        #canvas = self._canvas
        # just get the width and height from the window?
        #width = FRAME_WIDTH     #canvas.winfo_width()
        #height = FRAME_HEIGHT   # canvas.winfo_height()
        #self._color = "black"
        #self._color = wxPen(wxNamedColour("black"))

        self._origin = float(width)/2.0, float(height)/2.0
        self._position = self._origin
        self._odometer = 0.0
        self._odometerOn = 0    # don't waste time tracking unless requested
        self._angle = 0.0
        self._drawing = 1
        self._width = 1
        self._filling = 0
        self._path = []
        self._tofill = []
        self._dirty = 0
        self._visible = 0
        self._drawingTurtle = 0
        self._turtleDelay = 0

    # implicit save of pen state, penUp, pen state restore
    def moveTo(self, x=None, y=None):
        """move the turtle to position x, y"""
        if x is None:
            x = self._position[0]
        if y is None:
            y = self._position[1]
        drawingState = self._drawing
        self._drawing = 0
        self._goto(x, y)
        self._drawing = drawingState

    # this might get an implicit save of pen state, penDown, pen state restore
    def lineTo(self, x=None, y=None):
        """draw a line between the current turtle position and x, y
        the turtle is moved to the new x, y position"""
        if x is None:
            x = self._position[0]
        if y is None:
            y = self._position[1]
        self._goto(x, y)

    def draw(self, x, y):
        """equivelant to lineTo
        draw a line between the current turtle position and x, y
        the turtle is moved to the new x, y position"""
        self.lineTo(x, y)	# logo lineTo

    def _goto(self, x1, y1):
        """ override """
        pass
        