from math import * # Also for export
from aTurtle import aPen
from wxPython.wx import *
from types import *
import time

# these are temporary
# replace by looking at canvas dimensions
#FRAME_WIDTH     = 750
#FRAME_HEIGHT    = 600

class TurtleDC(wxClientDC):
    def __init__(self, device):
        self._deviceSize = device.GetClientSize()
        wxClientDC.__init__(self, device)

class Pen(aPen):
    def __init__(self, canvas):
        self._canvas = canvas
        self.dc = canvas
        self.dc.SetOptimization

        aPen.__init__(self)

    # illegal syntax, so I used None and the if tests instead
    #def plot(self, x=self._position[0], y=self._position[1]):
    def plot(self, x=None, y=None):
        """draws a point at x, y using the current pen color and width
        note that the the turtle position is not changed by plot"""
        #self.dc.SetPen(self._color, self._width)
        self.dc.SetPen(self._pen)
        if x is None:
            x = self._position[0]
        if y is None:
            y = self._position[1]
        # setting the color makes chaos1 over 3 times slower
        # so we probably need to do a simpler if/then check
        # prior to setting the color
        # that means keeping the variable in a form that is
        # simple to compare to the underlying dc canvas
        self.dc.DrawPoint(x, y)
        # other variations
        #self.dc.DrawLine(x, y, x+1, y+1)
        #self._goto(x, y)   # actually this isn't equivelant


    # probably replace this with a wxPython primitive for polygons
    # so we can support filled polys...
    # def polygon(self, sides, distance):


    """
    # Logo functions to implement
    # see http://www.embry.com/rLogo/rLogoReference.html
    # and http://www.embry.com/rLogo/Index.html for a working logo applet
    
    # setpencolor will be handled by color()
    setpencolor and setpc (red) (green) (blue)
    - changes the pen color to the specified color.
    (red), (green), and (blue) must range from 0 to 255.
    
    setbackcolor and setbc (red) (green) (blue)
    - changes the background color to the specified color.
    (red), (green), and (blue) must range from 0 to 255
    """

    def write(self, txt):
        """prints txt at the current turtle position"""
        self.dc.DrawText(txt, self._position[0], self._position[1])
    
    def _clear(self):
        self.dc.Clear()
        self._dirty = 0

    # need to enhance this to support the various
    # color settings: colourName, rgb
    """ valid named colors from wxWindows http://www.lpthe.jussieu.fr/~zeitlin/wxWindows/docs/wxwin64.htm#wxcolourdatabase

aquamarine, black, blue, blue violet, brown, cadet blue, coral, cornflower blue, cyan, dark grey, 
dark green, dark olive green, dark orchid, dark slate blue, dark slate grey dark turquoise, dim 
grey, firebrick, forest green, gold, goldenrod, grey, green, green yellow, indian red, khaki, 
light blue, light grey, light steel blue, lime green, magenta, maroon, medium aquamarine, medium 
blue, medium forest green, medium goldenrod, medium orchid, medium sea green, medium slate blue, 
medium spring green, medium turquoise, medium violet red, midnight blue, navy, orange, orange 
red, orchid, pale green, pink, plum, purple, red, salmon, sea green, sienna, sky blue, slate 
blue, spring green, steel blue, tan, thistle, turquoise, violet, violet red, wheat, white, 
yellow, yellow green
    """

    def width(self, w):
        """set the pen width"""
        if self._dirty:
            self._drawTurtle()
        #self._width = float(width)
        #self._width = w
        self._pen.SetWidth(w)
        if self._visible:
            self._drawTurtle()

    def color(self, *args):
        """set the foreground pen color
        both (r, g, b) values and named colors are valid"""
        
        if self._dirty:
            self._drawTurtle()
        # _color is actually the pen here
        # I need to just keep a pen and change the elements
        # of it
        if len(args) == 1:
            #self._color = wxPen(wxNamedColour(args[0]))
            #self._color = wxNamedColour(args[0])
            self._pen.SetColour(wxNamedColour(args[0]))
        else:
            # self._color = wxPen(wxColour(args[0], args[1], args[2]))
            #self._color = wxColour(args[0], args[1], args[2])
            self._pen.SetColour(wxColour(args[0], args[1], args[2]))

        if self._visible:
            self._drawTurtle()
        # http://www.lpthe.jussieu.fr/~zeitlin/wxWindows/docs/wxwin265.htm
        # wxPen reference for pen width and style
        # colourdb.py for actual rgb color values of named colors in wxPython
        
        """
        print self._color.GetCap()
        print self._color.GetColour()
        #print self._color.GetDashes()
        print self._color.GetJoin()
        print self._color.GetStipple()
        print self._color.GetStyle()
        print self._color.GetWidth()
        """
        """ example output
        130
        (0, 0, 255)
        122
        <C wxBitmap instance at _3331988_wxBitmap_p>
        100
        1
        """
        
        #print self._color
        # self.dc.SetPen(wxPen(wxNamedColour(args[0])))

    # the background is shared, not specific to each turtle
    def setBackColor(self, *args):
        """set the background pen color
        both (r, g, b) values and named colors are valid"""
        brush = self.dc.GetBackground()
        if len(args) == 1:
            brush.SetColour(wxNamedColour(args[0]))
        else:
            brush.SetColour(wxColour(args[0], args[1], args[2]))
        self.dc.SetBackground(brush)

    # non-optimized turtle
    # the turtle rotates from its base rather than a pivot point
    # in the "center" of the turtle shape
    #
    # as currently implemented drawTurtle is designed to erase
    # itself on a subsequent call to drawTurtle
    # in addition, this version does not use offscreen bitmaps
    # so an inversion is done on the pixels underneath the turtle
    # rather than showing the actual color of the current turtle
    def _drawTurtle(self):
        """private method for drawing the turtle when showTurtle()
        has been called to make the turtle visible"""
        #if not self._drawingTurtle:
        self._dirty = not self._dirty
        self._drawingTurtle = 1
        drawingState = self._drawing
        currentPos = self._position
        currentAngle = self._angle
        #self._pen.SetCap(wxCAP_PROJECTING)
        #self._pen.SetJoin(wxJOIN_BEVEL)
        self.dc.SetPen(self._pen)
        self.dc.SetLogicalFunction(wxINVERT)

        a = 30  # make a larger to get a bigger turtle
        b = 2 * a * tan(15 * self._invradian)
        c = a / cos(15 * self._invradian)
        self.pu(); self.forward(a); self.pd()
        self.left(165)
        self.forward(c)
        self.left(105)
        self.forward(b)
        self.left(105)
        self.forward(c)
        
        #self._pen.SetJoin(wxJOIN_ROUND)
        #self._pen.SetCap(wxCAP_ROUND)
        self.dc.SetLogicalFunction(wxCOPY)
        self._angle = currentAngle
        self._drawing = drawingState
        self._position = currentPos
        self._drawingTurtle = 0
        if self._dirty and self._turtleDelay > 0:
            time.sleep(self._turtleDelay)
        #for j in range(1000000): pass	# huge delay so we can see the turtle

    def reset(self):
        """reset the turtle to its initial state"""
        #canvas = self._canvas
        # just get the width and height from the window?
        #width = FRAME_WIDTH     #canvas.winfo_width()
        #height = FRAME_HEIGHT   # canvas.winfo_height()
        width, height = self._canvas._deviceSize
        #self._color = "black"
        self._color = wxPen(wxNamedColour("black"))
        self._pen = wxPen('black', 1, wxSOLID)

        #aPen.reset()        
        self._origin = float(width)/2.0, float(height)/2.0
        #print "width: %d, height: %d" % (width, height)
        #print "_origin.x: %f, _origin.y: %f" % (self._origin[0], self._origin[1])
        self._position = self._origin
        self._odometer = 0.0
        self._odometerOn = 0    # don't waste time tracking unless requested
        self._angle = 0.0
        self._drawing = 1       # whether the pen is down
        self._width = 1
        #self._filling = 0
        #self._path = []
        #self._tofill = []
        self._visible = 0       # whether the turtle is visible, independent of the pen state
        self._dirty = 0         # if _dirty then erase old turtle before drawing
        self._drawingTurtle = 0 # only true while drawing the turtle
        self._turtleDelay = 0   # number of seconds to pause after drawing the turtle

    def _goto(self, x1, y1):
        if not self._drawingTurtle:
            if self._odometerOn:
                self._odometer += self.distanceXY(x1, y1)
            if self._dirty:
                # this is necessary to avoid an endless loop as drawTurtle uses _goto
                #self._dirty = 0
                #visible = self._visible
                #self._visible = 0
                self._drawTurtle()
                #self._visible = visible

        x0, y0 = start = self._position
        self._position = map(float, (x1, y1))
        #if self._filling:
        #    self._path.append(self._position)
        #print self._drawing
        if self._drawing:
            """
            if self._tracing:
                dx = float(x1 - x0)
                dy = float(y1 - y0)
                distance = hypot(dx, dy)
                nhops = int(distance)
                #print "tracing %d %d %d %d" % (x0, y0, x0, y0)
                #self.dc.SetPen(self._color, self._width)
                self.dc.SetPen(self._pen)
                self.dc.DrawLine(x0, y0, x1, y1)
            else:
            """
            #print "%d %d %d %d" % (x0, y0, x0, y0)
            #self.dc.SetPen(self._color, self._width)
            if not self._drawingTurtle:
                self.dc.SetPen(self._pen)
            self.dc.DrawLine(x0, y0, x1, y1)
                
        if not self._drawingTurtle:
            if self._visible:
                # this is necessary to avoid an endless loop as drawTurtle uses _goto
                #self._visible = 0
                #self._dirty = 1
                self._drawTurtle()
                #self._visible = 1
