#!/usr/bin/python

"""
KEA notes to myself

__version__ = "$Revision: 1.7 $"
__date__ = "$Date: 2001/08/15 23:41:21 $"
__author__ = "Kevin Altis<altis@semi-retired.com>"

"""

import PythonCardPrototype
from PythonCardPrototype.loader import configOptions
from PythonCardPrototype import dialog
import os, sys

BORDER = 5
RESIZE_LEFT = 1
RESIZE_RIGHT = 2
DRAG_X = 3
RESIZE_TOP = 4
RESIZE_BOTTOM = 5
DRAG_Y = 6

class WidgetsTest(PythonCardPrototype.model.Background):
    """
    All the widgets that can be changed have a three-letter prefix followed
    by the actual classname to distinguish them from other widgets in the
    background.
    Compare the method of changing the attribute state in on_chkEnabled_mouseClick
    versus on_chkVisible_mouseClick. I'm using the new findAllComponents method
    to get a list of all the widgets at once.
    """

    def on_chkEnabled_mouseClick(self, target, event):
        checked = target.checked
        for w in self.components.values():
            # for btnButton, wType would be widget.Button
            # __class__ would be widget.Button
            # we can't use isinstance unless we put it in a try/except
            # block because isinstance requires a valid class as the
            # 2nd argument, so passing in "elTextField" or something like
            # that triggers an exception
            # KEA 2001-08-09
            # need a better way of getting the widget prefix dynamically
            # I've just gone soft in the head and can't see the right way here
            wType = "PythonCardPrototype.widget." + w.name[3:]
            if str(w.__class__) == wType:
                w.enabled = checked
        self.components.fldTextFieldNoBorder.enabled = checked

    def on_chkVisible_mouseClick(self, target, event):
        checked = target.checked
        for w in self.components.values():
            wType = "PythonCardPrototype.widget." + w.name[3:]
            if str(w.__class__) == wType:
                w.visible = checked
        self.components.fldTextFieldNoBorder.visible = checked

    # editable only applies to TextField, PasswordField, and TextArea
    def on_chkEditable_mouseClick(self, target, event):
        checked = target.checked
        self.components.fldTextField.editable = checked
        self.components.fldPasswordField.editable = checked
        self.components.fldTextArea.editable = checked
        self.components.fldTextFieldNoBorder.editable = checked
        
    def on_btnBackgroundColor_mouseClick(self, target, event):
        dlg = dialog.ColorDialog(self)
        if dlg.accepted():
            color = dlg.getColor()
            for w in self.components.values():
                wType = "PythonCardPrototype.widget." + w.name[3:]
                if str(w.__class__) == wType:
                    w.backgroundColor = color
            self.components.fldTextFieldNoBorder.backgroundColor = color

    # compare this method of expliciting setting each component
    # one at a time compared to the for loop above
    def on_btnForegroundColor_mouseClick(self, target, event):
        dlg = dialog.ColorDialog(self)
        if dlg.accepted():
            color = dlg.getColor()
            # print color
            self.components.btnButton.foregroundColor = color
            self.components.fldTextField.foregroundColor = color
            self.components.fldPasswordField.foregroundColor = color
            self.components.fldTextArea.foregroundColor = color
            self.components.txtStaticText.foregroundColor = color
            self.components.chkCheckBox.foregroundColor = color
            self.components.radRadioGroup.foregroundColor = color
            self.components.popChoice.foregroundColor = color
            self.components.lstList.foregroundColor = color
            self.components.sldSlider.foregroundColor = color
            self.components.imgImage.foregroundColor = color
            self.components.imgImageButton.foregroundColor = color
            self.components.fldTextFieldNoBorder.foregroundColor = color

    def on_btnFont_mouseClick(self, target, event):
        dlg = dialog.FontDialog(self)
        if dlg.accepted():
            color = dlg.getColor()
            font = dlg.getFont()
            print color
            print font

    def on_btnToolTip_mouseClick(self, target, event):
        dlg = dialog.TextEntryDialog(self, 'ToolTip', 'Enter a toolTip', 'Hello toolTip')
        if dlg.accepted():
            txt = dlg.getText()
            for w in self.components.values():
                wType = "PythonCardPrototype.widget." + w.name[3:]
                if str(w.__class__) == wType:
                    w.toolTip = txt

    def on_btnBgBackgroundColor_mouseClick(self, target, event):
        dlg = dialog.ColorDialog(self)
        if dlg.accepted():
            color = dlg.getColor()
            self.setBackgroundColor(color)

    """
    experimental drag and resize code
    hold down the control key and mouseDown to drag a widget
    hold down the control key and mouseDown within 5 pixels of the edge
    of the widget to resize

    This seems to work okay on Windows, but not so well on Linux.
    My algorithm is probably not very good.
    """
    def on_bgWidgets_mouseDown(self, target, event):
        global BORDER
        
        evt = event.getNativeEvent()
        if evt.ControlDown():
            # record the starting location so, that the drags are
            # always relative and the widget won't jump around
            evt = event.getNativeEvent()
            self.dragOffset = evt.GetPosition()
            xClick, yClick = self.dragOffset
            size = target._delegate.GetSize()
            rect = (0, 0, size[0] - BORDER, size[1] - BORDER)
            print rect
            print xClick, yClick
            if xClick < BORDER:
                print "resize left"
                self.xOp = RESIZE_LEFT
            elif xClick > rect[2]:
                print "resize right"
                self.xOp = RESIZE_RIGHT
            else:
                print "drag x"
                self.xOp = DRAG_X
            if yClick < BORDER:
                print "resize top"
                self.yOp = RESIZE_TOP
            elif yClick > rect[3]:
                print "resize bottom"
                self.yOp = RESIZE_BOTTOM
            else:
                print "drag y"
                self.yOp = DRAG_Y

    def on_bgWidgets_mouseDrag(self, target, event):
        evt = event.getNativeEvent()
        if evt.ControlDown():
            xPos, yPos = target.position   # use _delegate.GetPosition() ?
            width, height = target._delegate.GetSize()
            newWidth = width
            newHeight = height
            print "xPos %d, yPos %d" % (xPos, yPos)
            if self.xOp == DRAG_X and self.yOp == DRAG_Y:
                xOff = xPos + evt.GetX() - self.dragOffset[0]
                yOff = yPos + evt.GetY() - self.dragOffset[1]
                print "xOff %d, yOff %d" % (xOff, yOff)
                target.position = (xOff, yOff)
            else:
                xOff = xPos
                if self.xOp == RESIZE_LEFT:
                    # both width and the x position have to change
                    xOff = xPos + evt.GetX() - self.dragOffset[0]
                    newWidth = width + xPos - xOff #evt.GetX() + self.dragOffset[0]
                elif self.xOp == RESIZE_RIGHT:
                    # RESIZE_RIGHT
                    #newWidth += evt.GetX() - self.dragOffset[0]
                    #newWidth += 1
                    newWidth = evt.GetX()
                print "width %d, newWidth %d" % (width, newWidth)
                    
                yOff = yPos
                if self.yOp == RESIZE_TOP:
                    # both height and the y position have to change
                    yOff = yPos + evt.GetY() - self.dragOffset[1]
                    newHeight = height + yPos - yOff #evt.GetY() + self.dragOffset[1]
                elif self.yOp == RESIZE_BOTTOM:
                    # RESIZE_BOTTOM
                    #newHeight += evt.GetY() - self.dragOffset[1]
                    #newHeight += 1
                    newHeight = evt.GetY()
                print "height %d, newHeight %d" % (height, newHeight)
                    
                print "xOff %d, yOff %d" % (xOff, yOff)

                if xPos != xOff or yPos != yOff:
                    target.position = (xOff, yOff)
                if width != newWidth or height != newHeight:
                    #target._delegate.SetSize((width, height))
                    target._delegate.SetSize((newWidth, newHeight))
                #target._delegate.SetDimensions(xOff, yOff, newWidth, newHeight)                    


    def on_menuFileExit_select(self, menu, event):
        self.Close()


if __name__ == '__main__':
    base, ext = os.path.splitext(sys.argv[0])
    filename = base + ".rsrc.py"

    configOptions()

    app = PythonCardPrototype.model.PythonCardApp(filename)
    app.MainLoop()
