# __version__ = "$Revision: 1.4 $"
# __date__ = "$Date: 2001/08/23 15:41:50 $"

# The format of an entry is:
#
#    '<entry-name>' : {
#        'parent' : < <class-name> | None > ,
#        'events' : < [ '<event-name>', ... ] >,
#        'attributes' : < { '<attribute-name>' : { 
#                         'presence' : < 'mandatory' | 'optional' >,
#                         'default' : <valid-python-value> 
#                       > }
#   }

# How do we distinguish between resource/runtime

# Will 'mouseClick' work if specified in Widget's event list?

# Note that events and attributes are inherited.  The res.Spec class
# depends on the following entries being in order from top to bottom
# based on inheritance.  Component must appear before Widget, Widget
# must appear before TextField, Textfield must appear before PasswordField,
# ad nauseum ;)

[

   { 'name':'Component', 'info':{
        'parent' : None,
        'events' : [],
        'attributes' : {
            'name' : { 'presence' : 'mandatory' },
            'command' : { 'presence' : 'optional', 'default' : None }
        }
    }},

   { 'name':'Stack', 'info': {
        'parent' : None,
        'events' : [],
        'attributes' : {
            'name' : { 'presence' : 'mandatory' },
            'title' : { 'presence' : 'mandatory' },
            'position' : { 'presence' : 'optional', 'default' : [ -1, -1 ] },
            'size' : { 'presence' : 'optional', 'default' : [ -1, -1 ] }
        }
    }},

   { 'name':'MenuBar', 'info': {
        'parent' : None,
        'events' : [],
        'attributes' : {
            'menus' : { 'presence' : 'optional', 'default': [] }
        }
    }},

   { 'name':'Menu' , 'info': {
        'parent' : None,
        'events' : [],
        'attributes' : {
            'name' : { 'presence' : 'mandatory' },
            'label' : { 'presence' : 'mandatory' },
            'items' : { 'presence' : 'optional', 'default': [] }
        }
    }},

   { 'name':'MenuItem', 'info': {
        'parent' : 'Component',
        'events' : [],
        'attributes' : {
            'label' : { 'presence' : 'mandatory' }
        }
    }},

   { 'name':'Background', 'info': {
        'parent' : None,
        'events' : [],
        'attributes' : {
            'name' : { 'presence' : 'mandatory' },
            'file' : { 'presence' : 'mandatory' } ,
            'foregroundColor' : { 'presence' : 'optional', 'default' : None },
            'backgroundColor' : { 'presence' : 'optional', 'default' : None },
            'image' : { 'presence' : 'optional', 'default' : None } ,
            'tiled' : { 'presence' : 'optional', 'default' : 0 }
        }
    }},

    { 'name':'Widget', 'info': {
        'parent' : 'Component',
        'events' : [
            'gainFocus',
            'loseFocus',
            'mouseDoubleClick',
            'mouseDown',
            'mouseUp',
            'mouseMiddleDown',
            'mouseMiddleUp',
            'mouseMiddleDoubleClick',
            'mouseContextDown',
            'mouseContextUp',
            'mouseContextDoubleClick',
            'mouseEnter',
            'mouseLeave',
            'mouseMove',
            'mouseDrag'
        ],
        'attributes' : {
            'enabled' : { 'presence' : 'optional', 'default' : 1 },
            'visible' : { 'presence' : 'optional', 'default' : 1 },
            'foregroundColor' : { 'presence' : 'optional', 'default' : None },
            'backgroundColor' : { 'presence' : 'optional', 'default' : None },
            #'helpText' : { 'presence' : 'optional', 'default' : '' },
            'toolTip' : { 'presence' : 'optional', 'default' : '' },
            'font' : { 'presence' : 'optional', 'default' : None },
            'position' : { 'presence' : 'optional', 'default' : [ -1, -1 ] },
            'size' : { 'presence' : 'optional', 'default' : [ -1, -1 ] }
        }
    }},

    { 'name':'Button', 'info': { 
        'parent' : 'Widget', 
        'events' : [ 
            'mouseClick' 
        ],
        'attributes' : {
            'label' : { 'presence' : 'mandatory' }
            # KEA don't need style for Button unless we are going to support
            # Win32-specific attributes
            #'style' : { 'presence' : 'optional', 'default' : '3d', 'values' : [ '3d', 'none' ] },
            #'isdefault' : { 'presence' : 'optional', 'default' : 0 }
        }
    }},

    { 'name':'StaticText', 'info': { 
        'parent' : 'Widget', 
        'events' : [], 
        'attributes' : {
            'text' : { 'presence' : 'optional', 'default' : '' },
            'alignment' : { 'presence' : 'optional', 'default' : 'left', 'values' :[ 'left', 'right', 'center' ] }
        } 
    }},

    { 'name':'TextField', 'info':{
        'parent' : 'Widget',
        'events' : [
            'keUp',
            'keyDown',
            'keyPress',
            'textUpdate',
            'mouseClick',
            'closeField',
            'exitField'
        ],
        'attributes' : {
            'text' : { 'presence' : 'optional', 'default' : '' },
            'editable' : { 'presence' : 'optional', 'default' : 1 },
            'border' : { 'presence' : 'optional', 'default' : '3d', 'values' : [ '3d', 'none' ] }
        }
    }},

    { 'name':'PasswordField' , 'info': {
        'parent' : 'TextField',
        'events' : [],
        'attributes' : {}
    }},

    { 'name':'TextArea', 'info': {
        'parent' : 'TextField',
        'events' : [ 
            'textEnter' 
        ],
        'attributes' : {}
    }},

    { 'name':'RadioGroup', 'info': {
        'parent' : 'Widget',
        'events' : [ 
            'select' 
        ],
        'attributes' : { 
            'items' : { 'presence' : 'optional', 'default' : [] },
            'selected' : { 'presence' : 'optional', 'default' : 0 },
            'layout' : { 'presence' : 'optional', 'default' : 'vertical', 'values' : [ 'horizontal', 'vertical' ] },
            'max' : { 'presence' : 'optional', 'default' : 1 }
            # KEA we can't control the border of a wxRadioBox as far as I can tell
            #'border' : { 'presence' : 'optional', 'default' : None }        
        }
    }},

    { 'name':'List', 'info': {
        'parent' : 'Widget',
        'events' : [ 
            'select' 
        ],
        'attributes' : { 
            'items' : { 'presence' : 'optional', 'default' : [] },
            'selected' : { 'presence' : 'optional', 'default' : 0 } 
        }
    }},

    { 'name':'Choice', 'info': {
        'parent' : 'Widget',
        'events' : [
            'select' 
        ],
        'attributes': { 
            'items' : { 'presence' : 'optional', 'default' : [] },
            'selected' : { 'presence' : 'optional', 'default' : 0 }
        }
    }},

    { 'name':'CheckBox', 'info': { 
        'parent' : 'Widget', 
        'events' : [ 
            'mouseClick' 
        ],
        'attributes' : {
            'checked' : { 'presence' : 'optional', 'default' : 0 }
        }
    }},

    { 'name':'Image', 'info':{
        'parent' : 'Widget',
        'events' : [],
        'attributes' : {
            # KEA shouldn't there be a 'file' attribute here
            # could call it 'image' to match background above
            # but it is mandatory
            #'bitmap' : { 'presence' : 'optional', 'default' : None },
            # KEA kill border for now, until variations of what is possible are worked out
            # use ImageButton if you want images with transparent border
            #'border' : { 'presence' : 'optional', 'default' : '3d', 'values' : [ '3d', 'transparent', 'none' ] }
        }
    }},

    { 'name':'ImageButton', 'info': { 
        'parent' : 'Image', 
        'events' : [ 
            'mouseClick' 
        ],
        'attributes' : {
            # KEA since 3d is Windows specific, make transparent the default
            'border' : { 'presence' : 'optional', 'default' : 'transparent', 'values' : [ '3d', 'transparent', 'none' ] }
        }
    }},

    { 'name':'Slider', 'info': { 
        'parent' : 'Widget', 
        'events' : [ 
            'mouseClick' 
        ],
        'attributes' : { 
            'layout' : { 'presence' : 'optional', 'default' : 'horizontal', 'values' : [ 'horizontal', 'vertical' ]  },
            'min' : { 'presence' : 'optional', 'default' : 0 }, 
            'max' : { 'presence' : 'optional', 'default' : 100 }, 
            'value' : { 'presence' : 'optional', 'default' : 0 } 
        }
    }},

    # KEA added 2001-08-03
    { 'name':'StaticLine', 'info': { 
        'parent' : 'Widget', 
        'events' : [], 
        'attributes' : {
            'layout' : { 'presence' : 'optional', 'default' : 'horizontal', 'values' : [ 'horizontal', 'vertical' ]  }
        } 
    }}

]