
"""
__version__ = "$Revision: 1.29 $"
__date__ = "$Date: 2001/08/23 15:41:50 $"
"""

import sys
from types import FunctionType
import types    # KEA 2001-07-27
from wxPython.wx import *
from res import *
from uniqueid import *
from dispatch import *
from event import *
from error import *
from log import *
from binding import *

wxCLIP_SIBLINGS = 0x20000000
MAGIC_GET_PREFIX = '_get'
MAGIC_SET_PREFIX = '_set'

class Font :

    def __init__( self, name, size, style, weight ) :
        self.name = name
        self.size = size
        self.style = style
        self.weight = weight



# The meager beginnings of component object model. Maybe
# we could call it MOC, Model for Object Components.

# Example wrappers for GUI toolkit-specific widgets.

class Widget( Component ) :
    """
    The base class for all of our GUI controls.

    Each Widget must bind itself to the wxPython
    event model.  When it receives an event
    from wxPython, it will convert the event
    to a PythonCArd event.Event ( SelectEvent, ClickEvent,
     etc ) and post the event to the EventQueue.
    """

    _events = []

    def __init__( self, aParent, aResource ) :
        # disabled '_helpText', 
        attributes = ['_backgroundColor', '_command', '_delegate', '_enabled',
                      '_font', '_foregroundColor', '_id',
                      '_listeners', '_name', '_parent', '_position', '_size',
                      '_toolTip', '_visible']
        self._createAttributes(attributes)
        Component.__init__( self )
        self._parent = aParent 
        self._id = UniqueIdFactory().createUniqueId()
        self._name = aResource.name
        self._delegate = self._createDelegate( aParent, aResource )
        self._bindEvents()        
        assert self._delegate is not None, 'null delegate! ' + self._name 
        self._setCommand( aResource.command )
        self._setEnabled( aResource.enabled )
        self._setVisible( aResource.visible )
        self._setForegroundColor( aResource.foregroundColor )
        self._setBackgroundColor( aResource.backgroundColor )
        #self._setHelpText( aResource.helpText )
        self._setToolTip( aResource.toolTip )
        self._setFont( aResource.font )
        self._setPosition( aResource.position )
        self._setSize( aResource.size )

    def _getAttributeNames(self):
        """Return list of magic attributes to extend introspection.
        
        Returns only attributes having both "get" and "set" methods.
        Unlike dir(), this method does walk up its inheritance tree."""
        list = []
        dict = {}
        # Remove duplicates from the attribute list.
        for item in self._getAllAttributeNames(object=self):
            dict[item] = None
        list += dict.keys()
        list.sort()
        return list

    def _getAllAttributeNames(self, object):
        """Return list of all magic attributes for an object.
        
        Recursively walk through a class and all base classes."""
        list = []
        for name in dir(object):
            # Is this one of our magic "get" methods?
            if name.startswith(MAGIC_GET_PREFIX):
                # Remove the prefix from the name.
                name = name[len(MAGIC_GET_PREFIX):]
                # Check for a corresponding "set" method.
                if name and hasattr(self, MAGIC_SET_PREFIX + name):
                    # Add the stripped down name to our list.
                    list.append(name[0].lower() + name[1:])
        # Get any magic attributes for the class.
        if hasattr(object, '__class__'):
            list += self._getAllAttributeNames(object.__class__)
        # Also get attributes from any and all parent classes.
        if hasattr(object, '__bases__'):
            for base in object.__bases__:
                list += self._getAllAttributeNames(base)
        return list

    def __getattr__(self, name):
        if not name.startswith(MAGIC_GET_PREFIX) and name[0].islower():
            callName = MAGIC_GET_PREFIX + name[0].upper() + name[1:]
            return getattr(self, callName)()
        else:
            raise AttributeError, name

    def __setattr__(self, name, value):
        if self.__dict__.has_key(name):
            self.__dict__[name] = value
        elif not name.startswith(MAGIC_SET_PREFIX) and name[0].islower():
            callName = MAGIC_SET_PREFIX + name[0].upper() + name[1:]
            try:
                method = getattr(self, callName)
            except:
                raise AttributeError, name
            return method(value)
        else:
            raise AttributeError, name
        
    def _createAttributes(self, attrList):
        for attr in attrList:
            if not self.__dict__.has_key(attr):
                self.__dict__[attr] = None

    def _createDelegate( self, aParent, aResource ) :
        raise AbstractMethodException
    
    def _bindEvents( self ) :
        EventBinding( self )

    def _getDelegate( self ) :
        return self._delegate

    def __repr__( self ) :
        return str( self.__dict__ )

    def _getParent( self ) :
        return self._parent

    def getId( self ) :
        return self._id

    def _getName( self ) :
        return self._name 

    def _setName( self, aString ) :
        raise AttributeError, "name attribute is read-only"

    def _setPosition( self, aPosition ) :
        self._delegate.Move( aPosition )
        self._position = self._delegate.GetPosition()   # get the actual position, not (-1, -1)

    def _getPosition( self ) :
        return self._position

    def _setSize( self, aSize ) :
        # KEA 2001-07-28, what is p? commented it out
        #p = self._delegate.GetPositionTuple()
        #RDS-Disabled: self._delegate.SetSize( aSize[ 0 ], aSize[ 1 ] )
        # KEA enabled
        self._delegate.SetSize( aSize )
        self._size = self._delegate.GetSize()   # get the actual size, not (-1, -1)

    def _getSize( self ) :
        return self._size

    """
    # KEA this only works for wxButton
    # so wait to add getDefaultSize until
    # we figure out how to get the right values
    # for other widgets
    def getDefaultSize( self ) :
        return self._delegate.GetDefaultSize()
    """

    def _setEnabled( self, aBoolean ) :
        self._enabled = aBoolean
        self._delegate.Enable( aBoolean )

    def _getEnabled( self ) :
        return self._enabled

    def _setVisible( self, aBoolean ) :
        self._visible = aBoolean
        self._delegate.Show( aBoolean )

    def _getVisible( self ) :
        return self._visible

    def _getForegroundColor( self ) :
        #return self._foregroundColor
        return self._delegate.GetForegroundColour()

    def _getBackgroundColor( self ) :
        #return self._backgroundColor
        return self._delegate.GetBackgroundColour()

    #def _getHelpText( self ) :
    #    return self._helpText
    
    def _getToolTip( self ) :
        return self._toolTip.GetTip()
    
    def _getFont( self ) :
        return self._font
    
    def _setForegroundColor( self, aColor ) :
        aColor = self.__getDefaultColor( aColor )
        #self._foregroundColor = aColor
        self._delegate.SetForegroundColour( aColor )
        self._getDelegate().Refresh()   # KEA wxPython bug?
    
    def _setBackgroundColor( self, aColor ) :
        aColor = self.__getDefaultColor( aColor )
        #self._backgroundColor = aColor
        self._delegate.SetBackgroundColour( aColor )
        self._getDelegate().Refresh()   # KEA wxPython bug?

    # KEA enabled, but it doesn't really do anything
    #def _setHelpText( self, aString ) :
    #    self._helpText = aString
        #self._delegate.SetHelpText( aString )
        
    def _setToolTip( self, aString ) :
        self._toolTip = wxToolTip( aString )
        self._delegate.SetToolTip( self._toolTip )
    
    def _setFont( self, aFont ) :
        self._font = aFont
        if aFont is None :
            return
        aWxFont = wxFont( aFont.size, 
                           wxDEFAULT_FONT, 
                           aFont.style,
                           aFont.weight,
                           facename = aFont.name )
        self._delegate.SetFont( aWxFont )

    def _setCommand( self, aString ) :
        self._command = aString

    def _getCommand( self ) :
        return self._command

    def __getDefaultColor( self, aColor ) :
        if aColor is None :
            return wxNullColour
        else :
            # KEA 2001-07-27
            # is the right place for this check?
            if type(aColor) is types.TupleType and len(aColor) == 3:
                return wxColour(aColor[0], aColor[1], aColor[2])
            else:
                return aColor

class Frame :
    """
    The top-level container for Widgets.
    """

    def __init__( self, aFrame ) :
        self._objectLookup = aFrame
        # KEA wxFrame is a wxPython class
        #self._delegate = wxPanel( wxFrame, -1 )
        self._delegate = wxPanel( aFrame, -1 )

class Panel( wxPanel ) :

    def __init__( self, aParent, imageFile, tiled ) :
        wxPanel.__init__( self, aParent, -1 )
        self._frame = aParent
        self._imageFile = imageFile
        self._backgroundTiling = tiled
        # KEA 2001-07-27
        # load the bitmap once and keep it around
        # this could fail, so should be a try/except
        if imageFile is not None :
            self._bitmap = Bitmap(imageFile)
            #self._backgroundTiling = 1  # KEA should be an attribute
            EVT_ERASE_BACKGROUND( self, self.onEraseBackground )
        self._delegate = self

    def tileBackground(self, deviceContext):
        # tile the background bitmap
        sz = self.GetClientSize()
        bmp = self._bitmap.getBits()
        w = bmp.GetWidth()
        h = bmp.GetHeight()

        if isinstance(self, wxScrolledWindow):
            # adjust for scrolled position
            spx, spy = self.GetScrollPixelsPerUnit()
            vsx, vsy = self.GetViewStart()
            dx,  dy  = (spx * vsx) % w, (spy * vsy) % h
        else:
            dx, dy = (w, h)

        x = -dx
        while x < sz.width:
            y = -dy
            while y < sz.height:
                deviceContext.DrawBitmap(bmp, x, y)
                y = y + h
            x = x + w

    def getForegroundColor(self):
        return self.GetForegroundColour()

    def getBackgroundColor(self):
        return self.GetBackgroundColour()

    def _getDefaultColor(self, aColor):
        if aColor is None :
            return wxNullColour
        else :
            # KEA 2001-07-27
            # is the right place for this check?
            if type(aColor) is types.TupleType and len(aColor) == 3:
                return wxColour(aColor[0], aColor[1], aColor[2])
            else:
                return aColor
            
    def setForegroundColor(self, aColor):
        aColor = self._getDefaultColor(aColor)
        self.SetForegroundColour(aColor)
        self.Refresh()   # KEA wxPython bug?
    
    def setBackgroundColor(self, aColor):
        aColor = self._getDefaultColor(aColor)
        self.SetBackgroundColour(aColor)
        self.Refresh()   # KEA wxPython bug?

    def onEraseBackground(self, aWxEvent):
        deviceContext = aWxEvent.GetDC()
        if not deviceContext :
            deviceContext = wxClientDC(self)
            r = self.GetUpdateRegion().GetBox()
            deviceContext.SetClippingRegion(r.x, r.y, r.width, r.height)
                                                       
        # KEA 2001-07-27
        # just loading the bitmap once in __init__, not each time
        # bitMap = Bitmap( self._imageFile )
        #deviceContext.DrawBitmap( bitMap.getBits(), 0, 0 )
        if self._backgroundTiling:
            self.tileBackground( deviceContext )
        else:
            deviceContext.DrawBitmap( self._bitmap.getBits(), 0, 0 )

    def _getDelegate( self ) :
        return self._delegate

class TextField( Widget ) :
    """
    A text field.
    """

    _events = [ KeyPressedEvent, 
                KeyDownEvent, 
                KeyUpEvent, 
                TextEnterEvent,
                TextUpdateEvent ] + MOUSE_EVENTS

    def __init__( self, aParent,  aResource ) :
        attributes = ['_editable', '_border']
        self._createAttributes(attributes)
        self._border = aResource.border
        Widget.__init__( self, aParent, aResource )
        self._setEditable( aResource.editable )

    def _createDelegate( self, aParent, aResource ) :
        # KEA need to check all possible variations on Win32 and Linux
        if aResource.border == 'none':
            borderStyle = wxNO_BORDER
        else:
            borderStyle = 0

        delegate = wxTextCtrl(
            aParent._getDelegate(), 
            self.getId(), 
            aResource.text, 
            wxPoint( aResource.position[ 0 ], aResource.position[ 1 ] ), 
            wxSize( aResource.size[ 0 ], aResource.size[ 1 ] ), 
            style = wxTE_PROCESS_ENTER | borderStyle | wxCLIP_SIBLINGS,
            name = aResource.name )

        if aResource.border == 'none':
            # the erase background event doesn't appear to make the control
            # transparent, so further investigation is required
            #EVT_ERASE_BACKGROUND(delegate, lambda evt: None)
            delegate.SetBackgroundColour(delegate.GetParent().GetBackgroundColour())
        return delegate

    # KEA the methods for retrieving and manipulating the text
    # has to be greatly expanded to match wxPython
    # capabilities or more

    def _setText( self, aString ) :
        """Sets the text value and marks the widget as not-modified.
        aString may contain newline characters if the text widget is multi-line."""
        self._getDelegate().SetValue( aString )

    def _getText( self ) :
        """Gets the contents of the widget. Notice that for a multiline text
        widget, the lines will be separated by (Unix-style) \\n characters,
        even under Windows where they are separated by a \\r\\n sequence in
        the native control."""
        return self._getDelegate().GetValue()

    def _getEditable( self ) :
        """Returns whether the text widget is editable or read-only."""
        # KEA the test below doesn't seem to be working, so...
        #return not self._getDelegate().GetWindowStyleFlag() & wxTE_READONLY
        return self._editable

    def _setEditable( self, aBoolean ) :
        """Makes the text widget editable or read-only."""
        self._editable = aBoolean
        self._getDelegate().SetEditable( aBoolean )

    # KEA new methods to mirror wxPython wxTextCtrl capabilities
    def appendText( self, aString ) :
        """Appends the text to the end of the text widget.
        After the text is appended, the insertion point will be at the end
        of the text widget. If this behavior is not desired, the programmer
        should use getInsertionPoint and setInsertionPoint."""
        self._getDelegate().AppendText( aString )

    def canCopy( self ) :
        return self._getDelegate().CanCopy()

    def canCut( self ) :
        return self._getDelegate().CanCut()

    def canPaste( self ) :
        return self._getDelegate().CanPaste()

    def canRedo( self ) :
        return self._getDelegate().CanRedo()

    def canUndo( self ) :
        return self._getDelegate().CanUndo()

    def clear( self ) :
        self._getDelegate().Clear()

    def copy( self ) :
        self._getDelegate().Copy()

    def cut( self ) :
        self._getDelegate().Cut()

    def discardEdits( self ) :
        self._getDelegate().DiscardEdits()
    
    def getInsertionPoint( self ) :
        return self._getDelegate().GetInsertionPoint()

    def getLastPosition( self ) :
        return self._getDelegate().GetLastPosition()

    def getLineLength( self, aLineNumber ) :
        return self._getDelegate().GetLineLength( aLineNumber )

    def getLineText( self, aLineNumber ) :
        return self._getDelegate().GetLineText( aLineNumber )

    def getNumberOfLines( self ) :
        return self._getDelegate().GetNumberOfLines()

    def getSelection( self ) :
        return self._getDelegate().GetSelection()

    def getNumberOfLines( self ) :
        return self._getDelegate().GetNumberOfLines()

    # KEA rename to getModified?
    def isModified( self ) :
        """Returns 1 if the text has been modified, 0 otherwise."""
        return self._getDelegate().IsModified()

    # KEA support LoadFile? If so, it only makes sense for TextArea
    # many of the other methods only make sense for the multiline TextArea
    # not TextField and PasswordField

    # KEA OnChar ties into our user code handlers and our events,
    # need to think about this one some more

    # KEA OnDropFiles is windows-specific, if you try and call it under *nix
    # what happens? just an exception?

    def paste( self ) :
        self._getDelegate().Paste()

    def positionToXY( self, aPosition ) :
        return self._getDelegate().PositionToXY( aPosition )

    def redo( self ) :
        self._getDelegate().Redo()

    def remove( self, aFrom, aTo ) :
        self._getDelegate().Remove( aFrom, aTo )

    def replace( self, aFrom, aTo, aString ) :
        # KEA workaround for Replace bug, has the side effect of
        # possibly changing the insertion point
        #self._getDelegate().Replace( aFrom, aTo, aString )
        i = self._getDelegate().GetInsertionPoint()
        self._getDelegate().Remove( aFrom, aTo )
        self._getDelegate().SetInsertionPoint( aFrom )
        self._getDelegate().WriteText( aString )
        self._getDelegate().SetInsertionPoint( i )

    def replaceSelection( self, aString ) :
        ins = self._getDelegate().GetInsertionPoint()
        sel = self._getDelegate().GetSelection()
        self._getDelegate().Remove( sel[0], sel[1] )
        #self._getDelegate().SetInsertionPoint( aFrom )
        self._getDelegate().WriteText( aString )
        self._getDelegate().SetSelection( ins, ins + len(aString))
        #self._getDelegate().SetInsertionPoint( i )

    # KEA support SaveFile?

    def setInsertionPoint( self, aPosition ) :
        self._getDelegate().SetInsertionPoint( aPosition )

    def setInsertionPointEnd( self ) :
        self._getDelegate().SetInsertionPointEnd()

    def setSelection( self, aFrom, aTo ) :
        self._getDelegate().SetSelection( aFrom, aTo )

    def showPostion( self, aPosition ) :
        self._getDelegate().ShowPostion( aPosition )

    def undo( self ) :
        self._getDelegate().Undo()

    def writeText( self, aString ) :
        self._getDelegate().WriteText( aString )

    def xyToPosition( self, aX, aY ) :
        return self._getDelegate().XYToPosition( aX, aY )

    def _getBorder( self ) :
        return self._border

    def _setBorder( self, aString ) :
        raise AttributeError, "border attribute is read-only"


class PasswordField( TextField ) :
    """
    A text field that displays '*' characters in place
    of the actual characters typed - suitable for entering
    passwords securely.
    """

    _events = [ KeyPressedEvent, 
                KeyDownEvent, 
                KeyUpEvent, 
                TextEnterEvent,
                TextUpdateEvent,
                SelectEvent ] + MOUSE_EVENTS

    def __init__( self, aParent,  aResource ) :
        TextField.__init__( self, aParent, aResource )

    def _createDelegate( self, aParent, aResource ) :
        if aResource.border == 'none':
            borderStyle = wxNO_BORDER
        else:
            borderStyle = 0

        delegate = wxTextCtrl(
            aParent._getDelegate(), 
            self.getId(), 
            aResource.text, 
            wxPoint( aResource.position[ 0 ], aResource.position[ 1 ] ), 
            wxSize( aResource.size[ 0 ], aResource.size[ 1 ] ), 
            style = wxTE_PASSWORD | wxTE_PROCESS_ENTER | borderStyle | wxCLIP_SIBLINGS,
            name = aResource.name )
        
        if aResource.border == 'none':
            delegate.SetBackgroundColour(delegate.GetParent().GetBackgroundColour())
        return delegate


class TextArea( TextField ) :
    """
    A text area that can have multi-line text, scrollbars, etc..
    """

    _events = [ KeyPressedEvent, 
                KeyDownEvent, 
                KeyUpEvent, 
                TextEnterEvent,
                TextUpdateEvent,
                SelectEvent ] + MOUSE_EVENTS

    def __init__( self, aParent,  aResource ) :
        TextField.__init__( self, aParent, aResource )

    def _createDelegate( self, aParent, aResource ) :
        if aResource.border == 'none':
            borderStyle = wxNO_BORDER
        else:
            borderStyle = 0

        delegate = wxTextCtrl(
            aParent._getDelegate(), 
            self.getId(), 
            aResource.text, 
            wxPoint( aResource.position[ 0 ], aResource.position[ 1 ] ), 
            wxSize( aResource.size[ 0 ], aResource.size[ 1 ] ),
            style =  wxTE_PROCESS_TAB | wxTE_MULTILINE | borderStyle | wxCLIP_SIBLINGS,
            name = aResource.name )

        if aResource.border == 'none':
            delegate.SetBackgroundColour(delegate.GetParent().GetBackgroundColour())
        return delegate


class Button( Widget ) :
    """
    A simple push-button with a label.
    """

    _events = [ MouseClickEvent ] + MOUSE_EVENTS

    def __init__( self, aParent,  aResource ) :
        Widget.__init__( self, aParent, aResource )
        """
        if aResource.isdefault:
            #print "setting default"
            self._setDefault()
        """

    def _createDelegate( self, aParent, aResource ) :
        return wxButton( aParent._getDelegate(), 
                         self.getId(), 
                         aResource.label, 
                         wxPoint( aResource.position[ 0 ], aResource.position[ 1 ] ), 
                         wxSize( aResource.size[ 0 ], aResource.size[ 1 ] ),
                         style = wxCLIP_SIBLINGS,
                         name = aResource.name 
        )

    # KEA based on the wxPython docs, I'm not sure we should support
    # setting the default, but let's put it in and see what issues come up
    # actually it doesn't appear to do anything on Windows anyway since
    # our Panel isn't a dialog, so I'll leave this in and comment out the setting
    # above and remove isdefault from spec.py
    def _setDefault( self ) :
        self._getDelegate().SetDefault()

    def _getLabel( self ) :
        return self._getDelegate().GetLabel()
    
    def _setLabel( self, aString ) :
        self._getDelegate().SetLabel( aString )

class CheckBox( Widget ) :
    """
    A check box.
    """

    _events = [ MouseClickEvent ] + MOUSE_EVENTS

    def __init__( self, aParent,  aResource ) :
        Widget.__init__( self, aParent, aResource)

        #self.setChecked( aResource.checked == 'true' )
        self._setChecked( aResource.checked)

    def _createDelegate( self, aParent, aResource ) :
        return wxCheckBox(  aParent._getDelegate(), 
                            self.getId(), 
                            aResource.label, 
                            wxPoint( aResource.position[ 0 ], aResource.position[ 1 ] ), 
                            wxSize( aResource.size[ 0 ], aResource.size[ 1 ] ),
                            style = wxCLIP_SIBLINGS,
                            name = aResource.name 
                           )

    def _getLabel( self ) :
        return self._getDelegate().GetLabel()
    
    def _setLabel( self, aString ) :
        self._getDelegate().SetLabel( aString )

    def _setChecked( self, aBoolean ) :
        self._getDelegate().SetValue( aBoolean )

    def _getChecked( self ) :
        return self._getDelegate().GetValue()

class StaticText( Widget ) :
    """
    An uneditable block of text.
    """

    _events = MOUSE_EVENTS

    def __init__( self, aParent, aResource ) :
        attributes = ['_alignment']
        self._createAttributes(attributes)
        Widget.__init__( self, aParent, aResource )
        self._alignment = aResource.alignment

    def _createDelegate( self, aParent, aResource ) :
        return wxStaticText( aParent._getDelegate(), 
            self.getId(), 
            aResource.text, 
            wxPoint( aResource.position[ 0 ], aResource.position[ 1 ] ), 
            wxSize( aResource.size[ 0 ], aResource.size[ 1 ] ),
            style = self.__getAlignment( aResource.alignment ) | wxCLIP_SIBLINGS,
            name = aResource.name 
        )

    def __getAlignment( self, aString  ) :
        if aString == 'left' :
            return wxALIGN_LEFT
        elif aString == 'center' :
            return wxALIGN_CENTER | wxST_NO_AUTORESIZE
        elif aString == 'right' :
            return wxALIGN_RIGHT | wxST_NO_AUTORESIZE
        else :
            raise 'invalid StaticText.alignment value: ', aString

    #def _bindEvents( self ) :
    #    pass

    #def _setHelpText( self, aString ) :
    #    pass

    def _getText( self ) :
        return self._getDelegate().GetLabel()

    def _setText( self, aString ) :
        self._getDelegate().SetLabel( aString )
        self._getDelegate().Refresh()

    def _getAlignment( self ) :
        return self._alignment

    def _setAlignment( self, aString ) :
        raise AttributeError, "alignment attribute is read-only"

class List( Widget ) :
    """
    A list that only allows a single item to be selected.
    """
    
    _events = [ SelectEvent, MouseDoubleClickEvent ] + MOUSE_EVENTS

    def __init__( self, aParent, aResource ) :
        Widget.__init__( self, aParent, aResource )
        if aResource.selected != "" and aResource.selected :
            self._setSelection( aResource.selected )
         
    def _createDelegate( self, aParent, aResource ) :
        return wxListBox( aParent._getDelegate(), 
                            self.getId(), 
                            wxPoint( aResource.position[ 0 ], aResource.position[ 1 ] ), 
                            wxSize( aResource.size[ 0 ], aResource.size[ 1 ] ), 
                            aResource.items,
                            style = wxLB_SINGLE | wxCLIP_SIBLINGS,
                            name = aResource.name )

    def _getItems( self ) :
        delegate = self._getDelegate()
        items = []
        for i in range( delegate.Number() ) :
            items.append( delegate.GetString( i ) )
        return items

    # KEA fixed setValue
    # there should be more flexibility than clearing the whole list
    # and replacing it with a new one
    def _setItems( self, aList ) :
        #self._getDelegate().setValue( aList )
        self._getDelegate().Clear()
        self._getDelegate().InsertItems( aList, 0 )

    # KEA this either needs to be renamed
    # so that we can set the selection via string and
    # position or handle it with an if statement in the existing method
    # the same goes for getSelection
    # wxPython has GetSelection/SetSelection for an integer position
    # and GetStringSelection/SetStringSelection for strings
    # I think we should go with those names
    def _setSelection( self, aString, select=1 ) :
        if type(aString) is types.IntType:
            self._getDelegate().SetSelection( aString, select ) # KEA should be an integer
        else:
            self._getDelegate().SetStringSelection( aString, select )

    # KEA this is redundant, may remove it later
    def setStringSelection( self, aString, select=1 ) :
        self._getDelegate().SetStringSelection( aString, select )
        
    def getSelectionIndex( self ) :
        return self._delegate.GetSelection()

    # KEA change in behavior to mimic wxPython
    def _getSelection( self ) :
        # Bug in wxPython? self._getDelegate().GetStringSelection()
        #i = self.getSelectionIndex()
        #return self._delegate.GetString( i )
        return self._delegate.GetSelection()

    def getSelections( self ) :
        return self._delegate.GetSelections()

    def getStringSelection( self ) :
        return self._getDelegate().GetStringSelection()

    # KEA more wxListBox methods

    def append( self, aString ) :
        self._getDelegate().Append( aString )

    def clear( self ) :
        self._getDelegate().Clear()

    def delete( self, aPosition ) :
        self._getDelegate().Delete( aPosition )

    def findString( self, aString ) :
        return self._getDelegate().FindString( aString )

    def getString( self, aPosition ) :
        return self._getDelegate().GetString( aPosition )

    def insertItems( self, aList, aPosition ) :
        self._getDelegate().InsertItems( aList, aPosition )

    # KEA this should probably have a different name
    def getNumber( self ) :
        return self._getDelegate().Number( )

    # KEA was getSelected
    def _isSelected( self, aPosition ) :
        """Determines whether an item is selected.
        aPosition is the zero-based item index
        Returns 1 if the given item is selected, 0 otherwise.
        """
        return self._getDelegate().Selected( )

    # KEA 2001-08-14
    # temporary fix
    # naming of selected, selection, stringSelection, etc. needs to be clarified
    # and standardized across List, RadioGroup, Choice
    # and then spec.py, debug.py, etc. needs to be updated

    def _getSelected( self ) :
        return self._getDelegate().GetStringSelection( )

    def _setSelected( self, aString ) :
        self.setStringSelection(aString)

    def setString( self, aString ) :
        self._getDelegate().SetString( aString )

class RadioGroup( Widget ) :
    """
    A group of radio buttons.
    """

    _events = [ SelectEvent ] + MOUSE_EVENTS

    def __init__( self, aParent, aResource ) :
        attributes = ['_labels', '_layout', '_max']
        self._createAttributes(attributes)
        Widget.__init__( self, aParent, aResource )
        self._labels = aResource.items
        self._layout = aResource.layout
        self._max = aResource.max
        self._setSelection( aResource.selected )
         
    def _createDelegate( self, aParent, aResource ) :
        return wxRadioBox( aParent._getDelegate(), 
                            self.getId(), 
                            aResource.label, 
                            wxPoint( aResource.position[ 0 ], aResource.position[ 1 ] ), 
                            wxSize( aResource.size[ 0 ], aResource.size[ 1 ] ), 
                            aResource.items,
                            aResource.max,
                            self.__getLayout( aResource.layout ) | wxCLIP_SIBLINGS,
                            name=aResource.name
                          )

    def __getLayout( self, aString  ) :
        if aString == 'horizontal' :
            return wxRA_SPECIFY_COLS
        elif aString == 'vertical' :
            return wxRA_SPECIFY_ROWS
        else :
            raise 'invalid RadioGroup.layout value: ', aString

    def _getItems( self ) :
        return self._labels

    def _setItems( self, aList ) :
        #self._getDelegate().SetClientData( aString )
        raise NotImplementedError

    # KEA added to match spec.py 'selected' attribute
    # plus get methods for other attributes
    def _getLabel( self ) :
        return self._getDelegate().GetLabel()

    def _setLabel( self, aString ) :
        self._getDelegate().SetLabel( aString )

    def _getLayout( self ) :
        return self._layout

    def _setLayout( self, aString ) :
        raise AttributeError, "layout attribute is read-only"

    def _getMax( self ) :
        return self._max

    def _setMax( self, aMax ) :
        raise AttributeError, "max attribute is read-only"

    # KEA 2001-08-14
    # temporary fix
    # naming of selected, selection, stringSelection, etc. needs to be clarified
    # and standardized across List, RadioGroup, Choice
    # and then spec.py, debug.py, etc. needs to be updated
    def _getSelected( self ) :
        return self._getDelegate().GetStringSelection()

    def _setSelected( self, aString ) :
        self._getDelegate().SetStringSelection( aString )

    def _getSelection( self ) :
        return self._getDelegate().GetStringSelection()

    def _setSelection( self, aString ) :
        self._getDelegate().SetStringSelection( aString )

class Choice( Widget ) :
    """
    A popup menu.
    """

    _events = [ SelectEvent ] + MOUSE_EVENTS

    def __init__( self, aParent, aResource ) :
        attributes = ['_items']
        self._createAttributes(attributes)
        Widget.__init__( self, aParent, aResource )
        # KEA 2001-08-12
        # need to fix this, these are supposed to be optional
        self._items = aResource.items
        self._setSelection( aResource.selected )
        
    def _createDelegate( self, aParent, aResource ) :
        return wxChoice( aParent._getDelegate(), 
            self.getId(), 
            wxPoint( aResource.position[ 0 ], aResource.position[ 1 ] ), 
            wxSize( aResource.size[ 0 ], aResource.size[ 1 ] ), 
            aResource.items,
            style =  wxCLIP_SIBLINGS,
            name = aResource.name 
        )
    
    def _getItems( self ) :
        return self._items

    def _setItems( self, items ) :
        self._getDelegate().Clear()
        for item in items :
            self._getDelegate().Append( item )

    def _getSelection( self ) :
        return self._getDelegate().GetStringSelection()

    def _setSelection( self, aString ) :
        self._getDelegate().SetStringSelection( aString )

    # KEA 2001-08-14
    # temporary fix
    # naming of selected, selection, stringSelection, etc. needs to be clarified
    # and standardized across List, RadioGroup, Choice
    # and then spec.py, debug.py, etc. needs to be updated
    def _getSelected( self ) :
        return self._getDelegate().GetStringSelection()

    def _setSelected( self, aString ) :
        self._getDelegate().SetStringSelection( aString )


class Slider( Widget ) :
    """
    A slider component.
    """

    _events = [ SelectEvent ] + MOUSE_EVENTS

    def __init__( self, aParent, aResource ) :
        Widget.__init__( self, aParent, aResource )
        attributes = ['_layout']
        self._createAttributes(attributes)
        self._layout = aResource.layout

    def _createDelegate( self, aParent, aResource ) :
        return wxSlider( aParent._getDelegate(), 
            self.getId(), 
            aResource.value, aResource.min, aResource.max,
            wxPoint( aResource.position[ 0 ], aResource.position[ 1 ] ), 
            wxSize( aResource.size[ 0 ], aResource.size[ 1 ] ), 
            style= self.__getLayout( aResource.layout ) | wxCLIP_SIBLINGS,
            name = aResource.name 
        )

    def __getLayout( self, aString  ) :
        if aString == 'horizontal' :
            return wxSL_HORIZONTAL
        elif aString == 'vertical' :
            return wxSL_VERTICAL
        else :
            raise 'invalid Slider.layout value: ', aString
            
    def setRange( self, aMin, aMax ) :
        self._getDelegate().SetRange( aMin, aMax )

    def _getValue( self ) :
        return self._getDelegate().GetValue()

    def _setValue( self, aValue ) :
        self._getDelegate().SetValue( aValue )

    def _getMin( self ) :
        return self._getDelegate().GetMin()

    def _setMin( self, aMin ) :
        self._getDelegate().SetRange( aMin, self._getDelegate().GetMax() )

    def _getMax( self ) :
        return self._getDelegate().GetMax()

    def _setMax( self, aMax ) :
        self._getDelegate().SetRange( self._getDelegate().GetMin(), aMax )

    def _getLayout( self ) :
        return self._layout

    def _setLayout( self, aString ) :
        raise AttributeError, "layout attribute is read-only"


class Image( Widget ) :
    """
    A static image.
    """

    _events = MOUSE_EVENTS

    def __init__( self, aParent, aResource ) :
        attributes = ['_bitmap', '_file']
        self._createAttributes(attributes)
        self._bitmap = Bitmap(aResource.file, aResource.size)
        self._file = aResource.file
        Widget.__init__( self, aParent, aResource )

    def _createDelegate( self, aParent, aResource ) :
        # KEA
        # should we modify aResource instead?
        # is aResource used again later?
        #print aResource.size
        x = aResource.size[0]
        if x == -2:
            x = self._bitmap.getWidth()
        y = aResource.size[1]
        if y == -2:
            y = self._bitmap.getHeight()
        size = wxSize( x, y )
        #size = wxSize( self._bitmap.GetWidth(), self._bitmap.GetHeight() )
        """
        if aResource.border == 'transparent':
            mask = wxMaskColour(self._bitmap, wxBLACK)
            self._bitmap.SetMask(mask)
        """
        delegate = wxStaticBitmap( aParent._getDelegate(), 
            self.getId(), # was -1
            self._bitmap.getBits(), 
            aResource.position, 
            size,
            style = wxCLIP_SIBLINGS,
            name = aResource.name                       
        )
        # KEA 2001-07-27
        # set transparency, same issues as ImageButton, so code above
        # needs to be changed
        #print aResource.border
        #print aResource
        """
        if aResource.border == 'transparent':
            print aResource.border
            EVT_ERASE_BACKGROUND( delegate, lambda evt: None) #self.bogusErase)
        """
        return delegate

    # KEA added getBitmap, setBitmap
    def _getBitmap( self ) :
        return self._bitmap

    def _setBitmap( self, aValue ) :
        self._bitmap = aValue
        self._getDelegate().SetBitmap( aValue._getBits() )
        # may actually need to refresh the panel as well
        # depending on the size of the new bitmap compared
        # to the old one
        # in addition, the size of the image or imagebutton needs
        # to be set appropriately after changing the bitmap so
        # there are still a few issues to work out
        self._getDelegate().Refresh()

    # KEA do we query the Bitmap to find the actual dimensions
    # _size can contain -1, and -2
    # or provide a special getBitmapSize method?
    # this getSize is actually the same as its parent
    def _getSize( self ) :
        return self._size

    # KEA special handling for -2 size option
    def _setSize( self, aSize ):
        x = aSize[0]
        if x == -2:
            x = self._bitmap.getWidth()
        y = aSize[1]
        if y == -2:
            y = self._bitmap.getHeight()
        self._delegate.SetSize( ( x, y ) )
        self._size = self._delegate.GetSize()   # get the actual size, not (-1, -1)

    # KEA 2001-08-02
    # right now the image is loaded from a filename
    # during initialization
    # but later these might not make any sense
    # if setBitmap is used directly in user code
    def _getFile( self ) :
        return self._file

    # KEA 2001-08-14
    # if we're going to support setting the file
    # after initialization, then this will need to handle the bitmap loading
    # overhead
    def _setFile( self, aFile ) :
        self._file = aFile
        self._setBitmap(Bitmap(aFile))


class ImageButton( Image ) :
    """
    An image that plays the role of a button.
    """

    _events = [ MouseClickEvent ] + MOUSE_EVENTS

    def __init__( self, aParent, aResource ) :
        attributes = ['_border']
        self._createAttributes(attributes)
        self._border = aResource.border
        Image.__init__(self, aParent, aResource )
         
    def _createDelegate( self, aParent, aResource ) :
        # KEA
        # should we modify aResource instead?
        # is aResource used again later?
        #print aResource.size
        x = aResource.size[0]
        if x == -2:
            x = self._bitmap.getWidth()
        y = aResource.size[1]
        if y == -2:
            y = self._bitmap.getHeight()
        size = wxSize( x, y )

        # KEA need to check all possible variations on Win32 and Linux
        if aResource.border == '3d':
            style = wxBU_AUTODRAW   # Windows specific ?!
        else:
            style = 0
        
        delegate = wxBitmapButton( aParent._getDelegate(), 
            self.getId(), 
            self._bitmap.getBits(), 
            aResource.position, 
            size,
            style | wxCLIP_SIBLINGS,
            name = aResource.name 
        )
        # KEA 2001-07-27
        # we should only be binding this if the button
        # is supposed to be transparent
        # so maybe a setTransparent method with true/false parameter?
        # keep track of _transparent attribute and if _transparent
        # then unbind EVT_ERASE_BACKGROUND
        # before changing _transparent to false
        # same kind of thing for setTransparent
        # don't rebind events if not needed
        # getTransparent as well
        # setBorder/getBorder
        #print aResource.border
        if aResource.border == 'transparent':
            #print aResource.border
            EVT_ERASE_BACKGROUND( delegate, lambda evt: None)
        return delegate

    #def _setHelpText( self, aString ) :
    #    pass

    # KEA added getBitmap, setBitmap
    # KEA wxWindows uses a different set for wxStaticBitmap
    # so we have to override
    def _getBitmap( self ) :
        return self._bitmap

    # KEA
    # should something other than Refresh be used to force
    # a screen update?
    def _setBitmap( self, aValue ) :
        self._bitmap = aValue
        self._getDelegate().SetBitmapLabel( aValue.getBits() )
        self._getDelegate().Refresh()

    def _getBorder( self ) :
        return self._border

    def _setBorder( self, aString ) :
        raise AttributeError, "border attribute is read-only"


class Bitmap :

    def __init__( self, fileName, size=(-1, -1) ) :
        self._fileName = fileName
        self._type = self._getBitmapType( fileName )
        if self._type == None:
            self._bits = wxEmptyBitmap(size[0], size[1], 2)
        else:
            self._bits = wxBitmap( self._fileName, self._type )

    def getBits( self ) :
        return self._bits

    def setBits( self ) :
        return self._bits

    def getHeight( self ) :
        return self._bits.GetHeight()

    def getWidth( self ) :
        return self._bits.GetWidth()

    def getSize( self ) :
        return (self._bits.GetWidth(), self._bits.GetHeight())

    # KEA special handling for -2 size option
    def setSize( self, aSize ):
        raise NotImplementedError

    def getType( self ) :
        return  self._type

    def _getBitmapType( self, filename ) :
        """
        Get the type of an image from the file's extension ( .jpg, etc. )
        """
        # KEA 2001-07-27
        # was
        #name, ext = filename.split( '.' )
        #ext = ext.upper()
        if filename == '':
            return None

        name, ext = os.path.splitext(filename)
        ext = ext[1:].upper()
        if ext == 'JPG' or ext == 'JPEG':
            return wxBITMAP_TYPE_JPEG
        elif ext == 'GIF':
            return wxBITMAP_TYPE_GIF
        elif ext == 'BMP':
            return wxBITMAP_TYPE_BMP
        elif ext == 'PNG':
            return wxBITMAP_TYPE_PNG
        elif ext == 'XBM':
            return wxBITMAP_TYPE_XBM
        else:
            raise 'invalid graphics format'    # should throw an exception here

class StaticLine( Widget ) :
    """
    A static line is just a line which may be used to separate
    a groups of controls. The line may be only vertical or horizontal.
    """

    _events = MOUSE_EVENTS

    def __init__( self, aParent, aResource ) :
        Widget.__init__( self, aParent, aResource )
        attributes = ['_layout']
        self._createAttributes(attributes)
        # set attributes directly that can only be set at initialization
        self._layout = aResource.layout

    def _createDelegate( self, aParent, aResource ) :
        return wxStaticLine( aParent._getDelegate(), 
            self.getId(), 
            wxPoint( aResource.position[ 0 ], aResource.position[ 1 ] ), 
            wxSize( aResource.size[ 0 ], aResource.size[ 1 ] ),
            style = self.__getLayout( aResource.layout ) | wxCLIP_SIBLINGS,
            name = aResource.name 
        )

    def __getLayout( self, aString  ) :
        if aString == 'horizontal' :
            return wxLI_HORIZONTAL
        elif aString == 'vertical' :
            return wxLI_VERTICAL
        else :
            raise 'invalid StaticLine.layout value: ', aString

    #def _bindEvents( self ) :
    #    pass

    #def _setHelpText( self, aString ) :
    #    pass

    def _setLayout( self, aString ) :
        raise AttributeError, "layout attribute is read-only"

    def _getLayout( self ) :
        return self._layout

class WidgetFactory :
    """
    An abstract factory for creating Widgets from a Resource description.
    """
        
    def createWidget( self, aScriptable, aParent, aResource ) :

            # Find the class object based on a component's Resource definition.
            
        # KEA 2001-08-05 is there a better way of coming up with 
        # the module name for model as the package name changes?
        aPackage, aModule = __name__.split('.')
        clazz = sys.modules[ aPackage + '.model' ].__dict__[ aResource.__dict__[ 'type' ] ]
        #clazz = sys.modules[ 'model' ].__dict__[ aResource.__dict__[ 'type' ] ]
        
            # Construct a new Component.

        component = clazz( aParent, aResource )

            # Bind the Component to an EventDispatch that will translate
            # events generated by Component into calls to handlers that
            # are defined in a Scriptable object.

        EventDispatch( component, aScriptable )
    
            # Return the newly created Component

        return component

