"""
__version__ = "$Revision: 1.4 $"
__date__ = "$Date: 2001/08/20 22:32:38 $"
"""

from wxPython.wx import *
from binding import *

class wxPython_EventBinding ( EventBindingAdapter ) :
    """
    A superclass specific to the wxPython cross-platform GUI toolkit.
    """
    def __init__( self, aComponent ) :
        EventBindingAdapter.__init__( self, aComponent )
        """
        There are a number of mouse specific events that are
        supported by most Widgets, so we bind them here instead
        of requiring any subclasses to do the work.
        """
        self.wxRegistry = { GainFocusEvent : ( EVT_SET_FOCUS, wxEVT_SET_FOCUS ),
                            LoseFocusEvent : ( EVT_KILL_FOCUS, wxEVT_KILL_FOCUS ),
                            MouseDownEvent : ( EVT_LEFT_DOWN, wxEVT_LEFT_DOWN ),
                            MouseUpEvent : ( EVT_LEFT_UP, wxEVT_LEFT_UP ),      
                            MouseDoubleClickEvent : ( EVT_LEFT_DCLICK, wxEVT_LEFT_DCLICK ),   
                            MouseMiddleDownEvent : ( EVT_MIDDLE_DOWN, wxEVT_MIDDLE_DOWN ),
                            MouseMiddleUpEvent : ( EVT_MIDDLE_UP, wxEVT_MIDDLE_UP ),
                            MouseContextDownEvent : ( EVT_RIGHT_DOWN, wxEVT_RIGHT_DOWN ),
                            MouseContextUpEvent : ( EVT_RIGHT_UP, wxEVT_RIGHT_UP ),
                            MouseMoveEvent : ( EVT_MOTION, wxEVT_MOTION ),
                            MouseMiddleDoubleClickEvent : ( EVT_MIDDLE_DCLICK, wxEVT_MIDDLE_DCLICK ), 
                            MouseContextDoubleClickEvent : ( EVT_RIGHT_DCLICK, wxEVT_RIGHT_DCLICK ),  
                            MouseLeaveEvent : ( EVT_LEAVE_WINDOW, wxEVT_LEAVE_WINDOW ),  
                            MouseEnterEvent : ( EVT_ENTER_WINDOW, wxEVT_ENTER_WINDOW ) }

        self.wxEventIdMap = {}

        for key in self.wxRegistry.keys() :
            value = self.wxRegistry[ key ]
            self.wxEventIdMap[ value[ 1 ] ] = key

    def bindEvents( self ) :
        classes = ComponentInspector( self._component ).getEventClasses()

        # The _bindMouseEvents method makes a copy of the classes list,
        # removes each mouse event that is found in the copy then returns 
        # the copy.  We use the returned copy that is stripped of mouse
        # events as they are all handled in this class and subclasses
        # don't need to see them.

        remainingClasses = self._bindMouseEvents( classes )
        
        # Bind all of the Events supported by the component.

        for eventClass in remainingClasses :
            self.bindEvent( eventClass )

    def bindEvent( self, aEventClass ) :
        raise AbstractMethodError

    def _bindMouseEvents( self, aEventClassList ) :
                
        # Make a copy of the event class list so we can
        # strip out the mouse event classes.

        widgetClasses = []
        widgetClasses = widgetClasses + aEventClassList

        parent = self._component._getParent()

        id = self._component.getId()

        otherClasses = []

        for widgetClass in widgetClasses :
            index = self._findEventClass( widgetClass, MOUSE_EVENTS ) 
            if index >= 0 :
                wxFunction = self.wxRegistry[ widgetClass ][ 0 ]
                wxFunction( self._component._delegate, self._dispatch )
            else :
                otherClasses.append( widgetClass )

        return otherClasses

    def _findEventClass( self, aClass, aArray ) :
        i = 0
        for e in aArray :
            if e == aClass :
                return i
            i += 1
        return -1

    """
    Subclasses MUST call wxPython_EventBinding._dispatch( self, aWxEvent )
    in order for the generic mouse events to get processed.

    The _dispatch method must return true if it handled the
     event and false if it didn't.
    """
    def _dispatch( self, aWxEvent ) :

        eventType = aWxEvent.GetEventType()

        if self.wxEventIdMap.has_key( eventType ) :
            event = self.wxEventIdMap[ eventType ]( self._component )
            if event.__class__ is MouseMoveEvent :
                if aWxEvent.Dragging() :
                    event = MouseDragEvent( self._component )
            event._nativeEvent = aWxEvent
            self._component.notifyEventListeners( event )
            aWxEvent.Skip()
            return 1
        else :
            return 0
        

    def _createEvent( self, aEventClass, aWxEvent ) :
        event = aEventClass( self._component )
        event.setNativeEvent( aWxEvent )
        return event

class wxPython_ButtonEventBinding( wxPython_EventBinding ) :
    """
    Bind the Events supported by event.Button to wxPython.
    """
    def __init__( self, aComponent ) :
        wxPython_EventBinding.__init__( self, aComponent )

    def bindEvent( self, aEventClass ) :
        parent = self._component._getParent()._delegate

        if aEventClass is MouseClickEvent :
            EVT_BUTTON( parent, self._component.getId(), self._dispatch )

    def _dispatch( self, aWxEvent ) :

        component = self._component 

        # Call our superclass to dispatch the standard mouse
        # events that every widget should get.
        if wxPython_EventBinding._dispatch( self, aWxEvent ) :
            return

        event = None

        if aWxEvent.GetEventType() == wxEVT_COMMAND_BUTTON_CLICKED :
            if component._getCommand() is not None :
                event = CommandEvent( component._getCommand(), component )
            else :
                event = self._createEvent( MouseClickEvent, aWxEvent )

        if event is not None :
            component.notifyEventListeners( event )


class wxPython_CheckBoxEventBinding( wxPython_EventBinding ) :
    """
    Bind the Events supported by event.CheckBox to wxPython.
    """
    def __init__( self, aComponent ) :
        wxPython_EventBinding.__init__( self, aComponent )

    def bindEvent( self, aEventClass ) :
        parent = self._component._getParent()._delegate

        if aEventClass is MouseClickEvent :
            EVT_CHECKBOX( parent, self._component.getId(), self._dispatch )

    def _dispatch( self, aWxEvent ) :

        # Call our superclass to dispatch the standard mouse
        # events that every widget should get.
        if wxPython_EventBinding._dispatch( self, aWxEvent ) :
            return
        
        event = None

        if aWxEvent.GetEventType() == wxEVT_COMMAND_CHECKBOX_CLICKED :
            event = self._createEvent( MouseClickEvent, aWxEvent )

        if event is not None :
            self._component.notifyEventListeners( event )

class wxPython_ChoiceEventBinding( wxPython_EventBinding ) :
    """
    Bind the Events supported by event.Choice to wxPython.
    """
    def __init__( self, aComponent ) :
        wxPython_EventBinding.__init__( self, aComponent )

    def bindEvent( self, aEventClass ) :
        parent = self._component._getParent()._delegate

        if aEventClass is SelectEvent :
            EVT_CHOICE( parent, self._component.getId(), self._dispatch )

    def _dispatch( self, aWxEvent ) :
        # Call our superclass to dispatch the standard mouse
        # events that every widget should get.
        if wxPython_EventBinding._dispatch( self, aWxEvent ) :
            return

        event = None

        if aWxEvent.GetEventType() == wxEVT_COMMAND_CHOICE_SELECTED :
            event = self._createEvent( SelectEvent, aWxEvent )

        if event is not None :
            self._component.notifyEventListeners( event )

class wxPython_TextFieldEventBinding( wxPython_EventBinding ) :
    """
    Bind the Events supported by event.TextField to wxPython.
    """
    def __init__( self, aComponent ) :
        wxPython_EventBinding.__init__( self, aComponent )

    def bindEvent( self, aEventClass ) :

        parent = self._component._getParent()._delegate

        if aEventClass is TextUpdateEvent :
            EVT_TEXT( parent, self._component.getId(), self._dispatch )

        if aEventClass is TextEnterEvent :
            EVT_TEXT_ENTER( parent, self._component.getId(), self._dispatch )
        
    def _dispatch( self, aWxEvent ) :
        # Call our superclass to dispatch the standard mouse
        # events that every widget should get.
        if wxPython_EventBinding._dispatch( self, aWxEvent ) :
            return
        
        event = None

        if aWxEvent.GetEventType() == wxEVT_COMMAND_TEXT_UPDATED :
            event = self._createEvent( TextUpdateEvent, aWxEvent )

        if aWxEvent.GetEventType() == wxEVT_COMMAND_TEXT_ENTER :
            event = self._createEvent( TextEnterEvent, aWxEvent )
            self._component.notifyEventListeners( CloseField( self._component ) )

        if event is not None :
            self._component.notifyEventListeners( event )

class wxPython_TextAreaEventBinding( wxPython_EventBinding ) :
    """
    Bind the Events supported by event.TextArea to wxPython.
    """
    def __init__( self, aComponent ) :
        wxPython_EventBinding.__init__( self, aComponent )

    def bindEvent( self, aEventClass ) :

        parent = self._component._getParent()._delegate

        if aEventClass is TextUpdateEvent :
            EVT_TEXT( parent, self._component.getId(), self._dispatch )

        if aEventClass is TextEnterEvent :
            EVT_TEXT_ENTER( parent, self._component.getId(), self._dispatch )
        
        #if aEventClass is KeyDownEvent :
        #    EVT_KEY_DOWN( self._component._delegate, self._dispatch )

        #if aEventClass is KeyUpEvent :
        #    EVT_KEY_UP( parent, self._dispatch )

    def _dispatch( self, aWxEvent ) :
        # Call our superclass to dispatch the standard mouse
        # events that every widget should get.
        if wxPython_EventBinding._dispatch( self, aWxEvent ) :
            return
        
        event = None

        component = self._component

        if aWxEvent.GetEventType() == wxEVT_COMMAND_TEXT_UPDATED :
            event = TextUpdateEvent( component )

        if aWxEvent.GetEventType() == wxEVT_COMMAND_TEXT_ENTER :
            event = self._createEvent( TextEnterEvent, aWxEvent )
        
        #if aWxEvent.GetEventType() is wxEVT_KEY_DOWN :
        #    event = self._createEvent( KeyDownEvent, aWxEvent )

        #if aWxEvent.GetEventType() is wxEVT_KEY_UP :
        #    event = self._createEvent( KeyUpEvent, aWxEvent )

        if event is not None :
            component.notifyEventListeners( event )


class wxPython_PasswordFieldEventBinding( wxPython_TextFieldEventBinding ) :
    """
    Bind the Events supported by event.PasswordField to wxPython.
    """
    def __init__( self, aComponent ) :
        wxPython_TextFieldEventBinding.__init__( self, aComponent )


class wxPython_ImageButtonEventBinding( wxPython_ButtonEventBinding ) :
    """
    Bind the Events supported by event.ImageButton to wxPython.
    """
    def __init__( self, aComponent ) :
        wxPython_ButtonEventBinding.__init__( self, aComponent )

    def bindEvent( self, aEventClass ) :
        parent = self._component._getParent()._delegate

        if aEventClass is MouseClickEvent :
            EVT_BUTTON( parent, self._component.getId(), self._dispatch )


class wxPython_ImageEventBinding( wxPython_EventBinding ) :
    """
    Bind the Events supported by event.Image to wxPython.
    """
    def __init__( self, aComponent ) :
        wxPython_EventBinding.__init__( self, aComponent )

    def bindEvent( self, aEventClass ) :
        parent = self._component._getParent()._delegate


class wxPython_ListEventBinding( wxPython_EventBinding ) :
    """
    Bind the Events supported by event.List to wxPython.
    """
    def __init__( self, aComponent ) :
        wxPython_EventBinding.__init__( self, aComponent )

    """
    Bind an Event class that is supported by this objects peer Widget.
    """
    def bindEvent( self, aEventClass ) :
        parent = self._component._getParent()._delegate
        id = self._component.getId()
        if aEventClass is SelectEvent :
            EVT_LISTBOX( parent, id, self._dispatch )

        if aEventClass is MouseDoubleClickEvent :
            EVT_LISTBOX_DCLICK( parent, id, self._dispatch )
    """
    Intercept all wxPyton events, convert them to PythonCard events, and
    post them to the EventQueue.
    """
    def _dispatch( self, aWxEvent ) :
        # Call our superclass to dispatch the standard mouse
        # events that every widget should get.
        if wxPython_EventBinding._dispatch( self, aWxEvent ) :
            return

        event = None 

        if aWxEvent.GetEventType() == wxEVT_COMMAND_LISTBOX_SELECTED :
            event = self._createEvent( SelectEvent, aWxEvent )

        elif aWxEvent.GetEventType() == wxEVT_COMMAND_LISTBOX_DOUBLECLICKED :
            event = self._createEvent( MouseDoubleClickEvent, aWxEvent )

        if event is not None :
            self._component.notifyEventListeners( event )

class wxPython_RadioGroupEventBinding( wxPython_EventBinding ) :
    """
    Bind the Events supported by event.RadioGroup to wxPython.
    """
    def __init__( self, aComponent ) :
        wxPython_EventBinding.__init__( self, aComponent )

    def bindEvent( self, aEventClass ) :
        parent = self._component._getParent()._delegate

        if aEventClass is SelectEvent :
            EVT_RADIOBOX( parent, self._component.getId(), self._dispatch )

    def _dispatch( self, aWxEvent ) :
        # Call our superclass to dispatch the standard mouse
        # events that every widget should get.
        if wxPython_EventBinding._dispatch( self, aWxEvent ) :
            return

        event = None

        if aWxEvent.GetEventType() == wxEVT_COMMAND_RADIOBOX_SELECTED :
            event = self._createEvent( SelectEvent, aWxEvent )

        if event is not None :
            self._component.notifyEventListeners( event )

class wxPython_SliderEventBinding( wxPython_EventBinding ) :
    """
    Bind the Events supported by event.Slider to wxPython.
    """
    def __init__( self, aComponent ) :
        wxPython_EventBinding.__init__( self, aComponent )

                        
    def bindEvent( self, aEventClass ) :
        parent = self._component._getParent()._delegate

        if aEventClass is SelectEvent :
            EVT_SLIDER( parent, self._component.getId(), self._dispatch )

    def _dispatch( self, aWxEvent ) :
        # Call our superclass to dispatch the standard mouse
        # events that every widget should get.
        if wxPython_EventBinding._dispatch( self, aWxEvent ) :
            return

        event = None

        if aWxEvent.GetEventType() == wxEVT_COMMAND_SLIDER_UPDATED :
            event = self._createEvent( SelectEvent, aWxEvent )

        if event is not None :
            self._component.notifyEventListeners( event )


class wxPython_StaticTextEventBinding( wxPython_EventBinding ) :
    """
    Bind the Events supported by event.StaticText to wxPython.
    """
    def __init__( self, aComponent ) :
        wxPython_EventBinding.__init__( self, aComponent )

    def bindEvent( self, aEventClass ) :
        parent = self._component._getParent()._delegate


class wxPython_StaticLineEventBinding( wxPython_EventBinding ) :
    """
    Bind the Events supported by event.StaticLine to wxPython.
    """
    def __init__( self, aComponent ) :
        wxPython_EventBinding.__init__( self, aComponent )

    def bindEvent( self, aEventClass ) :
        parent = self._component._getParent()._delegate


